#!ruby -Ku
#
#  TOPPERS Configrator by Ruby
#
#  Copyright (C) 2015 by Embedded and Real-Time Systems Laboratory
#              Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2015 by FUJI SOFT INCORPORATED, JAPAN
#
#  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
#  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
#  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
#  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
#      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
#      スコード中に含まれていること．
#  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
#      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
#      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
#      の無保証規定を掲載すること．
#  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
#      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
#      と．
#    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
#        作権表示，この利用条件および下記の無保証規定を掲載すること．
#    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
#        報告すること．
#  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
#      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
#      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
#      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
#      免責すること．
#
#  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
#  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
#  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
#  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
#  の責任を負わない．
#
#  $Id: cfg.rb 809 2017-03-08 12:00:54Z suzuki-kawaguchi $
#

if ($0 == __FILE__)
  TOOL_ROOT = File.expand_path(File.dirname(__FILE__)) + "/"
  $LOAD_PATH.unshift(TOOL_ROOT)
end

require "pp"
require "csv"
require "optparse"
require "common/GenFile.rb"
require "common/StrVal.rb"
require "common/SRecord.rb"
require "common/functions.rb"

######################################################################
# 定数定義
######################################################################
# 共通
VERSION = "0.1.0"

# cfg1_out関係
CFG1_PREFIX         = "TOPPERS_cfg_"
CFG1_SIGNED         = "signed_t"
CFG1_UNSIGNED       = "unsigned_t"
CFG1_ADDRESS        = "volatile void* const"
CFG1_SREC           = "cfg1_out.srec"
CFG1_SYMS           = "cfg1_out.syms"
CFG1_MAGIC_NUM      = "magic_number"
CFG1_SIZEOF_SIGNED  = "sizeof_signed_t"
CFG1_SIZEOF_POINTER = "sizeof_pointer"


######################################################################
# エラー発生有無フラグ
######################################################################
$bErrorFlg = false


######################################################################
# pass3のtrb用関数
######################################################################
def SYMBOL(sSymbol)
  if ($hSymbolAddress.has_key?(sSymbol))
    return $hSymbolAddress[sSymbol]
  else
    return nil
  end
end

def BCOPY(nStartIData, nStartData, nSize)
  sCopyData = $cRomImage.get_data(nStartIData, nSize)
  $cRomImage.set_data(nStartData, sCopyData)
end

def PEEK(nAddress, nSize)
  return $cRomImage.get_value(nAddress, nSize, false)
end


######################################################################
# 指定したパラメータと数値が合致する静的API情報を返す(ハッシュ版)
######################################################################
def GetCfgDataHash(hData, lParam, nValue)
  hCfg = nil
  hData.each{|cID, hParam|
    if (hParam[lParam].val == nValue)
      hCfg = hParam.dup()
      break
    end
  }
  return hCfg
end


######################################################################
# 指定したパラメータと数値が合致する静的API情報を返す(配列版)
######################################################################
def GetCfgDataArray(aData, lParam, nValue)
  hCfg = nil
  aData.each{|hParam|
    if (hParam[lParam].val == nValue)
      hCfg = hParam.dup()
      break
    end
  }
  return hCfg
end


######################################################################
# 指定したパラメータでソートした静的API情報を返す
######################################################################
def GetSortDataArray(aData, lParam)
  aCfg = []
  aSortKey = []
  aData.each{|hParam|
    aSortKey.push(hParam[lParam].val)
  }
  aSortKey.uniq!()
  aSortKey.sort!()
  aSortKey.each{|nVal|
    aData.each{|hParam|
      if (hParam[lParam].val == nVal)
        aCfg.push(hParam.dup())
      end
    }
  }
  return aCfg
end


######################################################################
# 各エラー/警告表示関数
######################################################################
def warning(hParam, sMessage)
  STDERR.puts("cfg:#{hParam[:_file_]}:#{hParam[:_line_]}: warning: #{sMessage}")
end
def error_exit(sMessage)
  abort("cfg: error: #{sMessage}")
end
def error(sMessage)
  STDERR.puts("cfg: error: #{sMessage}")
  $bErrorFlg = true
end
def error2(sFileLine, sMessage)
  STDERR.puts("cfg:#{sFileLine}: error: #{sMessage}")
  $bErrorFlg = true
end
def error_common(sErrCode, hParam, sMessage)
  error2("#{hParam[:_file_]}:#{hParam[:_line_]}", "#{sErrCode}: #{sMessage}")
end
def error_illegal3(sErrCode, hParam, sName, sValue, sApi)
  error_common(sErrCode, hParam, "illegal #{sName} `#{sValue}' in #{sApi}")
end
def error_illegal4(sErrCode, hParam, sName, sValue, sID, sApi)
  error_common(sErrCode, hParam, "illegal #{sName} `#{sValue}' of `#{sID}' in #{sApi}")
end
def error_illegal5(sErrCode, hParam, sName1, sValue, sName2, sID, sApi)
  error_common(sErrCode, hParam, "illegal #{sName1} `#{sValue}' of #{sName2} `#{sID}' in #{sApi}")
end
def error_is_xxx3(sErrCode, hParam, sName, sValue, sApi, sXxx)
  error_common(sErrCode, hParam, "#{sName} `#{sValue}' in #{sApi} is #{sXxx}")
end
def error_is_xxx4(sErrCode, hParam, sName, sValue, sID, sApi, sXxx)
  error_common(sErrCode, hParam, "#{sName} `#{sValue}' of `#{sID}' in #{sApi} is #{sXxx}")
end


######################################################################
# PASS 1
######################################################################
def Pass1()

  #
  # cfg1_out.c ソースコード作成開始
  #
  cCfg1Out = GenFile.new("cfg1_out.c")

  # 固定コード1
  cCfg1Out.add("/* cfg1_out.c */")
  cCfg1Out.add("#define TOPPERS_CFG1_OUT 1")
  cCfg1Out.add("#include \"kernel/kernel_int.h\"")

  # インクルードヘッダファイル
  $aIncludeFiles.each{|sFile|
    cCfg1Out.add("#include \"#{sFile}\"")
  }

  # 固定コード2
  sTemp = <<-EOS

#ifdef INT64_MAX
  typedef int64_t signed_t;
  typedef uint64_t unsigned_t;
#else
  typedef int32_t signed_t;
  typedef uint32_t unsigned_t;
#endif

#include "target_cfg1_out.h"

const uint32_t TOPPERS_cfg_magic_number = 0x12345678;
const uint32_t TOPPERS_cfg_sizeof_signed_t = sizeof(signed_t);
const uint32_t TOPPERS_cfg_sizeof_pointer = sizeof(const volatile void*);
const unsigned_t TOPPERS_cfg_CHAR_BIT = ((unsigned char)~0u == 0xff ? 8 : 16);
const unsigned_t TOPPERS_cfg_CHAR_MAX = ((char)-1 < 0 ? (char)((unsigned char)~0u >> 1) : (unsigned char)~0u);
const unsigned_t TOPPERS_cfg_CHAR_MIN = (unsigned_t)((char)-1 < 0 ? -((unsigned char)~0u >> 1) - 1 : 0);
const unsigned_t TOPPERS_cfg_SCHAR_MAX = (signed char)((unsigned char)~0u >> 1);
const unsigned_t TOPPERS_cfg_SHRT_MAX = (short)((unsigned short)~0u >> 1);
const unsigned_t TOPPERS_cfg_INT_MAX = (int)(~0u >> 1);
const unsigned_t TOPPERS_cfg_LONG_MAX = (long)(~0ul >> 1);

  EOS

  cCfg1Out.add(sTemp)


  # 値取得シンボル
  $hSymbolTable.each{|sName, hData|
    # プリプロセッサ命令(#)
    if ((hData[:EXPR][0, 1] == "#") || !hData[:TRUE].nil? || !hData[:FALSE].nil?)
      # 先頭が#の場合除去
      if (hData[:EXPR][0, 1] == "#")
        hData[:EXPR].slice!(0, 1)
      end
      # 真偽値設定
      if (hData[:TRUE].nil?)
        hData[:TRUE] = "1"
      end
      if (hData[:FALSE].nil?)
        hData[:FALSE] = "0"
      end
      cCfg1Out.add("const #{hData[:SIGNED]} #{CFG1_PREFIX}#{sName} =")
      cCfg1Out.add("#if #{hData[:EXPR]}#{NL}(#{hData[:TRUE]});")
      cCfg1Out.add("#else#{NL}(#{hData[:FALSE]});#{NL}#endif")
    # アドレス(@)
    elsif (hData[:EXPR][0, 1] == "@")
      # 先頭のaを除去
      cCfg1Out.add("const #{CFG1_ADDRESS} = (#{hData[:EXPR].slice!(0, 1)});")
    # 値のみ
    else
      cCfg1Out.add("const #{hData[:SIGNED]} #{CFG1_PREFIX}#{sName} = ( #{hData[:SIGNED]} )#{hData[:EXPR]};")
    end
  }


  # 静的APIを順に処理する
  $aCfgFileData.each{|aLine|
    # インクルードヘッダファイルのコメント
    if (aLine[0] =~ /^#include\s+".+"/)
      cCfg1Out.add2("#{NL}/* #{aLine[0]} */")
    # #include以外のプリプロセッサディレクティブ
    elsif ((aLine[0] =~ /^#if/) || (aLine[0] =~ /^#e/) || (aLine[0] =~ /^#pragma/))
      cCfg1Out.add2(aLine[0])
    # 静的API宣言
    elsif (aLine[0] =~ /[A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z]\(.+\);/)
      # 静的API名
      sApiName = aLine[0].gsub(/([A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z])\(.+\);/, "\\1")
      if (!$hApiInfo.has_key?(sApiName))
        error2("#{aLine[1]}:#{aLine[2]}", "static API `#{sApiName}' is unknown !")
      end
      # 静的APIインデックスが無ければエラー終了
      if (aLine[3].nil?)
        error_exit("Fatal error !! [#{__FILE__}:#{__LINE__}]")
      end

      # タブを除去して正規表現で設定情報を取得
      sTemp = aLine[0].gsub("\t", "")
      cMatch = sTemp.match(/#{$hApiInfo[sApiName][:REGEXP]}/)

      # パラメータの数が正しいか確認
      if (cMatch.size() != ($hApiInfo[sApiName][:PARAM].size() + 1))
        error2("#{aLine[1]}:#{aLine[2]}", "static API invalid parameters !")
      end

      # 処理対象のcfgファイル名と行数を#lineディレクティブで出力
      cCfg1Out.add("#line #{aLine[2]} \"#{aLine[1]}\"")

      # 整数定数式パラメータをcfg1_out.cに出力する
      $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
        if (aParam[1] == :UNSIGNED)
          cCfg1Out.append("const unsigned_t TOPPERS_cfg_valueof_#{aParam[0]}_#{aLine[3]} = ( unsigned_t )( #{cMatch[nIdx + 1].strip()} );")
        elsif (aParam[1] == :SIGNED)
          cCfg1Out.append("const signed_t TOPPERS_cfg_valueof_#{aParam[0]}_#{aLine[3]} = ( signed_t )( #{cMatch[nIdx + 1].strip()} );")
        elsif (aParam[1] == :OBJID)
          cCfg1Out.add2("#define #{cMatch[nIdx + 1].strip()}#{TAB}(<>)")

          # define宣言により改行が入るため，#lineディレクティブを改めて出力する
          cCfg1Out.add("#line #{aLine[2]} \"#{aLine[1]}\"")
        end
      }

      # 静的APIナンバーを付与(エラー発生時の行数を正しくするため最後に出力する)
      cCfg1Out.add("#{NL}const unsigned_t TOPPERS_cfg_static_api_#{aLine[3]} = #{aLine[3]};")
    end
  }
end


######################################################################
# PASS 2
######################################################################
def Pass2()
  #
  # テンプレートファイルを実行する
  #
  IncludeTrb($sTemplateFile)
end


######################################################################
# PASS 3
######################################################################
def Pass3()
  #
  # 指定されたsrecファイル読み込み
  #
  $cRomImage = SRecord.new($sOptRomImage)

  #
  # テンプレートファイルを実行する
  #
  IncludeTrb($sTemplateFile)

  #
  # 終了メッセージ
  #
  if ($sKernel == "asp")
    STDERR.puts("check complete")
  end
end


######################################################################
# PASS M (依存関係出力モード)
######################################################################
def PassM()
  print("#{$sDependencyFile}:")
  $aDependencyFile.each{|sFile|
    print(" #{sFile}")
  }
  puts("")
end


######################################################################
# オプション処理
######################################################################
cOpt = OptionParser.new(banner="Usage: cfg.rb [options]...", 35)
cOpt.version = VERSION
$sKernel = nil
$sPass = nil
$aInclude = []
sTemplateFile =nil
aOptApiTable = []
aOptCfg1DefTable = []
sOptRomImage = nil
sOptSymbolTable = nil
sDependencyFile = nil
cOpt.on("-k KERNEL", "--kernel KERNEL", "kernel profile name") {|xVal|
  $sKernel = xVal
}
cOpt.on("-p PASS", "--pass PASS", "process number") {|xVal|
  $sPass = xVal
}
cOpt.on("-I PATH", "--include-path PATH", "include directory path") {|xVal|
  $aInclude.push(xVal)
}
cOpt.on("-T TEMPLATE FILE", "--template-file TEMPLATE FILE", "template ruby file") {|xVal|
  sTemplateFile = xVal
}
cOpt.on("--api-table CSV", "csv file of static API table") {|xVal|
  aOptApiTable.push(xVal)
}
cOpt.on("--cfg1-def-table CSV", "csv file of symbol table") {|xVal|
  aOptCfg1DefTable.push(xVal)
}
cOpt.on("--rom-image SREC", "rom image file (s-record)") {|xVal|
  sOptRomImage = xVal
}
cOpt.on("--symbol-table SYMS", "symbol table file (nm)") {|xVal|
  sOptSymbolTable = xVal
}
cOpt.on("-M FILE", "--print-dependencies FILE", "dependency file") {|xVal|
  sDependencyFile = xVal
}
cOpt.on("-v", "--version", "show version information") {
  abort(cOpt.ver())
}
cOpt.on("-h", "--help", "show help (this)") {
  abort(cOpt.help())
}

begin
  aOptCfgData = cOpt.parse(ARGV)
rescue OptionParser::ParseError
  abort(cOpt.help())
end

# 依存関係出力モード
if (!sDependencyFile.nil?)
  $sPass = "M"
# --kernel，--pass，--api-tableオプションは必須
elsif ($sKernel.nil? || $sPass.nil? || aOptApiTable.empty?)
  puts("`--kernel' and `--pass' ans `--api-table' must be set !")
  abort(cOpt.help())
end

# pass2，pass3ではテンプレートファイルが必須
if (["2", "3"].include?($sPass) && sTemplateFile.nil?)
  puts("`--template-file' must be set with `pass2' or `pass3' !")
  abort(cOpt.help())
end

# pass3ではROMイメージ，シンボルテーブルが必須
if (($sPass == "3") && (sOptRomImage.nil? || sOptSymbolTable.nil?))
  puts("`--rom-image' and `--symbol-table' must be set with `pass3' !")
  abort(cOpt.help())
end

# 暫定的にaspのみ対応
if (($sKernel != "asp") && ($sPass != "M"))
  error_exit("Now, support only `asp' !")
end


######################################################################
# ファイル存在チェック＆パス設定
######################################################################
if (!sTemplateFile.nil?)
  $sTemplateFile = GetFilePath(sTemplateFile)
  if ($sTemplateFile.nil?)
    error_exit("`#{sTemplateFile}' is not found !")
  end
end

$aApiTable = []
aOptApiTable.each{|sFile|
  $aApiTable.push(GetFilePath(sFile))
  if ($aApiTable[-1].nil?)
    error_exit("`#{sFile}' is not found !")
  end
}

$aCfg1DefTable = []
aOptCfg1DefTable.each{|sFile|
  $aCfg1DefTable.push(GetFilePath(sFile))
  if ($aCfg1DefTable[-1].nil?)
    error_exit("`#{sFile}' is not found !")
  end
}

if (!sOptRomImage.nil?)
  $sOptRomImage = GetFilePath(sOptRomImage)
  if ($sOptRomImage.nil?)
    error_exit("`#{sOptRomImage}' is not found !")
  end
end

if (!sOptSymbolTable.nil?)
  $sOptSymbolTable = GetFilePath(sOptSymbolTable)
  if ($sOptSymbolTable.nil?)
    error_exit("`#{sOptSymbolTable}' is not found !")
  end
end

if (!sDependencyFile.nil?)
  $sDependencyFile = GetFilePath(sDependencyFile)
  if ($sDependencyFile.nil?)
    error_exit("`#{sDependencyFile}' is not found !")
  end
end

$sCfgFile = nil
aOptCfgData.each{|sFile|
  if ($sCfgFile.nil?)
    $sCfgFile = GetFilePath(sFile)
    if ($sCfgFile.nil?)
      error_exit("`#{sFile}' is not found !")
    end
  else
    error_exit("cfg file must be only one file !")
  end
}


######################################################################
# 共通処理
######################################################################

#
# 静的APIテーブル情報取得
#
$hApiInfo = {}
$aApiTable.each{|sFileName|
  cApiCsv = CSV.open(sFileName, "r")
  cApiCsv.each{|aRecord|
    $hApiInfo[aRecord[1]] = {}
    $hApiInfo[aRecord[1]][:TYPE] = aRecord[0]
    $hApiInfo[aRecord[1]][:PARAM] = aRecord[2]
    if (aRecord[3].nil?)
      $hApiInfo[aRecord[1]][:ID_POS] = "0"
    else
      $hApiInfo[aRecord[1]][:ID_POS] = aRecord[3]
    end
    $hApiInfo[aRecord[1]][:FLG] = aRecord[4]
  }
}
$hApiInfo.each{|sName, hData|
  # スペース区切りでパラメータリストを分割
  aParam = hData[:PARAM].split(" ")

  # 配列にして取得したパラメータ情報を格納する
  hData[:PARAM] = []

  # パラメータ情報を取得しつつ，静的APIの正規表現を作成する
  sRegExp = sName + '\('
  aParam.each{|sItem|
    case sItem[0]
    when "#"
      # オブジェクト識別名
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("#"), :OBJID])
    when "%"
      # 定義済みオブジェクト識別名
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("%"), :DEFINED])
    when "."
      # 符号無し整数定数式パラメータ
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("."), :UNSIGNED])
    when "+"
      # 符号付き整数定数式パラメータ
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("+"), :SIGNED])
    when "&"
      # 一般定数式パラメータ
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("&"), :GENERAL])
    when "$"
      # 文字列定数式パラメータ
      sRegExp += '([^,]+),'
      hData[:PARAM].push([sItem.delete("$"), :STRING])
    when "{"
      # 区切り括弧開く
      sRegExp += '[\s]*{'
    when "}"
      # 末尾がカンマの場合除去する
      if (sRegExp[-1] == ",")
        sRegExp.chop!()
      end
      # 区切り括弧閉じる
      sRegExp += '}'
    else
      # 上記以外はエラー
      error_exit("`#{sItem}' of `#{sName}' is invalid !")
    end
  }
  sRegExp += '[\s]*\)'

  # 静的APIの正規表現を保存
  hData[:REGEXP] = sRegExp
}
$hApiInfo.freeze()


#
# 各def.csvから外部スクリプトに渡す値取得シンボルを取得
#
$hSymbolTable = {}
$aCfg1DefTable.each{|sFile|
  cCsvData = CSV.open(sFile, "r")
  cCsvData.each{|aRecord|
    # 変数名
    if (aRecord[0].nil?)
      error_exit("`#{sFile}' is invalid format !")
    else
      $hSymbolTable[aRecord[0]] = {}
    end

    # 式
    if (aRecord[1].nil?)
      error_exit("`#{sFile}(#{aRecord[0]})' is invalid record !")
    else
      $hSymbolTable[aRecord[0]][:EXPR] = aRecord[1]
    end

    # 符号付きフラグ
    if (aRecord[2].nil? || (aRecord[2] == "u"))
      $hSymbolTable[aRecord[0]][:SIGNED] = CFG1_UNSIGNED
    elsif (aRecord[2] == "s" || (aRecord[2] == "signed"))
      $hSymbolTable[aRecord[0]][:SIGNED] = CFG1_SIGNED
    else
      error_exit("`#{sFile}(#{aRecord[0]})' is invalid value(#{aRecord[2]}) !")
    end

    # 式が真の場合の値
    if (aRecord[3].nil?)
      $hSymbolTable[aRecord[0]][:TRUE] = nil;
    else
      $hSymbolTable[aRecord[0]][:TRUE] = aRecord[3];
    end

    # 式が偽の場合の値
    if (aRecord[4].nil?)
      $hSymbolTable[aRecord[0]][:FALSE] = nil;
    else
      $hSymbolTable[aRecord[0]][:FALSE] = aRecord[4];
    end
  }
}
$hSymbolTable.freeze()


#
# cfgファイル情報取得
#
$aCfgFileData = []
File.open($sCfgFile){|cFile|
  cFile.each_with_index{|sLine, nIdx|
    # 改行コード，コメントを除去
    sLine.gsub!("\r", "")
    sLine.gsub!("\n", "")
    sLine.gsub!(/\/\*.*\*\//, "")
    $aCfgFileData.push([sLine, $sCfgFile, nIdx + 1])
  }
}
# INCLUDEディレクティブ展開(再起的に展開)
bChk = true
$aDependencyFile = [$sCfgFile]
while (bChk == true)
  bChk = false
  # 現在の最新状態をコピーして検索する
  aTempData = $aCfgFileData.dup()
  aTempData.each_with_index{|aLine, nIdx|
    # INCLUDEディレクティブにマッチする行のみ処理する
    if (aLine[0] =~ /INCLUDE\(".+"\).*;/)
      # ファイル名を取得
      sFile = aLine[0].gsub(/INCLUDE\("(.+)"\).*;/, "\\1")

      # 全パスを検索し，存在しなければエラー終了
      sIncludeFile = GetFilePath(sFile)
      if (sIncludeFile.nil?)
        error_exit("`#{sFile}' is not found !")
      end

      # 依存関係ファイルとして保持
      $aDependencyFile.push(sIncludeFile)

      # INCLUDEディレクティブが記述されていた行を削除
      $aCfgFileData.delete_at(nIdx)

      # INCLUDEディレクティブが記述されていた行にINCLUDEしたファイル情報を挿入
      File.open(sIncludeFile){|cFile|
        cFile.each_with_index{|sLine2, nIdx2|
          # 改行コード，コメントを除去
          sLine2.gsub!("\r", "")
          sLine2.gsub!("\n", "")
          sLine2.gsub!(/\/\*.*\*\//, "")
          $aCfgFileData.insert(nIdx + nIdx2, [sLine2, sIncludeFile, nIdx2 + 1])
        }
      }

      # 1ファイル展開したら，一旦終了し，再度先頭から検索しなおす
      bChk = true
      break
    end
  }
end
# 1行で完結していない静的APIをまとめる
aTempData = $aCfgFileData.dup()
nStartPos = 0
bRemainFlg = false
nApiCnt = 0
aTempData.each_with_index{|aLine, nIdx|
  if (bRemainFlg == true)
    # 途中の静的APIコードを移動する
    $aCfgFileData[nStartPos][0] += aLine[0]
    if (aLine[0].include?(";"))
      bRemainFlg = false
    end
    # 移動元の配列は削除しておく
    $aCfgFileData[nIdx][0] = ""
  elsif (aLine[0] =~ /[A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z]\(/)
    # 静的API開始コード
    if (!aLine[0].include?(";"))
      # 静的APIが";"で終了していない場合，以降のコードを移動する
      nStartPos = nIdx
      bRemainFlg = true
    end

    # 静的APIインデックスを付与
    $aCfgFileData[nIdx].push(nApiCnt)
    nApiCnt += 1
  end
}


#
# #includeで指定されたヘッダファイル名取得
#
$aIncludeFiles = []
$aCfgFileData.each{|aLine|
  if (aLine[0] =~ /^#include\s+".+"/)
    $aIncludeFiles.push(aLine[0].gsub(/^#include\s+"(.+)"/, "\\1"))
  end
}
$aIncludeFiles.freeze()


#
# pass2，pass3共通処理
#
$hSymbolAddress = {}
if (["2", "3"].include?($sPass))
  #
  # 必須ファイル存在チェック
  #
  if (!File.exist?(CFG1_SYMS))
    error_exit("`#{CFG1_SYMS}' is not found !")
  end
  if (!File.exist?(CFG1_SREC))
    error_exit("`#{CFG1_SREC}' is not found !")
  end

  #
  # symsファイル読み込み
  #
  aSymbolFiles = [CFG1_SYMS]
  if (!$sOptSymbolTable.nil? && (CFG1_SYMS != File.basename($sOptSymbolTable)))
    aSymbolFiles.push($sOptSymbolTable)
  end

  aSymbolFiles.each{|sFile|
    File.open(sFile){|cFile|
      cFile.each {|sLine|
        # スペース区切りで分解
        aTemp = sLine.split(" ")
        # 3列になっていない行は除外
        if (aTemp.size() == 3)
          # シンボルに"_"が付与されるコンパイラ対応
          if (aTemp[2] =~ /^_TOPPERS_cfg_/)
            aTemp[2].gsub!("_TOPPERS_cfg_", "TOPPERS_cfg_")
          end

          # TOPPERS_cfg_を除去したシンボル名をキーとして，アドレスを数値で保存する
          if (aTemp[2] =~ /^TOPPERS_cfg_/)
            $hSymbolAddress[aTemp[2].gsub("TOPPERS_cfg_", "")] = aTemp[0].hex()
          else
            $hSymbolAddress[aTemp[2]] = aTemp[0].hex()
          end
        end
      }
    }
  }
  $hSymbolAddress.freeze()

  # magic_numberが無ければエラー終了
  if (!$hSymbolAddress.has_key?(CFG1_MAGIC_NUM))
    error_exit("`#{CFG1_MAGIC_NUM}' is not found in `#{CFG1_SYMS}'")
  end

  #
  # cfg1_out.srecファイル読み込み
  #
  cCfg1Out = SRecord.new(CFG1_SREC)

  # エンディアンを取得(magic_numberは4byte固定)
  nMagicNumAddr = $hSymbolAddress[CFG1_MAGIC_NUM]
  sMagicNumData = cCfg1Out.get_data(nMagicNumAddr, 4)
  $bLittleEndian = nil
  if (sMagicNumData == "12345678")
    $bLittleEndian = false
  elsif (sMagicNumData == "78563412")
    $bLittleEndian = true
  else
    error_exit("`#{CFG1_MAGIC_NUM}' is invalid in `#{CFG1_SREC}'")
  end

  # signed_tのサイズを取得
  nSizeOfSigned = cCfg1Out.get_value($hSymbolAddress[CFG1_SIZEOF_SIGNED], 4, false)

  # 各アドレスからデータを取得
  $hToppersCfgSymbol = {}
  $hSymbolAddress.each{|sSymbol, nAddress|
    # 符号の有無を取得
    bSigned = false
    if ($hSymbolTable.has_key?(sSymbol))
      if ($hSymbolTable[sSymbol][:SIGNED] == CFG1_SIGNED)
        bSigned = true
      end
    elsif (sSymbol =~ /^valueof_/)
      # パラメータ名と静的APIインデックスを取得
      cMatch = sSymbol.match(/valueof_(\w+)_(\d+)/)
      if (cMatch.size() != 3)
        error_exit("invalid symbol: #{sSymbol} !")
      end
      $aCfgFileData.each{|aLine|
        # インデックスが一致する静的API宣言を取得
        if (!aLine[3].nil? && (aLine[3] == cMatch[2].to_i()))
          sApiName = aLine[0].gsub(/([A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z])\(.+\);/, "\\1")
          # 名称が一致するパラメータを取得
          $hApiInfo[sApiName][:PARAM].each{|aData|
            if ((aData[0] == cMatch[1]) && (aData[1] == :SIGNED))
              bSigned = true
              break
            end
          }
        end
      }
    elsif (sSymbol =~ /^static_api_/)
      # 静的APIインデックスは符号無しで固定
    else
      # 上記に合致しないシンボルは使用しない
      next
    end

    nValue = cCfg1Out.get_value(nAddress, nSizeOfSigned, bSigned)
    # srecファイルに値が入っていないシンボルは除外
    if (!nValue.nil?)
      $hToppersCfgSymbol[sSymbol] = nValue
    end
  }
  $hToppersCfgSymbol.freeze()


  #
  # 静的APIデータをハッシュ形式へ変換
  #
  $hCfgData = {}
  hIdCnt = Hash.new(1)
  hChkDup = Hash.new([])
  $aCfgFileData.each{|aLine|
    # 静的API宣言のみ処理する
    if (aLine[0] =~ /[A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z]\(.+\);/)
      # 静的API名
      sApiName = aLine[0].gsub(/([A-Z][A-Z][A-Z]_[A-Z][A-Z][A-Z])\(.+\);/, "\\1")
      if (!$hApiInfo.has_key?(sApiName))
        error2("#{aLine[1]}:#{aLine[2]}", "static API `#{sApiName}' is unknown !")
      end
      # 静的APIインデックスが無ければエラー終了
      if (aLine[3].nil?)
        error_exit("Fatal error !! [#{__FILE__}:#{__LINE__}]")
      end
      # cfg1_out.symsに静的APIインデックスが存在しなければ読み飛ばす
      # (ifdef等で消えた静的API)
      if (!$hToppersCfgSymbol.has_key?("static_api_#{aLine[3]}"))
        next
      end

      # タブを除去して正規表現で設定情報を取得
      sTemp = aLine[0].gsub("\t", "")
      cMatch = sTemp.match(/#{$hApiInfo[sApiName][:REGEXP]}/)

      # パラメータの数が正しいか確認
      if (cMatch.size() != ($hApiInfo[sApiName][:PARAM].size() + 1))
        error2("#{aLine[1]}:#{aLine[2]}", "static API invalid parameters !")
      end

      # 従属しない静的API
      if ($hApiInfo[sApiName][:FLG] != "1")
        # IDが存在する静的API
        if ($hApiInfo[sApiName][:ID_POS] != "-1")
          # 種別をラベルに変換
          lType = $hApiInfo[sApiName][:TYPE].to_sym()

          # 初出の種別に対してハッシュ初期化
          if (!$hCfgData.has_key?(lType))
            $hCfgData[lType] = {}
          end

          # IDを文字列＋数値クラスで取得する
          cID = nil
          $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
            if (nIdx == $hApiInfo[sApiName][:ID_POS].to_i())
              sID = cMatch[nIdx + 1].strip()

              # 同じIDが既に存在する場合はエラーとする
              if ($hCfgData[lType].has_key?(sID))
                error2("#{aLine[1]}:#{aLine[2]}", "E_OBJ: #{aParam[0]} `#{sID}' in #{sApiName} is duplicated")
              end

              # オブジェクト識別する場合，0から連番を付与する
              if (aParam[1] == :OBJID)
                cID = StrVal.new(sID, hIdCnt[lType])
                hIdCnt[lType] += 1
              # オブジェクト識別でない場合(符号無し整数の場合)，シンボルがあれば値を取得する
              else
                if ($hToppersCfgSymbol.has_key?("valueof_#{aParam[0]}_#{aLine[3]}"))
                  cID = StrVal.new(sID, $hToppersCfgSymbol["valueof_#{aParam[0]}_#{aLine[3]}"])
                else
                  cID = StrVal.new(sID)
                end
              end
              break
            end
          }

          # 対象オブジェクトにID以外のパラメータを格納する
          $hCfgData[lType][cID] = {}
          $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
            if (nIdx != $hApiInfo[sApiName][:ID_POS].to_i())
              if ($hToppersCfgSymbol.has_key?("valueof_#{aParam[0]}_#{aLine[3]}"))
                $hCfgData[lType][cID][aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip(), $hToppersCfgSymbol["valueof_#{aParam[0]}_#{aLine[3]}"])
              else
                $hCfgData[lType][cID][aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip())
              end
            elsif (aParam[1] != :OBJID)
              # オブジェクト識別でない場合，trbでの利便性のため，パラメータとしても格納しておく
              $hCfgData[lType][cID][aParam[0].to_sym()] = cID
            end
          }

          # ファイル，行数情報を付加
          $hCfgData[lType][cID][:_file_] = aLine[1]
          $hCfgData[lType][cID][:_line_] = aLine[2]

        # IDが存在しない静的API
        else
          # 種別をラベルに変換
          lType = $hApiInfo[sApiName][:TYPE].to_sym()

          # 初出の種別に対して配列初期化
          if (!$hCfgData.has_key?(lType))
            $hCfgData[lType] = []
          end

          # 対象オブジェクトに各パラメータを格納する
          hTemp = {}
          $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
            if ($hToppersCfgSymbol.has_key?("valueof_#{aParam[0]}_#{aLine[3]}"))
              hTemp[aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip(), $hToppersCfgSymbol["valueof_#{aParam[0]}_#{aLine[3]}"])
            else
              hTemp[aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip())
            end
          }

          # ファイル，行数情報を付加
          hTemp[:_file_] = aLine[1]
          hTemp[:_line_] = aLine[2]

          $hCfgData[lType].push(hTemp)
        end

      # 従属する静的API
      else
        # IDが存在しない場合，従属する静的APIが識別できない
        if ($hApiInfo[sApiName][:ID_POS] == "-1")
          error_exit("#{sApiName} must have ID position !")
        end

        # IDを取得
        sID = nil
        $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
          if (nIdx == $hApiInfo[sApiName][:ID_POS].to_i())
            sID = cMatch[nIdx + 1].strip()
            # 同じIDに対する従属をチェック
            if (hChkDup[sApiName].include?(sID))
              error2("#{aLine[1]}:#{aLine[2]}", "E_OBJ: #{aParam[0]} `#{sID}' in #{sApiName} is duplicated")
            else
              hChkDup[sApiName].push(sID)
            end
            break
          end
        }

        # 従属する静的API情報に追加
        bChk = false
        $hCfgData.each{|lType, hList|
          hList.each{|cID, aParam|
            if (cID == sID)
              lAddType = $hApiInfo[sApiName][:TYPE].to_sym()
              $hCfgData[lType][cID][lAddType] = {}
              $hApiInfo[sApiName][:PARAM].each_with_index{|aParam, nIdx|
                if (nIdx != $hApiInfo[sApiName][:ID_POS].to_i())
                  if ($hToppersCfgSymbol.has_key?("valueof_#{aParam[0]}_#{aLine[3]}"))
                    $hCfgData[lType][cID][lAddType][aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip(), $hToppersCfgSymbol["valueof_#{aParam[0]}_#{aLine[3]}"])
                  else
                    $hCfgData[lType][cID][lAddType][aParam[0].to_sym()] = StrVal.new(cMatch[nIdx + 1].strip())
                  end
                end
              }
              # 従属静的APIのファイル，行数情報を付加
              $hCfgData[lType][cID][lAddType][:_file_] = aLine[1]
              $hCfgData[lType][cID][lAddType][:_line_] = aLine[2]

              bChk = true
            end
          }
        }
        # 従属する静的APIが存在しなければエラー
        if (bChk == false)
          error2("#{aLine[1]}:#{aLine[2]}", "E_NOEXS: `#{sID}' is undefined !")
        end
      end
    end
  }

  # cfgファイルに登場しなかった静的APIは空のハッシュ/配列を定義しておく
  $hApiInfo.each{|sName, hData|
    if (!$hCfgData.has_key?(hData[:TYPE].to_sym()))
      if (hData[:ID_POS] != "-1")
        $hCfgData[hData[:TYPE].to_sym()] = {}
      else
        $hCfgData[hData[:TYPE].to_sym()] = []
      end
    end
  }
  $hCfgData.freeze()


  # テンプレートファイルから参照する変数定義
  # (グローバル変数として用意)
  $hToppersCfgSymbol.each{|sName, nValue|
    if ((sName !~ /^valueof_/) && (sName !~ /^static_api_/))
      eval("$#{sName} = StrVal.new(\"#{sName}\", #{nValue})")
    end
  }
end


######################################################################
# PASSに従って各処理を実行
######################################################################
case $sPass
  when "1"
    Pass1()
  when "2"
    Pass2()
  when "3"
    Pass3()
  when "M"
    PassM()
  else
    error_exit("Invalid pass: #{sPass}")
end

# エラー発生時は終了コードを1にする
if ($bErrorFlg == true)
  if ($0 == __FILE__)
    exit(1)
  else
    # simplecov対応
    raise()
  end
end

######################################################################
# 作成したすべてのファイルを出力する
######################################################################
GenFile.output()
