#!ruby -Ku
#
#  TOPPERS Generator for AUTOSAR by Ruby
#
#  Copyright (C) 2015 by Embedded and Real-Time Systems Laboratory
#              Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2015 by FUJI SOFT INCORPORATED, JAPAN
#
#  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
#  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
#  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
#  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
#      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
#      スコード中に含まれていること．
#  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
#      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
#      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
#      の無保証規定を掲載すること．
#  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
#      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
#      と．
#    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
#        作権表示，この利用条件および下記の無保証規定を掲載すること．
#    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
#        報告すること．
#  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
#      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
#      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
#      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
#      免責すること．
#
#  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
#  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
#  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
#  用する者に対して，AUTOSARパートナーになることを求めている．
#
#  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
#  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
#  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
#  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
#  の責任を負わない．
#
#  $Id: gen.rb 809 2017-03-08 12:00:54Z suzuki-kawaguchi $
#

if ($0 == __FILE__)
  TOOL_ROOT = File.expand_path(File.dirname(__FILE__))+ "/"
  $LOAD_PATH.unshift(TOOL_ROOT)
end

require "pp"
require "kconv"
require "yaml"
require "optparse"
require "rexml/document"
include REXML

require "common/GenFile.rb"
require "common/functions.rb"

######################################################################
# 定数定義
######################################################################
# 共通
VERSION = "0.1.0"

# ARXML関係
XML_SNAME    = "SHORT-NAME"
XML_DEF      = "DEFINITION-REF"
XML_VAL      = "VALUE"
XML_VAL_R    = "VALUE-REF"
XML_PARAM    = "PARAMETER-VALUES"
XML_REFER    = "REFERENCE-VALUES"
XML_SUB      = "SUB-CONTAINERS"
XML_ECUC_C   = "ECUC-CONTAINER-VALUE"
XML_ECUC_R   = "ECUC-REFERENCE-VALUE"
XML_P_NUM    = "ECUC-NUMERICAL-PARAM-VALUE"
XML_P_TXT    = "ECUC-TEXTUAL-PARAM-VALUE"


######################################################################
# 文字列変換関数
######################################################################
# 文字列→BOOLEAN変換
def StringToBoolean(sValue)
  if (["1", "true", "TRUE", "ON", "ENABLE"].include?(sValue))
    return "TRUE"
  elsif (["0", "false", "FALSE", "OFF", "DISABLE"].include?(sValue))
    return "FALSE"
  else
    return nil
  end
end
# 文字列→INT変換
def StringToInteger(sValue)
  # 10進数
  if (sValue =~ /^-??[1-9][0-9]*$/)
    return sValue.to_i()
  # 16進数
  elsif (sValue =~ /^-??0x[0-9A-Fa-f]+$/)
    return sValue.hex()
  # 8進数
  elsif (sValue =~ /^-??0[0-7]*$/)
    return sValue.oct()
  else
    return nil
  end
end
# 文字列→FLOAT変換
def StringToFloat(sValue)
  if ((sValue =~ /^-??[0-9]+\.??[0-9]+$/) ||
      (sValue =~ /^-??[0-9]+\.[0-9]+[eE][+-][0-9]+$/))
    return sValue.to_f()
  else
    return nil
  end
end


######################################################################
# 指定した条件によるIDの並べ替え
######################################################################
def SortObject(hTargetData, lCondition, lKey, aValue = nil)
  nCnt = 0
  hReturn = {}
  case lCondition
  # 指定したキーを持っているオブジェクトを先にする
  when :HAS_KEY
    hTargetData.each{|sName, hConfig|
      if (hConfig.has_key?(lKey))
        hConfig[:ID] = nCnt
        nCnt += 1
        hReturn[sName] = hConfig.dup()
      end
    }
    hTargetData.each{|sName, hConfig|
      if (!hConfig.has_key?(lKey))
        hConfig[:ID] = nCnt
        nCnt += 1
        hReturn[sName] = hConfig.dup()
      end
    }
  # 指定したキーを持っていないオブジェクトを先にする
  when :NOT_HAS_KEY
    hTargetData.each{|sName, hConfig|
      if (!hConfig.has_key?(lKey))
        hConfig[:ID] = nCnt
        nCnt += 1
        hReturn[sName] = hConfig.dup()
      end
    }
    hTargetData.each{|sName, hConfig|
      if (hConfig.has_key?(lKey))
        hConfig[:ID] = nCnt
        nCnt += 1
        hReturn[sName] = hConfig.dup()
      end
    }
  # 指定したキーの値の順番でオブジェクトをソートする
  when :VALUE
    aValue.each{|sValue|
      hTargetData.each{|sName, hConfig|
        if (hConfig[lKey] == sValue)
          hConfig[:ID] = nCnt
          nCnt += 1
          hReturn[sName] = hConfig.dup()
        end
      }
    }
  else
    abort("Invalid condition `:#{lCondition}'.#{NL}#{caller[0]}")
  end
  return hReturn
end


######################################################################
# 指定したIDによる並べ替え(0から連番でない場合エラー)
######################################################################
def SortObjectByID(hTargetData, lKey)
  hReturn = {}

  (0..(hTargetData.size() - 1)).each{|nCnt|
    bChk = false
    hTargetData.each{|sName, hConfig|
      if (!hConfig.has_key?(lKey))
        abort("Invalid key `:#{lKey}'.#{NL}#{caller[0]}")
      end
      if (hConfig[lKey] == nCnt)
        hReturn[sName] = hConfig.dup()
        bChk = true
      end
    }
    if (bChk == false)
      error_exit("#{lKey.to_s()} `#{nCnt}' is not found. #{lKey.to_s()} should be sequential from 0.")
    end
  }

  return hReturn
end


######################################################################
# パッケージパース関数
######################################################################
def ParsePackage(cElement, sPath)
  if (cElement.nil? || !cElement.has_elements?)
    return
  end

  cElement.elements.each("AR-PACKAGES/AR-PACKAGE"){|cElement1|
    ParsePackage(cElement1, sPath + "/" + cElement1.elements[XML_SNAME].text())
  }

  cElement.elements.each("ELEMENTS/ECUC-MODULE-CONFIGURATION-VALUES"){|cElement1|
    sPackageName = sPath
    if (!$hArxmlData.has_key?(sPackageName))
      $hArxmlData[sPackageName] = {}
    end

    sParamShortName = cElement1.elements[XML_SNAME].text()
    sParamDefName = cElement1.elements[XML_DEF].text()

    # 同じパスにおいて，ショートネームが重複している場合，エラー終了
    # todo:モジュールのショートネームに限っては許可する？
    #if ($hArxmlData[sPackageName].has_key?(sParamShortName))
    #  abort("#{XML_SNAME}: '#{sParamShortName}' is duplicated !")
    #end

    if (!$hArxmlData[sPackageName].has_key?(sParamShortName))
      $hArxmlData[sPackageName][sParamShortName] = {}
      $hArxmlData[sPackageName][sParamShortName][:DEF_REF] = sParamDefName
    end

    cElement1.elements.each("CONTAINERS/ECUC-CONTAINER-VALUE"){|cElement2|
      ParseContainer(cElement2, $hArxmlData[sPackageName][sParamShortName])
    }
  }
end


######################################################################
# コンテナパース関数
######################################################################
def ParseContainer(cElement, hTarget)
  sParamShortName = cElement.elements[XML_SNAME].text()
  sParamDefName = cElement.elements[XML_DEF].text()

  # 対象とするコンテナでなければ終了
  if (!$hContainerInfo.has_key?(sParamDefName))
    return
  end

  # 同じパスにおいて，ショートネームが重複している場合，エラー終了
  if (hTarget.has_key?(sParamShortName))
    abort("#{XML_SNAME}: '#{sParamShortName}' is duplicated !")
  end

  hTarget[sParamShortName] = {}
  hTarget[sParamShortName][:DEF_REF] = sParamDefName

  # パラメータ
  cElement.elements.each(XML_PARAM){|cElementC|
    [XML_P_NUM, XML_P_TXT].each{|sParamValue|
      cElementC.elements.each(sParamValue){|cElementG|
        sName = File.basename(cElementG.elements[XML_DEF].text())
        sValue = cElementG.elements[XML_VAL].text()
        # 複数多重度対応
        lName = sName.to_sym()
        if (hTarget[sParamShortName].has_key?(lName))
          if (hTarget[sParamShortName][lName].is_a?(Array))
            # 既に複数ある場合は配列に追加
            hTarget[sParamShortName][lName].push(sValue)
          else
            # 1つだけ定義されていた場合は配列に変更
            sTemp = hTarget[sParamShortName][lName]
            hTarget[sParamShortName][lName] = [sTemp, sValue]
          end
        else
          hTarget[sParamShortName][lName] = sValue
        end
      }
    }
  }

  # 参照，外部参照，選択参照
  cElement.elements.each("#{XML_REFER}/#{XML_ECUC_R}"){|cElementC|
    if (cElementC.elements[XML_VAL_R].nil?)
      abort("<VALUE> is not found in '#{sParamShortName}' !")
    end
    sName = File.basename(cElementC.elements[XML_DEF].text())
    lName = sName.to_sym()
    sValue = cElementC.elements[XML_VAL_R].text()
    # 複数多重度対応
    if (hTarget[sParamShortName].has_key?(lName))
      if (hTarget[sParamShortName][lName].is_a?(Array))
        # 既に複数ある場合は配列に追加
        hTarget[sParamShortName][lName].push(sValue)
      else
        # 1つだけ定義されていた場合は配列に変更
        sTemp = hTarget[sParamShortName][lName]
        hTarget[sParamShortName][lName] = [sTemp, sValue]
      end
    else
      hTarget[sParamShortName][lName] = sValue
    end
  }

  # サブコンテナ(再帰呼出し)
  cElement.elements.each("#{XML_SUB}/#{XML_ECUC_C}"){|cElementC|
    ParseContainer(cElementC, hTarget[sParamShortName])
  }
end


######################################################################
# 多重度，データ型，ENUM値チェック関数
######################################################################
def CheckParameter(hCheckData, sPath)
  if (hCheckData.has_key?(:DEF_REF))
    # ショートネームによるパス作成
    hCheckData[:SN_PATH] = sPath
  end

  # 対象コンテナの情報のみを取得する
  hTargetInfo = {}
  $hContainerInfo.each{|sDef, hInfo|
    if (sDef.include?(hCheckData[:DEF_REF]))
      # 対象コンテナ自身と，サブコンテナ以下のパラメータは除外する
      nCheckLayer = sDef.gsub(hCheckData[:DEF_REF] + "/", "").count("/")
      if (nCheckLayer == 0)
        hTargetInfo[sDef] = hInfo
      end
    end
  }
  if (hTargetInfo.empty?)
    return
  end

  hMultiplicity = Hash.new(0)
  hSwitchSingle = {}
  hSwitchMulti = {}
  hCheckData.each{|slKey, sahValue|
    if ((slKey == :DEF_REF) || (slKey == :SN_PATH))
      next
    end

    # サブコンテナの場合，コンテナ名をサブコンテナ側から取得する
    if (sahValue.is_a?(Hash))
      sDef = sahValue[:DEF_REF]

      if (hTargetInfo[sDef][:UPPER] == "1")
        # 多重度が最大1であるコンテナは利便性向上のためコンテナ名をキーに変更する
        # ※:SN_PATHにショートネームは残る
        lDef = File.basename(sDef).to_sym()
        # 0から連番のIDを付与
        sahValue[:ID] = $hIdCnt[lDef]
        hSwitchSingle[slKey] = {lDef => sahValue}
        $hIdCnt[lDef] += 1
      else
        # 多重度が*であるコンテナは，コンテナ名をキーとして，
        # ショートネーム毎のハッシュを作成する
        lDef = File.basename(sDef).to_sym()
        if (!hSwitchMulti.has_key?(lDef))
          hSwitchMulti[lDef] = {}
        end
        # 0から連番のIDを付与
        sahValue[:ID] = $hIdCnt[lDef]
        hSwitchMulti[lDef][slKey] = sahValue
        $hIdCnt[lDef] += 1
      end
    else
      sDef = hCheckData[:DEF_REF] + "/" + slKey.to_s()
    end

    # 処理対象コンテナのみカウントする
    if (hTargetInfo.has_key?(sDef))
      if (!sahValue.is_a?(Hash) && !sahValue.is_a?(Array))
        case hTargetInfo[sDef][:TYPE]
        when "BOOLEAN"
          hCheckData[slKey] = StringToBoolean(sahValue)
          if (hCheckData[slKey].nil?)
            error_exit_path_def(sPath, sDef, "data type is invalid. this data type is boolean. (#{sahValue})")
          end
        when "INT"
          hCheckData[slKey] = StringToInteger(sahValue)
          if (hCheckData[slKey].nil?)
            error_exit_path_def(sPath, sDef, "data type is invalid. this data type is integer. (#{sahValue})")
          end
          # レンジチェック
          if (hTargetInfo[sDef].has_key?(:MAX) && (hTargetInfo[sDef][:MAX] < hCheckData[slKey]))
            error_exit_path_def(sPath, sDef, "this value should be less than #{hTargetInfo[sDef][:MAX]}. (#{sahValue})")
          end
          if (hTargetInfo[sDef].has_key?(:MIN) && (hTargetInfo[sDef][:MIN] > hCheckData[slKey]))
            error_exit_path_def(sPath, sDef, "this value should be greater than #{hTargetInfo[sDef][:MIN]}. (#{sahValue})")
          end
        when "FLOAT"
          hCheckData[slKey] = StringToFloat(sahValue)
          if (hCheckData[slKey].nil?)
            error_exit_path_def(sPath, sDef, "data type is invalid. this data type is float. (#{sahValue})")
          end
          # レンジチェック
          if (hTargetInfo[sDef].has_key?(:MAX) && (hTargetInfo[sDef][:MAX].to_f() < hCheckData[slKey]))
            error_exit_path_def(sPath, sDef, "this value should be less than #{hTargetInfo[sDef][:MAX]}. (#{sahValue})")
          end
          if (hTargetInfo[sDef].has_key?(:MIN) && (hTargetInfo[sDef][:MIN].to_f() > hCheckData[slKey]))
            error_exit_path_def(sPath, sDef, "this value should be greater than #{hTargetInfo[sDef][:MIN]}. (#{sahValue})")
          end
        when "FUNCTION"
          if (sahValue !~ /^[a-zA-Z]+\w*$/)
            error_exit_path_def(sPath, sDef, "`#{sahValue}' does not according to C's identifier rules.")
          end
        when "ENUM"
          if (!hTargetInfo[sDef][:ENUM].include?(sahValue))
            error_exit_path_def(sPath, sDef, "enumeration value [#{hTargetInfo[sDef][:ENUM].join(', ')}] is invalid. (#{sahValue})")
          end
        end
      end

      # 多重度が1より大きいパラメータは一律配列にしておく
      if ((hTargetInfo[sDef][:UPPER] != "1") && !sahValue.is_a?(Array) && !sahValue.is_a?(Hash))
        hCheckData[slKey] = [sahValue]
      end

      if (sahValue.is_a?(Array))
        hMultiplicity[sDef] += sahValue.size()
      else
        hMultiplicity[sDef] += 1
      end
    end

    # サブコンテナの場合，再帰的に処理する
    if (sahValue.is_a?(Hash))
      CheckParameter(sahValue, sPath + "/" + slKey)
    end
  }
  # 多重度が最大1であるパラメータを入れ替え
  hSwitchSingle.each{|lKey, hData|
    hCheckData.delete(lKey)
    hCheckData.merge!(hData)
  }
  # 多重度が*であるパラメータを入れ替え
  hSwitchMulti.each{|lKey, hContainer|
    # 元のハッシュを削除
    hContainer.each{|sName, hData|
      hCheckData.delete(sName)
    }
    # コンテナ名をキーとして追加
    hCheckData[lKey] = hContainer
  }

  # 多重度チェック
  hTargetInfo.each{|sDef, hInfo|

    # パラメータが設定されている場合
    if (hMultiplicity.has_key?(sDef))
      # 多重度1のチェック
      if ((hInfo[:LOWER] == "1") && (hInfo[:UPPER] == "1"))
        if (hMultiplicity[sDef] != 1)
          error_exit_path_def(sPath, sDef, "multiplicity violation. (1)")
        end
      # 多重度1..*のチェック
      elsif ((hInfo[:LOWER] == "1") && (hInfo[:UPPER] != "1"))
        if (hMultiplicity[sDef] == 0)
          error_exit_path_def(sPath, sDef, "multiplicity violation. (1..#{hInfo[:UPPER]})")
        elsif ((hInfo[:UPPER] != '*') && (hMultiplicity[sDef] > hInfo[:UPPER].to_i))
          error_exit_path_def(sPath, sDef, "multiplicity violation. (1..#{hInfo[:UPPER]})")
        end
      # 多重度0..1のチェック
      elsif ((hInfo[:LOWER] == "0") && (hInfo[:UPPER] == "1"))
        if (hMultiplicity[sDef] > 1)
          error_exit_path_def(sPath, sDef, "multiplicity violation. (0..1)")
        end
      # 多重度0..*はチェック不要
      else
        if ((hInfo[:UPPER] != '*') && (hMultiplicity[sDef] > hInfo[:UPPER].to_i))
          error_exit_path_def(sPath, sDef, "multiplicity violation. (1..#{hInfo[:UPPER]})")
        end
      end
    # パラメータが設定されていない場合
    else
      # 多重度1のチェック
      if ((hInfo[:LOWER] == "1") && (hInfo[:UPPER] == "1"))
        error_exit_path_def(sPath, sDef, "multiplicity violation. (1)")
      # 多重度1..*のチェック
      elsif ((hInfo[:LOWER] == "1") && (hInfo[:UPPER] != "1"))
        error_exit_path_def(sPath, sDef, "multiplicity violation. (1..#{hInfo[:UPPER]})")
      # 多重度0..1，0..*はチェック不要
      else
      end
    end
  }
end


######################################################################
# ショートネームパスからのコンフィギュレーションデータ取得
######################################################################
def GetConfigData(sShortNamePath, hForError, hCurrentData = nil)
  hReturn = nil

  if (hCurrentData.nil?)
    $hArxmlData.each{|sPackage, hModule|
      hModule.each{|sModule, hAllConfig|
        hReturn = GetConfigData(sShortNamePath, hForError, hAllConfig)
        if (!hReturn.nil?)
          return hReturn
        end
      }
    }
  else
    hCurrentData.each{|slKey, sahData|
      if (sahData.is_a?(Hash))
        # ショートネームパスが一致したらデータを返す
        if (sahData.has_key?(:SN_PATH) && (sahData[:SN_PATH] == sShortNamePath))
          return sahData
        else
          # 再帰的に検索していく
          hReturn = GetConfigData(sShortNamePath, hForError, sahData)
          if (!hReturn.nil?)
            return hReturn
          end
        end
      end
    }
  end

  # データが見つからなかった場合はエラー終了
  if (!caller[0].include?("GetConfigData") && hReturn.nil?)
    error_exit_hash(hForError, "`#{sShortNamePath}' is not found.")
  end

  return hReturn
end


######################################################################
# 各エラー/警告表示関数
######################################################################
def error_exit(sMessage)
  abort("Generator Error: #{sMessage}")
end
def error_exit_path_def(sPath, sDef, sMessage)
  error_exit("#{NL}  PATH: #{sPath}#{NL}  DEF : #{sDef}#{NL}  INFO: #{sMessage}")
end
def error_exit_hash(hInfo, sMessage)
  error_exit_path_def(hInfo[:SN_PATH], hInfo[:DEF_REF], sMessage)
end
def warning(hInfo, sMessage)
  STDERR.puts("Generator Warning:#{NL}  PATH: #{hInfo[:SN_PATH]}#{NL}  DEF : #{hInfo[:DEF_REF]}#{NL}  INFO: #{sMessage}")
end


######################################################################
# オプション処理
######################################################################
cOpt = OptionParser.new(banner="Usage: gen.rb [options]...", 35)
cOpt.version = VERSION
sTemplateFile =nil
aOptContainerInfoFile = []
$aInclude = []
cOpt.on("-T TEMPLATE FILE", "--template-file TEMPLATE FILE", "template ruby file") {|xVal|
  sTemplateFile = xVal
}
cOpt.on("-c YAML", "--container-info YAML", "yaml file of XML container infomation") {|xVal|
  aOptContainerInfoFile.push(xVal)
}
cOpt.on("-I PATH", "--include-path PATH", "include directory path") {|xVal|
  $aInclude.push(xVal)
}
cOpt.on("-v", "--version", "show version information") {
  abort(cOpt.ver())
}
cOpt.on("-h", "--help", "show help (this)") {
  abort(cOpt.help())
}

begin
  aOptArxmlData = cOpt.parse(ARGV)
rescue OptionParser::ParseError
  abort(cOpt.help())
end

# -c/--container-infoオプションは必須
if (aOptContainerInfoFile.empty?)
  puts("`-c' or `--container-info' must be set !")
  abort(cOpt.help())
end

# テンプレートファイルは必須
if (sTemplateFile.nil?)
  puts("`--template-file' must be set !")
  abort(cOpt.help())
end


######################################################################
# ファイル存在チェック＆パス設定
######################################################################
$sTemplateFile = GetFilePath(sTemplateFile)
if ($sTemplateFile.nil?)
  abort("`#{sTemplateFile}' is not found !")
end

$aContainerInfoFile = []
aOptContainerInfoFile.each{|sFile|
  $aContainerInfoFile.push(GetFilePath(sFile))
  if ($aContainerInfoFile[-1].nil?)
    error_exit("`#{sFile}' is not found !")
  end
}

$aArxmlFiles = []
aOptArxmlData.each{|sFile|
  $aArxmlFiles.push(GetFilePath(sFile))
  if ($aArxmlFiles[-1].nil?)
    abort("`#{sFile}' is not found !")
  end
}


######################################################################
# メイン処理
######################################################################

#
# arxmlデータ読込み
#
$aArxmlData = []
$aArxmlFiles.each{|sFile|
  cXmlData = REXML::Document.new(open(sFile))
  $aArxmlData.push(cXmlData)
}
$aArxmlData.freeze()

#
# XMLコンテナ情報読込み
#
$hContainerInfo = {}
$aContainerInfoFile.each{|sFileName|
  hInfo = YAML.load(File.read(sFileName).toutf8())
  $hContainerInfo.merge!(hInfo)
}
$hContainerInfo.freeze()

#
# arxmlデータをハッシュ形式へ変換
#
$hArxmlData = {}
$aArxmlData.each{|cXmlData|
  cXmlData.elements.each("/AUTOSAR/AR-PACKAGES/AR-PACKAGE"){|cElement|
    ParsePackage(cElement, cElement.elements[XML_SNAME].text())
  }
}
$hArxmlData.freeze()


#
# 多重度，データ型，ENUM値チェック
#
hMultiplicity = Hash.new(0)
$hIdCnt = Hash.new(0)
$hArxmlData.each{|sPackage, hModule|
  hModule.each{|sModule, hAllConfig|
    # ショートネームによるパス作成
    hAllConfig[:SN_PATH] = "/#{sPackage}/#{sModule}"

    # モジュール多重度チェック
    hMultiplicity[hAllConfig[:DEF_REF]] += 1

    if ($hContainerInfo.has_key?(hAllConfig[:DEF_REF]))
      hInfo = $hContainerInfo[hAllConfig[:DEF_REF]]
      if ((hInfo[:LOWER] == "1") && (hInfo[:UPPER] == "1"))
        if (hMultiplicity[hAllConfig[:DEF_REF]] > 1)
          error_exit_hash(hAllConfig, "multiplicity violation. (1)")
        end
      else
        # todo:モジュール多重度は1のみ？
        # abort("Module multiplicity must be 1.")
      end
    else
      # 対象外のモジュールは読み飛ばす
      next
    end

    # 再帰的にパラメータをチェックする
    CheckParameter(hAllConfig, hAllConfig[:SN_PATH])
  }
}


#
# テンプレートファイルを実行する
#
IncludeTrb($sTemplateFile)


#
# 作成したすべてのファイルを出力する
#
GenFile.output()
