#!ruby -Ke
#
#  TOPPERS/A-WDG
#      Automotive Watchdog Driver
#
#  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
#                             Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
#  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
#  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
#  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
#  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
#  Copyright (C) 2014-2015 by Witz Corporation
#
#  本ソフトウェアは，車載制御システム向け高品質プラットフォームに関す
#  るコンソーシアム型共同研究において開発したものである．本ソフトウェ
#  アの著作権は，上記の著作権者に帰属する．
#
#  $Id: get_impl_id.rb 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
#

#
#  ソースコード中の仕様タグを抽出する
#

if ($0 == __FILE__)
  TOOL_ROOT = File.expand_path(File.dirname(__FILE__) + "/")
  $LOAD_PATH.unshift(TOOL_ROOT)
end

require "pp"

# 書き出すファイル
sWriteFileId = File.dirname(__FILE__) + "/wdg_impl_id.txt"

# 対象ソースコード
aTargetFiles = []
aTargetFiles.push("#{TOOL_ROOT}/../Wdg.c")
aTargetFiles.push("#{TOOL_ROOT}/../Wdg.h")
aTargetFiles.push("#{TOOL_ROOT}/../wdg.tf")
aTargetFiles.push("#{TOOL_ROOT}/../target/nios2_dev_gcc/Wdg_Irq.c")
aTargetFiles.push("#{TOOL_ROOT}/../target/nios2_dev_gcc/Wdg_Target.h")
aTargetFiles.push("#{TOOL_ROOT}/../target/nios2_dev_gcc/Wdg_Target.tf")

# 対象タグ正規表現
TAG_REG_WDG = "WDG[0-9][0-9][0-9]"
TAG_REG_WDG_CONF = "WDG[0-9][0-9][0-9]_Conf"
TAG_REG_WDGA = "WDGa[0-9][0-9][0-9]"
TAG_REG_NWDG = "NWDG[0-9][0-9][0-9]"
TAG_REG_DWDG = "DWDG[0-9][0-9][0-9]"
TAG_REG_ALL = "N?D?WDGa?[0-9][0-9][0-9][_Conf]*"

#
# 全ソースコード収集
#
aCodeData = []
aTargetFiles.each{|sFileName|
  # ファイル読み込み
  sFileData = File.read(sFileName)

  # 改行コードを変換
  sFileData.gsub!("\r\n","\n")
  sFileData.gsub!("\r","\n")

  aFileData = sFileData.split("\n")

  # [1行毎のコード，行数，ファイル名]のセットで保持する
  nLine = 1
  aFileData.each{|sLine|
    aCodeData.push([sLine, nLine, File.basename(sFileName)])
    nLine += 1
  }
}

# 取得結果
hSpecInfo = {}
aCodeData.each{|aLine|
  # 仕様タグが存在しない場合は何もしない
  if (aLine[0] !~ /\W#{TAG_REG_ALL}\W/)
    next
  end

  aTags = []
  # 1行に含まれるすべての仕様タグを抽出
  aTags.concat(aLine[0].scan(/\[#{TAG_REG_ALL}\]/))
  aTags.concat(aLine[0].scan(/\<#{TAG_REG_ALL}\>/))

  aTags.each{|sTag|
    if (!hSpecInfo.has_key?(sTag))
      hSpecInfo[sTag] = []
    end
    hSpecInfo[sTag].push("#{aLine[2]}:#{aLine[1]}")
  }
}

# 取得結果をIDでソート
aTags = hSpecInfo.keys
aTags.sort!

#
# 取得結果をファイルへ出力する
#
sResult = ""
aTags.each{|sTag|
  sPlace = ""
  hSpecInfo[sTag].each{|sLine|
    sPlace += "#{sLine}, "
  }
  sPlace.chop!().chop!()

  sResult += "#{sTag} @ #{sPlace}\n"
}

File.open(sWriteFileId, "w") {|io|
  io.puts(sResult)
}

#
# 対象仕様と非対象仕様の重複チェック
#
aTargetSpec = []
aNonTargetSpec = []
aCodeData.each{|aLine|
  # 仕様タグが存在しない場合は何もしない
  if (aLine[0] !~ /\W#{TAG_REG_ALL}\W/)
    next
  end

  # 1行に含まれるすべての仕様タグを抽出
  aTargetSpec.concat(aLine[0].scan(/\[#{TAG_REG_ALL}\]/))
  aNonTargetSpec.concat(aLine[0].scan(/\<#{TAG_REG_ALL}\>/))
}

aTmp = []
aTargetSpec.each{|sTarget|
  aTmp.push(sTarget.gsub(/\W(#{TAG_REG_ALL})\W/, "\\1"))
}
aTargetSpec = aTmp
aTmp = []
aNonTargetSpec.each{|sNonTarget|
  aTmp.push(sNonTarget.gsub(/\W(#{TAG_REG_ALL})\W/, "\\1"))
}
aNonTargetSpec = aTmp

# 重複しているものを削除し，IDでソート
aTargetSpec.uniq!
aNonTargetSpec.uniq!
aTargetSpec.sort!
aNonTargetSpec.sort!

nTargetWdg = 0
nTargetWdgConf = 0
nTargetWdgA = 0
nTargetNWdg = 0
nTargetDWdg = 0
aTargetSpec.each{|sTag|
  if (sTag =~ /^#{TAG_REG_WDG}$/)
    nTargetWdg = nTargetWdg + 1
  elsif (sTag =~ /^#{TAG_REG_WDG_CONF}$/)
    nTargetWdgConf = nTargetWdgConf + 1
  elsif (sTag =~ /^#{TAG_REG_WDGA}$/)
    nTargetWdgA = nTargetWdgA + 1
  elsif (sTag =~ /^#{TAG_REG_NWDG}$/)
    nTargetNWdg = nTargetNWdg + 1
  elsif (sTag =~ /^#{TAG_REG_DWDG}$/)
    nTargetDWdg = nTargetDWdg + 1
  else
    abort("Error!!")
  end
}

# サポート対象/非対象の件数表示
puts("=" * 35)
puts("All tags             : #{aTargetSpec.size() + aNonTargetSpec.size()}")
puts("-" * 35)
fRatio = aTargetSpec.size().to_f() / aTags.size().to_f * 100
puts("For support tags     : #{aTargetSpec.size()} (#{sprintf('%0.1f', fRatio)}%)")
fRatio = aNonTargetSpec.size().to_f() / aTags.size().to_f * 100
puts("For non-support tags : #{aNonTargetSpec.size()} (#{sprintf('%0.1f', fRatio)}%)")
puts("=" * 35)

# サポート対象の詳細表示
puts("For support tags     : #{aTargetSpec.size()}")
puts("-" * 35)
fRatio = nTargetWdg.to_f() / aTargetSpec.size().to_f() * 100
printf("WDG***               : %3d (%0.1f%%)\n", nTargetWdg, fRatio)
fRatio = nTargetWdgConf.to_f() / aTargetSpec.size().to_f() * 100
printf("WDG***_Conf          : %3d (%0.1f%%)\n", nTargetWdgConf, fRatio)
fRatio = nTargetWdgA.to_f() / aTargetSpec.size().to_f() * 100
printf("WDGa***              : %3d (%0.1f%%)\n", nTargetWdgA, fRatio)
fRatio = nTargetNWdg.to_f() / aTargetSpec.size().to_f() * 100
printf("NWDG***              : %3d (%0.1f%%)\n", nTargetNWdg, fRatio)
fRatio = nTargetDWdg.to_f() / aTargetSpec.size().to_f() * 100
printf("DWDG***              : %3d (%0.1f%%)\n", nTargetDWdg, fRatio)
puts("=" * 35)

# 重複したタグを表示
aTargetSpec.each{|sTarget|
  if (aNonTargetSpec.include?(sTarget))
    puts("#{sTarget} is conflicted !!")
    pp hSpecInfo["[#{sTarget}]"]
    pp hSpecInfo["<#{sTarget}>"]
  end
}
