$
$  TOPPERS/A-WDG
$      Automotive Watchdog Driver
$
$  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
$                             Graduate School of Information Science, Nagoya Univ., JAPAN
$  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
$  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
$  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
$  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
$  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
$  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
$  Copyright (C) 2016-2017 by SUZUKI MOTOR CORPORATION
$  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
$  Copyright (C) 2014-2017 by Witz Corporation
$
$  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
$  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
$  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
$  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
$      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
$      スコード中に含まれていること．
$  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
$      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
$      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
$      の無保証規定を掲載すること．
$  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
$      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
$      と．
$    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
$        作権表示，この利用条件および下記の無保証規定を掲載すること．
$    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
$        報告すること．
$  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
$      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
$      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
$      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
$      免責すること．
$
$  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
$  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
$  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
$  用する者に対して，AUTOSARパートナーになることを求めている．
$
$  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
$  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
$  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
$  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
$  の責任を負わない．
$
$  $Id: wdg.tf 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
$

$	// 定数定義
$tenth_int64_max = 0x7FFFFFFFFFFFFFFF / 10$


$ ====================================================================
$ boolean判定関数
$ ====================================================================
$FUNCTION IS_TRUE$
	$true_list = {1, "1", "true", "TRUE", "ON", "ENABLE"}$
	$false_list = {0, "0", "false", "FALSE", "OFF", "DISABLE"}$
	$error_flag = 1$

	$FOREACH false false_list$
		$IF EQ(false, ARGV[1])$
			$check_result = VALUE("FALSE", 0)$
			$error_flag = 0$
		$END$
	$END$
	$FOREACH true true_list$
		$IF EQ(true, ARGV[1])$
			$check_result = VALUE("TRUE", 1)$
			$error_flag = 0$
		$END$
	$END$

	$IF error_flag > 0$
		$ERROR ARGV[2]$$FORMAT(_("%1% must be boolean type"), ARGV[1])$$END$
	$END$

	$RESULT = check_result$
$END$

$ ====================================================================
$ 浮動小数型用割り算関数
$ ====================================================================
$FUNCTION DIV_FLOAT_VALUE$
	$arg1 = ARGV[1]$
	$arg2 = ARGV[2]$
	$arg_name1 = ARGV[3]$
	$arg_name2 = ARGV[4]$
	$arg_str1 = ETOF(arg1)$
	$arg_str2 = ETOF(arg2)$

$	// 引数に小数点が指定されていない場合はエラー
	$tmp = REGEX_REPLACE(arg_str1, "^[0-9]+\\.[0-9]+$", "")$
	$IF !EQ(tmp, "")$
		$ERROR$$FORMAT(_("[%1%]`%2%\' is not a float value."), arg_name1, arg_str1)$$END$$DIE()$
	$END$
	$tmp = REGEX_REPLACE(arg_str2, "^[0-9]+\\.[0-9]+$", "")$
	$IF !EQ(tmp, "")$
		$ERROR$$FORMAT(_("[%1%]`%2%\' is not a float value."), arg_name2, arg_str2)$$END$$DIE()$
	$END$

	$num1_l = REGEX_REPLACE(arg_str1, "^([0-9]+)\\.[0-9]+$", "$1")$
	$num1_r = REGEX_REPLACE(arg_str1, "^[0-9]+\\.([0-9]+)$", "$1")$
	$num1 = ATOI(CONCAT(num1_l, num1_r), 10)$
	$num2_l = REGEX_REPLACE(arg_str2, "^([0-9]+)\\.[0-9]+$", "$1")$
	$num2_r = REGEX_REPLACE(arg_str2, "^[0-9]+\\.([0-9]+)$", "$1")$
	$num2 = ATOI(CONCAT(num2_l, num2_r), 10)$

$	// 0割エラー
	$IF num2 == 0$
		$ERROR$$FORMAT(_("Must not be set 0 to %1%."), arg_name2)$$END$$DIE()$
	$END$

$	// num1が0ならば割り算を実行せず0を返す
	$div_result = 0$
	$IF num1 != 0$
$		// 倍率を揃える
		$adjust = 1$
		$digits_diff = STRING_SIZE(num1_r) - STRING_SIZE(num2_r)$
		$IF digits_diff > 0$
			$cnt = digits_diff$
			$WHILE(cnt > 0)$
				$IF num2 <= tenth_int64_max$
					$num2 = num2 * 10$
					$cnt = cnt - 1$
				$ELSE$
$					// num2がoverflowする時点でnum1<num2なので，div_resultは1と確定する
					$div_result = 1$
					$adjust = 0$
					$cnt = 0$
				$END$
			$END$
		$ELIF digits_diff < 0$
			$cnt = -digits_diff$
			$WHILE(cnt > 0)$
				$IF num1 <= tenth_int64_max$
					$num1 = num1 * 10$
					$cnt = cnt - 1$
				$ELSE$
					$IF cnt > 18$
						$ERROR$$FORMAT(_("overflow the 64bit. %1% / %2% : (%3%) / (%4%)"), arg1, arg2, arg_name1, arg_name2)$$END$$DIE()$
					$END$
					$WHILE(cnt > 0)$
						$adjust = adjust * 10$
						$cnt = cnt - 1$
					$END$
				$END$
			$END$
			
			$IF adjust != 1$
$				// overflowする部分をover_numに退避
				$over = 1000000000000000000 / adjust$
				$over_num = num1 / over$
$				// over_num >= num2 であれば除算結果もoverflowするので表現不可能
				$IF over_num >= num2$
					$ERROR$$FORMAT(_("overflow the 64bit. %1% / %2% : (%3%) / (%4%)"), arg1, arg2, arg_name1, arg_name2)$$END$$DIE()$
				$END$
$				// over_numを最大桁にする
				$over_num = over_num * over$
$				// under_num(overflowしない部分)にadjust(調整倍率)をかける
				$under_num = (num1 - over_num) * adjust$

$				// エラー条件を満たさなかったことから，(over_num / num2)の後であればadjustをかけることが可能
$				// あまりにadjustをかけてからnum2で割った数値を足すことで桁落ちを防止
				$tmp = (over_num % num2) * adjust$
				$over_div = (over_num / num2) * adjust + tmp / num2$
				$over_mod = tmp % num2$
				$under_div = under_num / num2$
				$under_mod = under_num % num2$

				$div_result = over_div + under_div$
$				// [NCOM005] 割り算の余りは繰り上げる
				$IF (over_mod + under_mod) > 0$
					$div_result = div_result + 1$
				$END$
			$END$
		$END$

$		調整不要(adjust == 1)の場合は通常の割り算を実施
		$IF adjust == 1$
			$div_result = num1 / num2$
			$mod_result = num1 % num2$
$			// [NCOM005] 割り算の余りは繰り上げる
			$IF mod_result > 0$
				$div_result = div_result + 1$
			$END$
		$END$
	$END$

$	// コード仕様で32bit以上の値は設定できないのでエラー
	$IF div_result > 0xFFFFFFFF$
		$ERROR$$FORMAT(_("overflow the 32bit. %1% / %2% : (%3%) / (%4%)"), arg1, arg2, arg_name1, arg_name2)$$END$$DIE()$
	$END$

	$RESULT = div_result$
$END$

$ ====================================================================
$ 文字列サイズ取得関数
$ ====================================================================
$FUNCTION STRING_SIZE$
	$size_result = 0$
	$string_tmp = ARGV[1]$
	$WHILE !EQ(string_tmp, "")$
		$string_tmp = REGEX_REPLACE(string_tmp, "^.(.*)$", "$1")$
		$size_result = size_result + 1$
	$END$
	$RESULT = size_result$
$END$

$ ====================================================================
$  指数表記から小数点表記への変換関数
$ ====================================================================
$FUNCTION ETOF$
$	// 引数に指数表記の値が指定されていない場合は処理しない
	$tmp = REGEX_REPLACE(ARGV[1], "^[0-9]+\\.[0-9]+[e|E][+|-][0-9]+$", "")$
	$IF !EQ(tmp, "")$
		$RESULT = ARGV[1]$
	$ELSE$
		$num_l = REGEX_REPLACE(ARGV[1], "^([0-9]+)\\.[0-9]+[e|E][+|-][0-9]+$", "$1")$
		$num_r = REGEX_REPLACE(ARGV[1], "^[0-9]+\\.([0-9]+)[e|E][+|-][0-9]+$", "$1")$
		$sign  = REGEX_REPLACE(ARGV[1], "^[0-9]+\\.[0-9]+[e|E]([+|-])[0-9]+$", "$1")$
		$indx  = REGEX_REPLACE(ARGV[1], "^[0-9]+\\.[0-9]+[e|E][+|-]([0-9]+)$", "$1")$
		$IF EQ(sign, "+")$
			$result = num_l$
			$FOREACH i RANGE(1, ATOI(indx, 10))$
				$IF EQ(num_r, "")$
					$result = CONCAT(result, "0")$
				$ELSE$
					$result = CONCAT(result, REGEX_REPLACE(num_r, "^(.).*$", "$1"))$
					$num_r = REGEX_REPLACE(num_r, "^.(.*)$", "$1")$
				$END$
			$END$
			$IF EQ(num_r, "")$
				$result = CONCAT(result, ".0")$
			$ELSE$
				$result = CONCAT(result, ".")$
				$result = CONCAT(result, num_r)$
			$END$
		$ELIF EQ(sign, "-")$
			$result = num_r$
			$FOREACH i RANGE(1, ATOI(indx))$
				$IF EQ(num_l, "")$
					$result = CONCAT("0", result)$
				$ELSE$
					$result = CONCAT(REGEX_REPLACE(num_l, "^.*(.)$", "$1"), result)$
					$num_l = REGEX_REPLACE(num_l, "^(.*).$", "$1")$
				$END$
			$END$
			$IF EQ(num_l, "")$
				$result = CONCAT("0.", result)$
			$ELSE$
				$result = CONCAT(".", result)$
				$result = CONCAT(num_l, result)$
			$END$
		$END$
		$RESULT = result$
	$END$
$END$

$ ====================================================================
$ ウィンドウオープン期間設定関数
$ ====================================================================
$FUNCTION SET_WINDOW_PARAM$
	$window_open_rate_check = "FALSE"$
	$FOREACH window_open_rate TARGET_WINDOW_OPEN_LIST$
		$IF EQ(ARGV[1], window_open_rate)$
			$window_open_rate_check = "TRUE"$
		$END$
	$END$
	$IF EQ(window_open_rate_check, "FALSE")$
		$ERROR$$FORMAT(_("%1% is not valid in this target."), ARGV[1])$$END$$DIE()$
	$END$
$END$

$ ====================================================================
$ タイムアウト時間設定関数
$ ====================================================================
$FUNCTION SET_HW_TIMEOUT_PARAM$
	$hw_timeout = DIV_FLOAT_VALUE(ARGV[1], "0.001", "WdgTimeout", "Constant Value")$
	$IF LENGTH(TARGET_HW_TIMEOUT_LIST)$ 
		$hw_timeout_check = "FALSE"$
		$FOREACH hw_timeout_list TARGET_HW_TIMEOUT_LIST$
			$IF EQ(hw_timeout, hw_timeout_list)$
				$hw_timeout_check = "TRUE"$
			$END$
		$END$
		$IF EQ(hw_timeout_check, "FALSE")$
			$ERROR$$FORMAT(_("%1% is not valid in this target."), ARGV[1])$$END$$DIE()$
		$END$
	$END$
	$RESULT = hw_timeout$
$END$

$ ====================================================================
$  エラーチェック関数
$ ====================================================================
$FUNCTION ERROR_CHECK$
	$is_boolean = IS_TRUE(WdgGeneral.WdgDevErrorDetect[1])$
	$disable_allow = IS_TRUE(WdgGeneral.WdgDisableAllowed[1])$
	$is_boolean = IS_TRUE(WdgGeneral.WdgVersionInfoApi[1])$

	$toggle_allow = IS_TRUE(TARGET_TOGGLE_ALLOW)$
	$window_allow = IS_TRUE(TARGET_WINDOW_ALLOW)$
	$trigger_location_allow = IS_TRUE(TARGET_TRIGGER_LOCATION_ALLOW)$
	$target_run_area_allow = IS_TRUE(TARGET_RUN_AREA_ALLOW)$
	$reset_allow = IS_TRUE(TARGET_TIMEOUT_RESET_ALLOW)$
	$nmi_allow = IS_TRUE(TARGET_TIMEOUT_NMI_ALLOW)$
	$mode_set_allow = IS_TRUE(TARGET_MODE_SET_ALLOW)$

$	//[WDG090][WDG092] 依存部有効可否チェック
$	//ウォッチドッグモード設定チェック
	$IF EQ(WdgPublishedInformation.WdgTriggerMode[1], "WDG_BOTH")$
		$IF (EQ(toggle_allow, "FALSE") || EQ(window_allow, "FALSE"))$
			$ERROR$$FORMAT(_("WDG_BOTH is not valid in this target."))$$END$$DIE()$
		$ELSE$
			$FOREACH config_id WdgSettingsConfig.ID_LIST$
				$FOREACH fast_id WdgSettingsFast.ID_LIST$
					$IF EQ(config_id, WdgSettingsFast.PARENT[fast_id])$
$						//ウィンドウオープン期間設定
						$IF EQ(WdgSettingsFast.WdgTriggerMode[fast_id], "WDG_TOGGLE")$
$							//トグルモードの場合，ウィンドウオープン期間は100
							$WdgSettingsFast.WdgWindowOpenRate[fast_id] = 100$
						$ELIF EQ(WdgSettingsFast.WdgTriggerMode[fast_id], "WDG_WINDOW")$
$							//ウィンドウモードの場合，ウィンドウオープン期間が設定リストにあるか確認
							$SET_WINDOW_PARAM(WdgSettingsFast.WdgWindowOpenRate[fast_id])$
						$ELSE$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTriggerMode[fast_id])$$END$$DIE()$
						$END$
$						//タイムアウト時間設定
						$WdgSettingsFast.WdgTimeout[fast_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsFast.WdgTimeout[fast_id])$
					$END$
				$END$
				$FOREACH slow_id WdgSettingsSlow.ID_LIST$
					$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
$						//ウィンドウオープン期間設定
						$IF EQ(WdgSettingsSlow.WdgTriggerMode[slow_id], "WDG_TOGGLE")$
$							//トグルモードの場合，ウィンドウオープン期間は100
							$WdgSettingsSlow.WdgWindowOpenRate[slow_id] = 100$
						$ELIF EQ(WdgSettingsSlow.WdgTriggerMode[slow_id], "WDG_WINDOW")$
$							//ウィンドウモードの場合，ウィンドウオープン期間が設定リストにあるか確認
							$SET_WINDOW_PARAM(WdgSettingsSlow.WdgWindowOpenRate[slow_id])$
						$ELSE$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTriggerMode[slow_id])$$END$$DIE()$
						$END$
$						//タイムアウト時間設定
						$WdgSettingsSlow.WdgTimeout[slow_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsSlow.WdgTimeout[slow_id])$
					$END$
				$END$
			$END$
		$END$
	$ELIF EQ(WdgPublishedInformation.WdgTriggerMode[1], "WDG_TOGGLE")$
		$IF EQ(toggle_allow, "FALSE")$
			$ERROR$$FORMAT(_("WDG_TOGGLE is not valid in this target."))$$END$$DIE()$
		$ELSE$
			$FOREACH config_id WdgSettingsConfig.ID_LIST$
				$FOREACH fast_id WdgSettingsFast.ID_LIST$
					$IF EQ(config_id, WdgSettingsFast.PARENT[fast_id])$
$						//トグルモードの場合，ウィンドウオープン期間は100
						$IF !EQ(WdgSettingsFast.WdgTriggerMode[fast_id], "WDG_TOGGLE")$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTriggerMode[fast_id])$$END$$DIE()$
						$ELSE$
							$WdgSettingsFast.WdgWindowOpenRate[fast_id] = 100$
						$END$
$						//タイムアウト時間設定
						$WdgSettingsFast.WdgTimeout[fast_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsFast.WdgTimeout[fast_id])$
					$END$
				$END$
				$FOREACH slow_id WdgSettingsSlow.ID_LIST$
					$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
$						//トグルモードの場合，ウィンドウオープン期間は100
						$IF !EQ(WdgSettingsSlow.WdgTriggerMode[slow_id], "WDG_TOGGLE")$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTriggerMode[slow_id])$$END$$DIE()$
						$ELSE$
							$WdgSettingsSlow.WdgWindowOpenRate[slow_id] = 100$
						$END$
$						//タイムアウト時間設定
						$WdgSettingsSlow.WdgTimeout[slow_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsSlow.WdgTimeout[slow_id])$
					$END$
				$END$
			$END$
		$END$
	$ELIF EQ(WdgPublishedInformation.WdgTriggerMode[1], "WDG_WINDOW")$
		$IF EQ(window_allow, "FALSE")$
			$ERROR$$FORMAT(_("WDG_WINDOW is not valid in this target."))$$END$$DIE()$
		$ELSE$
			$FOREACH config_id WdgSettingsConfig.ID_LIST$
				$FOREACH fast_id WdgSettingsFast.ID_LIST$
					$IF EQ(config_id, WdgSettingsFast.PARENT[fast_id])$
						$IF !EQ(WdgSettingsFast.WdgTriggerMode[fast_id], "WDG_WINDOW")$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTriggerMode[fast_id])$$END$$DIE()$
						$ELSE$
$							//ウィンドウモードの場合，ウィンドウオープン期間が設定リストにあるか確認
							$SET_WINDOW_PARAM(WdgSettingsFast.WdgWindowOpenRate[fast_id])$
$							//タイムアウト時間設定
							$WdgSettingsFast.WdgTimeout[fast_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsFast.WdgTimeout[fast_id])$
						$END$
					$END$
				$END$
				$FOREACH slow_id WdgSettingsSlow.ID_LIST$
					$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
						$IF !EQ(WdgSettingsSlow.WdgTriggerMode[slow_id], "WDG_WINDOW")$
							$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTriggerMode[slow_id])$$END$$DIE()$
						$ELSE$
$							//ウィンドウモードの場合，ウィンドウオープン期間が設定リストにあるか確認
							$SET_WINDOW_PARAM(WdgSettingsSlow.WdgWindowOpenRate[slow_id])$
$							//タイムアウト時間設定
							$WdgSettingsSlow.WdgTimeout[slow_id] = SET_HW_TIMEOUT_PARAM(WdgSettingsSlow.WdgTimeout[slow_id])$
						$END$
					$END$
				$END$
			$END$
		$END$
	$ELSE$
		$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgPublishedInformation.WdgTriggerMode[1])$$END$$DIE()$
	$END$

$	//ランエリア設定チェック
	$IF EQ(target_run_area_allow, "FALSE")$
		$WARNING$$FORMAT(_("WdgRunArea is not valid in this target."))$$END$
	$ELSE$
		$IF (!EQ(WdgGeneral.WdgRunArea[1], "ROM") && !EQ(WdgGeneral.WdgRunArea[1], "RAM"))$
			$ERROR$$FORMAT(_("%1% must be ROM or RAM."), WdgGeneral.WdgRunArea[1])$$END$$DIE()$
		$END$
	$END$

$	//トリガロケーション設定チェック
	$IF EQ(trigger_location_allow, "FALSE")$
		$WARNING$$FORMAT(_("WdgTriggerLocation is not valid in this target."))$$END$
	$END$

$	//リセット or NMI設定チェック
	$FOREACH config_id WdgSettingsConfig.ID_LIST$
		$FOREACH fast_id WdgSettingsFast.ID_LIST$
			$IF EQ(config_id, WdgSettingsFast.PARENT[fast_id])$
				$IF EQ(WdgSettingsFast.WdgTimeoutReaction[fast_id], "WDG_NMI")$
					$IF EQ(nmi_allow, "FALSE")$
						$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTimeoutReaction[fast_id])$$END$$DIE()$
					$END$
				$ELIF EQ(WdgSettingsFast.WdgTimeoutReaction[fast_id], "WDG_RESET")$
					$IF EQ(reset_allow, "FALSE")$
						$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTimeoutReaction[fast_id])$$END$$DIE()$
					$END$
				$ELSE$
					$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsFast.WdgTimeoutReaction[fast_id])$$END$$DIE()$
				$END$
			$END$
		$END$
		$FOREACH slow_id WdgSettingsSlow.ID_LIST$
			$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
				$IF EQ(WdgSettingsSlow.WdgTimeoutReaction[slow_id], "WDG_NMI")$
					$IF EQ(nmi_allow, "FALSE")$
						$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTimeoutReaction[slow_id])$$END$$DIE()$
					$END$
				$ELIF EQ(WdgSettingsSlow.WdgTimeoutReaction[slow_id], "WDG_RESET")$
					$IF EQ(reset_allow, "FALSE")$
						$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTimeoutReaction[slow_id])$$END$$DIE()$
					$END$
				$ELSE$
					$ERROR$$FORMAT(_("%1% is not valid in this target."), WdgSettingsSlow.WdgTimeoutReaction[slow_id])$$END$$DIE()$
				$END$
			$END$
		$END$
	$END$

$	//[WDG173] 範囲チェック
	$init_num = DIV_FLOAT_VALUE(WdgGeneral.WdgInitialTimeout[1], "0.001", "WdgInitialTimeout", "Constant Value")$
	$max_num = DIV_FLOAT_VALUE(WdgGeneral.WdgMaxTimeout[1], "0.001", "WdgMaxTimeout", "Constant Value")$
	$IF init_num > max_num$
		$ERROR$$FORMAT(_("WdgInitialTimeout must be not larger than WdgMaxTimeout."))$$END$$DIE()$
	$END$

	$IF (init_num < 0) || (init_num > 65535)$
		$ERROR$$FORMAT(_("WdgInitialTimeout is illegal range."))$$END$$DIE()$
	$END$

	$IF (max_num < 0) || (max_num > 65535)$
		$ERROR$$FORMAT(_("WdgMaxTimeout is illegal range."))$$END$$DIE()$
	$END$

	$FOREACH config_id WdgSettingsConfig.ID_LIST$
$	//[WDG025] ウォッチドッグ無効化不許可でデフォルトモードがOFF_MODEの場合，エラー
		$IF (EQ(WdgSettingsConfig.WdgDefaultMode[config_id], "WDGIF_OFF_MODE") && EQ(disable_allow, "FALSE"))$
			$ERROR$$FORMAT(_("Disabling the watchdog driver at runtime is not allowed. (DEM_ERROR WDG_E_DISABLE_REJECTED)"))$$END$$DIE()$
		$ELIF (!EQ(WdgSettingsConfig.WdgDefaultMode[config_id], "WDGIF_OFF_MODE") && !EQ(WdgSettingsConfig.WdgDefaultMode[config_id], "WDGIF_FAST_MODE") && !EQ(WdgSettingsConfig.WdgDefaultMode[config_id], "WDGIF_SLOW_MODE"))$
			$ERROR$$FORMAT(_("WdgDefaultMode is illegal value."))$$END$$DIE()$
		$END$

$	//モード切り替え無効対応
		$IF EQ(mode_set_allow, "TRUE")$
			$mode_container_num = 2$
			$slow_confirmation = "FALSE"$
			$IF LENGTH(WdgSettingsSlow.ID_LIST) > 0$
				$FOREACH slow_id WdgSettingsSlow.ID_LIST$
					$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
						$slow_confirmation = "TRUE"$
					$END$
				$END$
				$IF EQ(slow_confirmation, "FALSE")$
					$ERROR$$FORMAT(_("WdgSlowMode setting is necessary."))$$END$$DIE()$
				$END$
			$END$
		$ELSE$
			$mode_container_num = 1$
$			//デフォルトモード補正
			$IF !EQ(WdgSettingsConfig.WdgDefaultMode[config_id], "WDGIF_FAST_MODE")$
				$WdgSettingsConfig.WdgDefaultMode[config_id] = "WDGIF_FAST_MODE"$
				$WARNING$$FORMAT(_("This target can be set only WDGIF_FAST_MODE. DefaultMode set to WDGIF_FAST_MODE"))$$END$
			$END$
			$slow_confirmation = "FALSE"$
			$FOREACH slow_id WdgSettingsSlow.ID_LIST$
				$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
					$slow_confirmation = "TRUE"$
				$END$
			$END$
			$IF EQ(slow_confirmation, "TRUE")$
				$WARNING$$FORMAT(_("WdgSlowMode setting is ignored."))$$END$$DIE()$
			$END$
		$END$
	$END$



$END$

$ ====================================================================
$  プレコンパイル用ヘッダ生成関数
$ ====================================================================
$FUNCTION GEN_WDG_CFG_H$
	$FILE "Wdg_Cfg.h"$
	/* Wdg_Cfg.h */$NL$

	#ifndef TOPPERS_A_WDG_CFG_H$NL$
	#define TOPPERS_A_WDG_CFG_H$NL$
	$NL$
$	// [WDG073_Conf] Wdg
$	// WdgDemEventParameterRefs 
$	// [WDG150_Conf] WDG_E_DISABLE_REJECTED
	#define WDG_DEM_DISABLE_REJECTED$TAB$
	$IF LENGTH(WdgDemEventParameterRefs.WDG_E_DISABLE_REJECTED[1])$
		$DemEventParameter.DemEventId[WdgDemEventParameterRefs.WDG_E_DISABLE_REJECTED[1]]$U
	$ELSE$
		WDG_INVALID_UINT16	/* [NWDG003] 無効 */
	$END$
	$NL$
$	// [WDG149_Conf] WDG_E_MODE_FAILED
	#define WDG_DEM_MODE_FAILED$TAB$$TAB$$TAB$
$	// [WDG148_Conf] WdgDemEventParameterRefs
	$IF LENGTH(WdgDemEventParameterRefs.WDG_E_MODE_FAILED[1])$
		$DemEventParameter.DemEventId[WdgDemEventParameterRefs.WDG_E_MODE_FAILED[1]]$U
	$ELSE$
		WDG_INVALID_UINT16	/* [NWDG003] 無効 */
	$END$
	$NL$
$	// [WDG156]
$	// [WDG114_Conf] WdgGeneral 
$	// [WDG115_Conf] WdgDevErrorDetect
	#define WDG_DEV_ERROR_DETECT$TAB$$TAB$$IS_TRUE(WdgGeneral.WdgDevErrorDetect[1])$$NL$
$	// [WDG116_Conf] WdgDisableAllowed
	#define WDG_DISABLE_ALLOWED$TAB$$TAB$$TAB$$IS_TRUE(WdgGeneral.WdgDisableAllowed[1])$$NL$
$	// [WDG117_Conf] WdgIndex
$	// [WDG130_Conf] WdgInitialTimeout
$	// 秒をミリ秒に変換する．掛け算関数がないため，DIV_FLOAT_VALUE関数で代用している
	$num1 = DIV_FLOAT_VALUE(WdgGeneral.WdgInitialTimeout[1], "0.001", "WdgInitialTimeout", "Constant Value")$
	#define WDG_INIT_TIMEOUT$TAB$$TAB$$TAB$$num1$U$NL$
$	// [WDG131_Conf] WdgMaxTimeout
	$num2 = DIV_FLOAT_VALUE(WdgGeneral.WdgMaxTimeout[1], "0.001", "WdgMaxTimeout", "Constant Value")$
	#define WDG_MAX_TIMEOUT$TAB$$TAB$$TAB$$TAB$$num2$U$NL$
$	// [WDG147_Conf] WdgRunArea
$	// [WDG118_Conf] WdgTriggerLocation
$	// [WDG119_Conf] WdgVersionInfoApi
	#define WDG_VERSION_INFO_API$TAB$$TAB$$IS_TRUE(WdgGeneral.WdgDisableAllowed[1])$$NL$
$	// モード切り替え許可判別
	#define WDG_SET_MODE_VALID$TAB$$TAB$$TAB$$mode_set_allow$$NL$
	$NL$
	#endif /* TOPPERS_A_WDG_CFG_H */
$	// [WDG074_Conf] WdgPublishedInformation
$	// [WDG127_Conf] WdgTriggerMode

$END$

$ ====================================================================
$  [WDG079] ポストビルド用ヘッダ生成関数
$ ====================================================================
$FUNCTION GEN_WDG_PB_CFG_H$
	$FILE "Wdg_PBcfg.h"$

	/* Wdg_PBcfg.h */$NL$

	#ifndef TOPPERS_A_WDG_PB_CFG_H$NL$
	#define TOPPERS_A_WDG_PB_CFG_H$NL$
	$NL$

	$FOREACH config_id WdgSettingsConfig.ID_LIST$
		extern const Wdg_ConfigType wdg_config_$config_id$;$NL$
	$END$
	$NL$
	#endif /* TOPPERS_A_WDG_PB_CFG_H */
$END$

$ ====================================================================
$  ポストビルド用実体生成関数
$ ====================================================================
$FUNCTION GEN_WDG_PB_CFG_C$
	$FILE "Wdg_PBcfg.c"$

	/* Wdg_PBcfg.c */$NL$
	$NL$

	#include "Os.h"$NL$
	#include "Wdg.h"$NL$
	#include "Wdg_PBcfg.h"$NL$
	#include "MemMap.h"$NL$
	$NL$

	/* トリガ管理ブロック : wdg_config_<WdgSettingConfig>_trigger */$NL$
	$FOREACH config_id WdgSettingsConfig.ID_LIST$
		static TRIGGER_CB wdg_config_$config_id$_trigger;$NL$
	$END$
	$NL$

$	// [WDG082_Conf][NWDG001] WdgSettingsConfig
	$FOREACH config_id WdgSettingsConfig.ID_LIST$
$		// [WDG121_Conf] WdgSettingsFast
		$FOREACH fast_id WdgSettingsFast.ID_LIST$
			$IF EQ(config_id, WdgSettingsFast.PARENT[fast_id])$
				/* WdgFastモード初期化ブロック : wdg_config_<WdgSettingConfig>_fast_mode_inib */$NL$
				static const WDG_MODE_INIB wdg_config_$config_id$_fast_mode_inib = {$NL$
					$TAB$$WdgSettingsFast.WdgTimeoutReaction[fast_id]$, /* [NWDG002] WdgTimeoutReaction */$NL$
					$TAB$$WdgSettingsFast.WdgWindowOpenRate[fast_id]$U, /* [NWDG002] WdgWindowOpenRate */$NL$
					$TAB$$WdgSettingsFast.WdgTimeout[fast_id]$U, /* [NWDG002] timeout */$NL$
					$num1 = DIV_FLOAT_VALUE(WdgSettingsFast.WdgTriggerInterruptPeriod[fast_id], "0.001", "WdgTriggerInterruptPeriod", "Constant Value")$
					$TAB$$num1$U /* [NWDG002] interrupt_period */$NL$
				};$NL$
				$NL$
			$END$
		$END$
$		// [WDG123_Conf][NWDG008] WdgSettingsSlow
		$IF EQ(mode_set_allow, "TRUE")$
			$FOREACH slow_id WdgSettingsSlow.ID_LIST$
				$IF EQ(config_id, WdgSettingsSlow.PARENT[slow_id])$
					/* WdgSlowモード初期化ブロック : wdg_config_<WdgSettingConfig>_slow_mode_inib */$NL$
					static const WDG_MODE_INIB wdg_config_$config_id$_slow_mode_inib = {$NL$
						$TAB$$WdgSettingsSlow.WdgTimeoutReaction[slow_id]$, /* [NWDG002] WdgTimeoutReaction */$NL$
						$TAB$$WdgSettingsSlow.WdgWindowOpenRate[slow_id]$U, /* [NWDG002] WdgWindowOpenRate */$NL$
						$TAB$$WdgSettingsSlow.WdgTimeout[slow_id]$U, /* [NWDG002] timeout */$NL$
						$num1 = DIV_FLOAT_VALUE(WdgSettingsSlow.WdgTriggerInterruptPeriod[slow_id], "0.001", "WdgTriggerInterruptPeriod", "Constant Value")$
						$TAB$$num1$U /* [NWDG002] interrupt_period */$NL$
					};$NL$
					$NL$
				$END$
			$END$
		$END$		
	$END$
	
$	// [WDG082_Conf][NWDG001] WdgSettingsConfig
	$FOREACH config_id WdgSettingsConfig.ID_LIST$
		/* Wdgコンフィギュレーション情報 : wdg_config_<WdgSettingConfig> */$NL$
		const Wdg_ConfigType wdg_config_$config_id$ = {$NL$
$		// [WDG120_Conf][NWDG008] WdgDefaultMode
		$TAB$$WdgSettingsConfig.WdgDefaultMode[config_id]$, /* WdgDefaultMode */$NL$
		$TAB$&wdg_config_$config_id$_fast_mode_inib, /* p_fast_mode_inib */$NL$
		$IF EQ(mode_set_allow, "TRUE") && (LENGTH(WdgSettingsSlow.ID_LIST) > 0)$
			$TAB$&wdg_config_$config_id$_slow_mode_inib, /* p_slow_mode_inib */$NL$
		$ELSE$
			$TAB$NULL_PTR, /* p_slow_mode_inib */$NL$
		$END$
		$TAB$&wdg_config_$config_id$_trigger /* p_trigger_cb */$NL$
		};$NL$
	$END$
$END$

$ ====================================================================
$  メイン処理
$ ====================================================================

$ [WDG086] エラーチェック
$ERROR_CHECK()$

$ [WDG157] プレコンパイル
$GEN_WDG_CFG_H()$

$ [WDG079][WDG159] ポストビルド用プログラム
$GEN_WDG_PB_CFG_H()$
$GEN_WDG_PB_CFG_C()$

$ DUMP()$

$	未サポート仕様タグ一覧
$	- DIO
$	  <WDG112_Conf><WDG113_Conf>

