/*
 *  TOPPERS/A-WDGIF
 *      Automotive Watchdog Interface
 *
 *  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2015-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2017 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgIf.c 3467 2017-03-07 06:43:26Z suzuki-kawaguchi $
 */

/**
* @file WdgIf.c
* @brief WdgIfの実装ファイル
* - WdgIfは一切の内部状態を持たないため，デバッグの際に参照する情報はない【IWDGIF001】〔WDGIF052〕〔WDGIF053〕〔WDGIF054〕〔WDGIF055〕
* - 本ファイルはDETエラーチェックを実施する場合，またはWdgIf_GetVersionInfoを利用する場合必要となる【IWDGIF002】〔WDGIF051〕
* - WdgのインタフェースへはWdgIf_WdgList.hを介してアクセスする【IWDGIF003】〔DWDGIF010〕〔DWDGIF013〕
*/
#include "WdgIf.h"
#include "WdgIf_WdgList.h"
#include "MemMap.h"
#include "SchM_WdgIf.h"

/**
* @defgroup det_proc DETマクロ
* @brief WDGM_DEV_ERROR_DETECTによってマクロの定義を変え，拡張エラーを検出するかどうかを変える【IWDGIF004】〔WDGIF007〕〔WDGIF031〕〔WDGIF032〕〔WDGIF048〕
*/
/*@{*/
#if WDGIF_DEV_ERROR_DETECT == STD_ON
#include "Det.h"

#define WDGIF_CHECK_PARAM(exp, ApiId, ErrorId)										\
	do {																			\
		if (!(exp)) {																\
			(void) Det_ReportError(WDGIF_MODULE_ID, 0x00U, (ApiId), (ErrorId));		\
			goto error_exit;														\
		}																			\
	} while (0)

#define WDGIF_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)									\
	do {																			\
		if (!(exp)) {																\
			ercd = E_NOT_OK;														\
			(void) Det_ReportError(WDGIF_MODULE_ID, 0x00U, (ApiId), (ErrorId));		\
			goto error_exit;														\
		}																			\
	} while (0)
#else /* WDGIF_DEV_ERROR_DETECT == STD_OFF */
#define WDGIF_CHECK_PARAM(exp, ApiId, ErrorId)
#define WDGIF_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)
#endif /* WDGIF_DEV_ERROR_DETECT = STD_ON */
/*@}*/


#if ( WDGIF_DEV_ERROR_DETECT == STD_ON ) && ( WDGIF_DEVICE_NUM > 1U )
/**
* @brief システムサービス：WdgIf_SetMode【IWDGIF005】〔WDGIF042〕
* - WDGIF_DEV_ERROR_DETECTがSTD_ONの場合は，WdgIf_SetModeを関数として実装する【IWDGIF006】〔DWDGIF005〕
*
* @param[in] DeviceIndex デバイスの番号
* @param[in] WdgMode Wdgのモード
* @return なし
*/
Std_ReturnType
WdgIf_SetMode(uint8 DeviceIndex, WdgIf_ModeType WdgMode)
{
	Std_ReturnType ercd;

	/** - 引数DeviceIndexが不正な場合，DETにWDGIF_E_PARAM_DEVICEをレポートして終了する【IWDGIF007】〔WDGIF028〕 */
	WDGIF_CHECK_PARAM_ERCD((DeviceIndex < WDGIF_DEVICE_NUM), WdgIfServiceId_SetMode, WDGIF_E_PARAM_DEVICE);

	/** - DeviceIndex以外のエラーチェックは実施しない【IWDGIF008】〔WDGIF009〕 */

	/** 
	* - 引数WdgModeを直接Wdg_SetModeに与えてコールする【IWDGIF009】〔WDGIF003〕〔WDGIF016〕〔WDGIF026〕〔WDGIF043〕
	* - Wdg_SetModeの返り値をそのまま本関数の返り値とする【IWDGIF010】〔WDGIF003〕〔WDGIF057〕〔WDGIF026〕
	*/
	ercd = WdgIf_SetModeList[DeviceIndex](WdgMode);

  error_exit:
	return(ercd);
}

/**
* @brief システムサービス：WdgIf_SetTriggerCondition【IWDGIF011】〔WDGIF044〕
* - WDGIF_DEV_ERROR_DETECTがSTD_ONの場合は，WdgIf_SetTriggerConditionを関数として実装する【IWDGIF012】〔DWDGIF005〕
*
* @param[in] DeviceIndex デバイスの番号
* @param[in] Timeout Wdgのトリガコンディション
* @return なし
*/
void
WdgIf_SetTriggerCondition(uint8 DeviceIndex, uint16 Timeout)
{

	/** - 引数DeviceIndexが不正な場合，DETにWDGIF_E_PARAM_DEVICEをレポートして終了する【IWDGIF013】〔WDGIF028〕 */
	WDGIF_CHECK_PARAM((DeviceIndex < WDGIF_DEVICE_NUM), WdgIfServiceId_SetTriggerCondition, WDGIF_E_PARAM_DEVICE);

	/** - DeviceIndex以外のエラーチェックは実施しない【IWDGIF014】〔WDGIF009〕 */

	/**
	* - 引数Timeoutを直接Wdg_SetTriggerConditionに与えてコールする【IWDGIF015】〔WDGIF003〕〔WDGIF045〕〔WDGIF026〕
	*/
	WdgIf_SetTriggerConditionList[DeviceIndex](Timeout);

  error_exit:
	return;
}
#endif

/**
* @brief システムサービス：WdgIf_GetVersionInfo【IWDGIF016】〔WDGIF046〕
* - WdgM_GetVersionInfoはWDGM_VERSION_INFO_APIがSTD_ONのときのみ有効である【IWDGIF017】〔WDGIF036〕〔DWDGIF006〕
*
* @param[out] VersionInfo バージョン情報を格納する領域
* @return なし
*/
#if WDGIF_VERSION_INFO_API != STD_OFF
void
WdgIf_GetVersionInfo(Std_VersionInfoType *VersionInfoPtr)
{
	/** - 引数VersionInfoがNULL_PTRである場合，DETにWDGIF_E_INV_POINTERをレポートして終了する【IWDGIF018】〔WDGIF058〕 */
	WDGIF_CHECK_PARAM(VersionInfoPtr != NULL_PTR, WdgIfServiceId_GetVersionInfo, WDGIF_E_INV_POINTER);

	/** - バージョン情報を取得する【IWDGIF019】〔WDGIF035〕 */
	VersionInfoPtr->vendorID = WDGIF_VENDOR_ID;
	VersionInfoPtr->moduleID = WDGIF_MODULE_ID;
	VersionInfoPtr->sw_major_version = WDGIF_SW_MAJOR_VERSION;
	VersionInfoPtr->sw_minor_version = WDGIF_SW_MINOR_VERSION;
	VersionInfoPtr->sw_patch_version = WDGIF_SW_PATCH_VERSION;

  error_exit:
	return;
}
#endif /* WDGIF_VERSION_INFO_API == STD_ON */
