/*
 *  TOPPERS/A-WDGIF
 *      AuTomotive Watchdog Interface
 *
 *  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2015-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2017 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgIf.h 3486 2017-03-09 07:17:36Z suzuki-kawaguchi $
 */

/**
* @file WdgIf.h
* @brief WdgIfのヘッダファイル
*/

#ifndef WDGIF_H
#define WDGIF_H

#include "WdgIf_Cfg.h" 
#include "WdgIf_Types.h"

/**
* @defgroup define_public_info BSW標準公開情報
* @brief BSW標準公開情報を記載する【PWDGIF001】〔WDGIF034〕
*/
/*@{*/
/* バージョン情報 */
#define WDGIF_AR_RELEASE_MAJOR_VERSION		4U
#define WDGIF_AR_RELEASE_MINOR_VERSION		0U
#define WDGIF_AR_RELEASE_REVISION_VERSION	3U
/* サプライヤバージョン情報 */
#define WDGIF_SW_MAJOR_VERSION				1U
#define WDGIF_SW_MINOR_VERSION				2U
#define WDGIF_SW_PATCH_VERSION				0U
#define WDGIF_VENDOR_ID						65U	/* NCESベンダID */
/* WDGIFモジュールID */
#define WDGIF_MODULE_ID						0x2bU
/*@}*/

/**
* @defgroup define_dev_erorr 拡張エラーコード
* @brief 拡張エラーコードを1Byteのunsigned型で定義【PWDGIF002】〔WDGIF006〕〔WDGIF030〕
*/
/*@{*/
#define WDGIF_E_PARAM_DEVICE				0x01U
#define WDGIF_E_INV_POINTER					0x02U
/*@}*/

/**
* @defgroup define_api_id システムサービスID
* @brief 各システムサービスのIDを1Byteのunsigned型で定義
*/
/*@{*/
#define WdgIfServiceId_SetMode				0x01U
#define WdgIfServiceId_SetTriggerCondition	0x02U
#define WdgIfServiceId_GetVersionInfo		0x03U
/*@}*/

/*
 * WdgIfシステムサービスプロトタイプ宣言
 */
extern void WdgIf_GetVersionInfo(Std_VersionInfoType *VersionInfoPtr);

#if WDGIF_DEVICE_NUM == 1U
extern const WdgIf_SetModeType WdgIf_SetModeList[WDGIF_DEVICE_NUM];
extern const WdgIf_SetTriggerConditionType WdgIf_SetTriggerConditionList[WDGIF_DEVICE_NUM];
/** 
* - WDGIF_DEV_ERROR_DETECTがSTD_OFFの場合，WdgIf_SetModeをマクロで実装する【PWDGIF003】〔WDGIF001〕〔WDGIF020〕〔DWDGIF003〕
* - デバイスの個数が1つのとき，DeviceIndexは無視する【PWDGIF004】〔WDGIF018〕
*/
#define WdgIf_SetMode(DeviceIndex, WdgMode) WdgIf_SetModeList[0](WdgMode)
/**
* - WDGIF_DEV_ERROR_DETECTがSTD_OFFの場合，WdgIf_SetTriggerConditionをマクロで実装する【PWDGIF005】〔WDGIF001〕〔WDGIF020〕〔DWDGIF003〕
* - デバイスの個数が1つのとき，DeviceIndexは無視する【PWDGIF006】〔WDGIF018〕
*/
#define WdgIf_SetTriggerCondition(DeviceIndex, Timeout) WdgIf_SetTriggerConditionList[0](Timeout)

#else

#if WDGIF_DEV_ERROR_DETECT == STD_ON
extern Std_ReturnType WdgIf_SetMode(uint8 DeviceIndex, WdgIf_ModeType WdgMode);
extern void WdgIf_SetTriggerCondition(uint8 DeviceIndex, uint16 Timeout);
#else /* WDGIF_DEV_ERROR_DETECT == STD_OFF */
extern const WdgIf_SetModeType WdgIf_SetModeList[WDGIF_DEVICE_NUM];
extern const WdgIf_SetTriggerConditionType WdgIf_SetTriggerConditionList[WDGIF_DEVICE_NUM];
/** WDGIF_DEV_ERROR_DETECTがSTD_OFFの場合，WdgIf_SetModeをマクロで実装する【PWDGIF007】〔WDGIF001〕〔WDGIF020〕〔DWDGIF003〕 */
#define WdgIf_SetMode(DeviceIndex, WdgMode) WdgIf_SetModeList[DeviceIndex](WdgMode)
/** WDGIF_DEV_ERROR_DETECTがSTD_OFFの場合，WdgIf_SetTriggerConditionをマクロで実装する【PWDGIF008】〔WDGIF001〕〔WDGIF020〕〔DWDGIF003〕 */
#define WdgIf_SetTriggerCondition(DeviceIndex, Timeout) WdgIf_SetTriggerConditionList[DeviceIndex](Timeout)
#endif

#endif

#endif /* WDGIF_H */
