/*
 *  TOPPERS/A-WDGM
 *      Automotive Watchdog Manager
 *
 *  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2015-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2017 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgM.c 3465 2017-03-07 06:18:20Z suzuki-kawaguchi $
 */

/**
* @file WdgM.c
* @brief WdgMの実装ファイル
* - DEMをインクルード【IWDGM001】〔WDGM126〕
* - WDGM_DEV_ERROR_DETECTがSTD_ONのときのみDet.hをインクルード【IWDGM002】〔DWDGM074〕
* - WDGM_IMMEDIATE_RESETがSTD_ONのときのみMcu.hをインクルード【IWDGM003】〔DWDGM076〕
* - WDGM_OSAP_SUPPORTがSTD_ONのときのみBswM_WdgM.hをインクルード【IWDGM004】〔DWDGM080〕
*/
#include "WdgM.h"
#include "Rte_WdgM.h"
#include "SchM_WdgM.h"
#include "MemMap.h"
#include "WdgM_NonInitialized_RAM.h"
#include "Dem.h"

#if WDGM_DEV_ERROR_DETECT == STD_ON
#include "Det.h"
#endif

#if WDGM_IMMEDIATE_RESET == STD_ON
#include "Mcu.h"
#endif

#if WDGM_OSAP_SUPPORT == STD_ON
#include "BswM_WdgM.h"
#endif



/*
 * ローカルで使用するマクロ
 */

/**
* @defgroup det_proc DETマクロ
* @brief WDGM_DEV_ERROR_DETECTによってマクロの定義を変え，拡張エラーを検出するかどうかを変える【IWDGM005】〔DWDGM005〕〔DWDGM075〕
*/
/*@{*/
#if WDGM_DEV_ERROR_DETECT == STD_ON
/** @brief 初期化済み判定マクロ【IWDGM006】〔DWDGM038〕 */
#define WDGM_CHECK_INITIALIZED(ApiId)												\
	do {																			\
		if (WdgM_GlobalStatus.global_status == WDGM_GLOBAL_STATUS_DEACTIVATED) {	\
			(void) Det_ReportError(WDGM_MODULE_ID, 0x00U, (ApiId), WDGM_E_NO_INIT);	\
			goto error_exit;														\
		}																			\
	} while (0)

#define WDGM_CHECK_PARAM(exp, ApiId, ErrorId)										\
	do {																			\
		if (!(exp)) {																\
			(void) Det_ReportError(WDGM_MODULE_ID, 0x00U, (ApiId), (ErrorId));		\
			goto error_exit;														\
		}																			\
	} while (0)

/** @brief 未初期化判定マクロ【IWDGM007】〔DWDGM038〕 */
#define WDGM_CHECK_UNITIALIZED(ApiId)													\
	do {																				\
		if (WdgM_GlobalStatus.global_status != WDGM_GLOBAL_STATUS_DEACTIVATED) {		\
			goto error_exit;															\
		}																				\
	} while (0)

/** @brief 同時にコールされてはならないシステムサービスをブロックするフラグ */
static boolean					concurrently_called_block = FALSE;
#define WDGM_CHECK_AND_LOCK_CONCURRENTLY_CALLED(ApiId)								\
	do {																			\
		if (concurrently_called_block != FALSE) {									\
			(void) Det_ReportError(WDGM_MODULE_ID, 0x00U, (ApiId), WDGM_E_CONCURRENTLY_CALLED);		\
			goto error_exit;														\
		}																			\
		concurrently_called_block = TRUE;											\
	} while (0)
#define WDGM_UNLOCK_CONCURRENTLY_CALLED()											\
	do {																			\
		concurrently_called_block = FALSE;											\
	} while (0)

#else /* WDGM_DEV_ERROR_DETECT == FALSE */
#define WDGM_CHECK_INITIALIZED(ApiId)
#define WDGM_CHECK_PARAM(exp, ApiId, ErrorId)
#define WDGM_CHECK_UNITIALIZED(ApiId)
#define WDGM_CHECK_AND_LOCK_CONCURRENTLY_CALLED(ApiId)
#define WDGM_UNLOCK_CONCURRENTLY_CALLED()
#endif /* WDGM_DEV_ERROR_DETECT == TRUE */
/*@}*/

/**
* @defgroup dem_proc DEMマクロ
* @brief DEM発行手続きの冗長コード簡略化【IWDGM008】〔DWDGM005〕〔WDGM004〕
* - DemEventStatusはDEM_EVENT_STATUS_FAILEDとして報告する【IWDGM009】〔NWDGM043〕
*/
/*@{*/
#define DEM_REPORT_IMPROPER_CALLER()																\
	do {																							\
		if (p_cur_wdgm_config->p_dem_inib != NULL_PTR) {											\
			if((p_cur_wdgm_config->p_dem_inib->WDGM_E_IMPROPER_CALLER) != NULL_PTR) {				\
				Dem_ReportErrorStatus(*(p_cur_wdgm_config->p_dem_inib->WDGM_E_IMPROPER_CALLER), DEM_EVENT_STATUS_FAILED);		\
			}																						\
		}																							\
	} while (0)

#define DEM_REPORT_STOPPED_SUPERVISION()															\
	do {																							\
		if (p_cur_wdgm_config->p_dem_inib != NULL_PTR) {											\
			if((p_cur_wdgm_config->p_dem_inib->WDGM_E_SUPERVISION) != NULL_PTR) {					\
				Dem_ReportErrorStatus(*(p_cur_wdgm_config->p_dem_inib->WDGM_E_SUPERVISION), DEM_EVENT_STATUS_FAILED);			\
			}																						\
		}																							\
	} while (0)

#define DEM_REPORT_SET_MODE()																		\
	do {																							\
		if (p_cur_wdgm_config->p_dem_inib != NULL_PTR) {											\
			if((p_cur_wdgm_config->p_dem_inib->WDGM_E_SET_MODE) != NULL_PTR) {						\
				Dem_ReportErrorStatus(*(p_cur_wdgm_config->p_dem_inib->WDGM_E_SET_MODE), DEM_EVENT_STATUS_FAILED);				\
			}																						\
		}																							\
	} while (0)

#define DEM_REPORT_MFCYCLE_OR_OSCOUNTER()															\
	do {																							\
		if (p_cur_wdgm_config->p_dem_inib != NULL_PTR) {											\
			if((p_cur_wdgm_config->p_dem_inib->WDGM_E_MFCYCLE_OR_OSCOUNTER) != NULL_PTR) {			\
				Dem_ReportErrorStatus(*(p_cur_wdgm_config->p_dem_inib->WDGM_E_MFCYCLE_OR_OSCOUNTER), DEM_EVENT_STATUS_FAILED);	\
			}																						\
		}																							\
	} while (0)

#define DEM_REPORT_OSCOUNTER()																		\
	do {																							\
		if (p_cur_wdgm_config->p_dem_inib != NULL_PTR) {											\
			if((p_cur_wdgm_config->p_dem_inib->WDGM_E_OSCOUNTER) != NULL_PTR) {						\
				Dem_ReportErrorStatus(*(p_cur_wdgm_config->p_dem_inib->WDGM_E_OSCOUNTER), DEM_EVENT_STATUS_FAILED);	\
			}																						\
		}																							\
	} while (0)
/*@}*/

/**
* @defgroup get_info 情報取得マクロ
* @brief 頻繁に取得するポインタの冗長コード簡略化【IWDGM010】〔DWDGM005〕
*/
/*@{*/
#define GET_MODE(Index)						(&p_cur_wdgm_config->p_mode_inib_list[(Index)])
#define GET_SE(SEID)						(WdgM_WdgMGeneral_WdgMSupervisedEntity_List[SEID])
#define GET_CP(SEID, CPID)					(&p_cur_wdgm_config->p_mode_inib_list[current_mode].pp_cp_inib_table[(SEID)][(CPID)])
/*@}*/

/**
* @brief WdgM_PerformReset後，無効となるシステムサービスのチェック処理【IWDGM011】〔DWDGM005〕
*/
#define WDGM_CHECK_UNPERFORMRESET()														\
	do {																				\
		if (prohibited_set_trigger_condition == TRUE) {									\
			goto error_exit;															\
		}																				\
	} while (0)

/**
 * @brief バイナリサーチ関数【IWDGM012】〔DWDGM005〕
 * - mid, left, rightが宣言されていること
 */
#define BINARY_SEARCH(list, tnum_list, search_val, result, mid, left, right)			\
	do {																				\
		(result) = FALSE;																\
		(left) = 0U;																	\
		(right) = (tnum_list) - 1U;														\
		if (((search_val) <= (list)[(right)]) && ((search_val) >= (list)[(left)])) {	\
			while ((left) <= (right)) {													\
				(mid) = ((left) + (right)) >> 1U;										\
				if ((search_val) == (list)[(mid)]) {									\
					(result) = TRUE;													\
					break;																\
				}																		\
				else if ((list)[(mid)] < (search_val)) {								\
					(left) = (mid) + 1U;												\
				}																		\
				else {																	\
					(right) = (mid) - 1U;												\
				}																		\
			}																			\
		}																				\
	} while (0)


/**
* @defgroup os_counter OSカウンタアクセスマクロ
* @brief OSカウンタの返り値が異常な場合，DEMを発行してグローバル監視状態をWDGM_GLOBAL_STATUS_STOPPEDにする【IWDGM013】〔NWDGM093〕〔DWDGM005〕
* - タイムスタンプの取得するだけの場合，GetOsCounterを利用する【IWDGM014】〔NWDGM087〕
*/
/*@{*/
#define GET_COUNTER_VALUE_MACRO(CounterID, Value, ercd)				\
	do {															\
		Std_ReturnType	ret;								\
		ret = GetCounterValue(CounterID, Value);			\
		if (ret != E_OK) {									\
			DEM_REPORT_OSCOUNTER();									\
			(ercd) = E_NOT_OK;										\
			set_global_status(WDGM_GLOBAL_STATUS_STOPPED);			\
			goto error_exit;										\
		}															\
	} while (0)

#define GET_ELAPSED_VALUE_MACRO(CounterID, Value, ElapsedValue, ercd)	\
	do {																\
		Std_ReturnType	ret;									\
		ret = GetElapsedValue(CounterID, Value, ElapsedValue);	\
		if (ret != E_OK) {										\
			DEM_REPORT_OSCOUNTER();										\
			(ercd) = E_NOT_OK;											\
			set_global_status(WDGM_GLOBAL_STATUS_STOPPED);				\
			goto error_exit;											\
		}																\
	} while (0)
/*@}*/


/**
 *  内部関数のプロトタイプ宣言
 */
/**
* @defgroup extern_api プロトタイプ宣言
* @brief プロトタイプ宣言【IWDGM015】〔DWDGM005〕
*/
/*@{*/
LOCAL_INLINE WdgM_LocalStatusType change_local_status(void);
LOCAL_INLINE void change_global_status(const WdgM_LocalStatusType worst_local_status);
LOCAL_INLINE void deadline_supervision(const WdgM_SEInitBlock *p_se_inib, const WdgM_CheckpointInitBlock *p_cp_inib, const TickType timestamp);
LOCAL_INLINE Std_ReturnType check_caller_id(uint16 CallerID);
LOCAL_INLINE Std_ReturnType mf_cycle_oscounter_supervision(void);
LOCAL_INLINE Std_ReturnType deadline_timeout(void);
LOCAL_INLINE TickType elapsed_timestamp(const TickType max, const TickType start, TickType end);
LOCAL_INLINE void ex_logical_supervision(const WdgM_SupervisedEntityIdType SEID, const WdgM_CheckpointIdType CheckpointID);
LOCAL_INLINE void in_logical_supervision(const WdgM_SupervisedEntityIdType SEID, const WdgM_CheckpointIdType CheckpointID);
LOCAL_INLINE void set_trigger_condition_correct(const WdgM_ModeInitBlock *p_mode_inib);
LOCAL_INLINE void set_trigger_condition_incorrect(const WdgM_ModeInitBlock *p_mode_inib);
LOCAL_INLINE void set_local_status(const WdgM_SEInitBlock *p_se_inib, const WdgM_LocalStatusType local_status);
LOCAL_INLINE void set_global_status(const WdgM_GlobalStatusType global_status);
LOCAL_INLINE Std_ReturnType set_mode_core(const WdgM_ModeInitBlock *p_mode_inib);
LOCAL_INLINE void clear_local_status(WdgM_LocalStatusControlBlock *p_local_status_cb);
LOCAL_INLINE void clear_in_logical(WdgM_InternalLogicalControlBlock *p_in_logical_cb);
LOCAL_INLINE void clear_mode_dependent_var(const WdgM_ModeInitBlock *p_mode_inib);
/*@}*/

/** @brief 現在実行中のWdgMコンフィギュレーション情報【IWDGM016】〔DWDGM006〕 */
static const WdgM_ConfigType	*p_cur_wdgm_config = NULL_PTR;

/** @brief 現在のWdgMモード【IWDGM017】〔DWDGM006〕 */
static WdgM_ModeType			current_mode = 0U;

/** 
* @brief グローバル監視状態【IWDGM018】〔DWDGM006〕〔DWDGM027〕〔DWDGM028〕
* - グローバル監視状態はコンフィグレーションに依存せず1つであるため，.cファイル内で1つだけ定義する【IWDGM019】〔WDGM213〕
* - デバッグにグローバル監視状態を利用する際はWdgM_GlobalStatusを参照することで可能【IWDGM020】〔WDGM239_1〕
* - global_statusをWDGM_GLOBAL_STATUS_DEACTIVATEDで初期化【IWDGM021】〔PWDGM007〕
* - expired_cntを0で初期化【IWDGM022】〔PWDGM009〕〔DWDGM018〕
*/
WdgM_GlobalStatusControlBlock			WdgM_GlobalStatus = { WDGM_GLOBAL_STATUS_DEACTIVATED, 0U };

/** 
* @brief トリガコンディション設定禁止フラグ【IWDGM023】〔DWDGM006〕 
* - FALSEで初期化【IWDGM024】〔DWDGM013〕
*/
static boolean					prohibited_set_trigger_condition = FALSE;

#if WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON
/** @brief MFコール周期/OSカウンタ監視の実施フラグ【IWDGM025】〔DWDGM006〕 */
static boolean					mf_oscounter_active_flg = FALSE;
#endif /* WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON */

/** - WDGM_DEFENSIVE_BEHAVIORがSTD_ONである場合のみ，WdgM_WdgMGeneral_WdgMCallerIds_WdgMCallerIdを宣言する【IWDGM026】〔DWDGM083〕*/
#if (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U)
/** @brief コーラーIDリスト(実態はWdgM_Cfg.c) */
extern const uint16 WdgM_WdgMGeneral_WdgMCallerIds_WdgMCallerId[WDGM_CALLER_ID_NUM];
#endif /* (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U) */

/** @brief 監視エンティティリスト(実態はWdgM_Cfg.c) */
extern const WdgM_SEInitBlock * const WdgM_WdgMGeneral_WdgMSupervisedEntity_List[WDGM_MAX_SEID + 1U];

/** @brief セレクタブル確認用コンフィグリスト(実態はWdgM_PBcfg.c) */
extern const WdgM_ConfigType *WdgM_WdgMConfigSetList[WDGM_CONFIG_NUM];

/**
* @brief システムサービス：WdgM_Init【IWDGM027】〔WDGM151〕
* @param[in] ConfigPtr WdgMConfigSet以下に設定されたポストビルドパラメータ情報
* @return なし
*/
void
WdgM_Init(const WdgM_ConfigType *ConfigPtr)
{
	Std_ReturnType						ercd;
	boolean								config_check;
	uint16								i;
	WdgM_SupervisedEntityIdType			se_index;
	const WdgM_ModeInitBlock			*p_initialmode_inib;
	WdgM_AliveControlBlock				init_WdgM_AliveControlBlock = { 0 };
	WdgM_DeadlineControlBlock			init_WdgM_DeadlineControlBlock = { 0 };
	WdgM_ExternalLogicalControlBlock	init_WdgM_ExternalLogicalControlBlock = { 0 };

	/** - 未初期化状態でない場合，なにもせずに終了する */
	WDGM_CHECK_UNITIALIZED(WdgMServiceId_Init);

	/** - 引数ConfigPtrがNULL_PTRである場合，DETにWDGM_E_INV_POINTERをレポートして終了する【IWDGM028】〔WDGM255〕 */
	WDGM_CHECK_PARAM((ConfigPtr != NULL_PTR), WdgMServiceId_Init, WDGM_E_INV_POINTER);

	/** 
	* - 引数ConfigPtrがWdgM_PBcfgで定義されたものであることをチェックし，
	*   そうでなかった場合，DETにWDGM_E_PARAM_CONFIGをレポートする【IWDGM029】〔WDGM010〕〔NWDGM052〕
	*/
	config_check = FALSE;
	for (i = 0U; i < WDGM_CONFIG_NUM; i++) {
		if (ConfigPtr == WdgM_WdgMConfigSetList[i]) {
			config_check = TRUE;
			break;
		}
	}
	WDGM_CHECK_PARAM(config_check, WdgMServiceId_Init, WDGM_E_PARAM_CONFIG);


	/** - 現在のコンフィグレーション情報を保持 */
	p_cur_wdgm_config = ConfigPtr;

	/** - バッファを一括初期化【IWDGM030】〔WDGM018〕 */
	for (i = 0U; i < p_cur_wdgm_config->max_size_WdgM_AliveControlBlock; i++) {
		p_cur_wdgm_config->p_buffer_WdgM_AliveControlBlock[i] = init_WdgM_AliveControlBlock;
	}
	for (i = 0U; i < p_cur_wdgm_config->max_size_WdgM_DeadlineControlBlock; i++) {
		p_cur_wdgm_config->p_buffer_WdgM_DeadlineControlBlock[i] = init_WdgM_DeadlineControlBlock;
	}
	for (i = 0U; i < p_cur_wdgm_config->max_size_WdgM_ExternalLogicalControlBlock; i++) {
		p_cur_wdgm_config->p_buffer_WdgM_ExternalLogicalControlBlock[i] = init_WdgM_ExternalLogicalControlBlock;
	}
	for (i = 0U; i < p_cur_wdgm_config->max_size_TickType; i++) {
		p_cur_wdgm_config->p_buffer_TickType[i] = 0U;
	}
	for (i = 0U; i <= WDGM_MAX_SEID; i++) {
		if (WdgM_WdgMGeneral_WdgMSupervisedEntity_List[i] != NULL_PTR) {
			clear_local_status(WdgM_WdgMGeneral_WdgMSupervisedEntity_List[i]->p_local_status_cb);
			clear_in_logical(WdgM_WdgMGeneral_WdgMSupervisedEntity_List[i]->p_in_logical_cb);
		}
	}

	/** - 初期モードを確立する【IWDGM031】〔WDGM135〕 */
	current_mode = p_cur_wdgm_config->initial_mode_id;
	p_initialmode_inib = GET_MODE(p_cur_wdgm_config->initial_mode_id);
	ercd = set_mode_core(p_initialmode_inib);

	/** - モード設定が正常に完了したら以下の処理を実施 */
	if (ercd == E_OK) {

		/** - 監視エンティティが初期モードでアクティブな場合ローカル監視状態をOKにする【IWDGM032】〔WDGM268〕〔PWDGM013〕 */
		for (se_index = 0U; se_index < p_initialmode_inib->tnum_active_se; se_index++) {
			set_local_status(p_initialmode_inib->p_active_se_inib_list[se_index].p_se_inib, WDGM_LOCAL_STATUS_OK);
		}

		/** - 監視エンティティが初期モードで非アクティブな場合ローカル監視状態をDEACTIVATEDにする【IWDGM033】〔WDGM269〕〔PWDGM014〕 */
		for (se_index = 0U; se_index < p_initialmode_inib->tnum_deactive_se; se_index++) {
			set_local_status(p_initialmode_inib->pp_deactive_se_inib_list[se_index], WDGM_LOCAL_STATUS_DEACTIVATED);
		}

		/** - 初期モードで有効な監視機能を初期化する */
		clear_mode_dependent_var(p_initialmode_inib);

		/** - FirstExpiredSEIDを操作する間，WdgM_CriticalSection_FirstExpiredSEIDで排他する【IWDGM034】〔DWDGM037〕 */
		SchM_Enter_WdgM_CriticalSection_FirstExpiredSEID();
		/** - FirstExpiredSEIDとその反転値を初期化する【IWDGM035】〔WDGM350〕 */
		WdgM_ClearFirstExpiredSEID();
		SchM_Exit_WdgM_CriticalSection_FirstExpiredSEID();

		/** - 最後にグローバル監視状態をOKで更新する 【IWDGM036】〔WDGM285〕〔DWDGM039〕 */
		set_global_status(WDGM_GLOBAL_STATUS_OK);

		/** - WdgM_InitではWdgM_GlobalStatus.expired_cntを初期化する【IWDGM037】〔DWDGM019〕 */
		WdgM_GlobalStatus.expired_cnt = 0U;

		/** - WdgM_Initではprohibited_set_trigger_conditionを初期化する【IWDGM038】〔DWDGM017〕 */
		prohibited_set_trigger_condition = FALSE;
	}
error_exit:
	return;
}

/**
* @brief システムサービス：WdgM_DeInit【IWDGM039】〔WDGM261〕
* @return なし
*/
void
WdgM_DeInit(void)
{
	/** WdgM_DeInitではWdgM_CriticalSection_Supervisionによる排他を実施しない【IWDGM040】〔DWDGM041〕 */

	/**
	* - 同時動作不可能なシステムサービスが動作中でないかチェックし，
	*   動作中でなければ自分自身が動作するためにロックする【IWDGM041】〔NWDGM102〕
	*/
	WDGM_CHECK_AND_LOCK_CONCURRENTLY_CALLED(WdgMServiceId_DeInit);

	/* API同時コール検出試験用 */
#ifdef TWDGM_TEST_MODE
	concurrently_call();
#endif

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートして終了する【IWDGM042】〔WDGM288〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_DeInit);

	/** - WdgM_PerformResetが呼ばれている場合，なにもせず終了する【IWDGM043】〔NWDGM070〕 */
	WDGM_CHECK_UNPERFORMRESET();

	/** - 現在のグローバル監視状態がWDGM_GLOBAL_STATUS_OKである場合は，WDGM_GLOBAL_STATUS_DEACTIVATEDに遷移する【IWDGM044】〔WDGM286〕 */
	if (WdgM_GlobalStatus.global_status == WDGM_GLOBAL_STATUS_OK) {
		set_global_status(WDGM_GLOBAL_STATUS_DEACTIVATED);
	}

	/** - WDGM_GLOBAL_STATUS_DEACTIVATEDに遷移するタイミングではトリガコンディション設定を実施しない【IWDGM045】〔WDGM292〕〔NWDGM001〕 */

error_exit:
	/** - ロック解除【IWDGM046】〔NWDGM102〕 */
	WDGM_UNLOCK_CONCURRENTLY_CALLED();

	return;
}

/**
* @brief システムサービス：WdgM_GetVersionInfo【IWDGM047】〔WDGM153〕
* - WdgM_GetVersionInfoはマクロで実装可能であるが，
*   可読性，引数の型チェック，Det.hのインクルード等を優先して関数として実装する【IWDGM048】〔WDGM262〕
*  - マクロによる高速化のメリットは小さい
*  - WdgM_GetVersionInfoがコールされない場合，コードサイズはWDGM_VERSION_INFO_APIをSTD_OFFにすることで削減可能
* - WdgM_GetVersionInfoはWDGM_VERSION_INFO_APIがSTD_ONのときのみ有効である【IWDGM049】〔WDGM110〕
*
* @param[out] VersionInfo バージョン情報を格納する領域
* @return なし
*/
#if WDGM_VERSION_INFO_API == STD_ON
void
WdgM_GetVersionInfo(Std_VersionInfoType *VersionInfo)
{
	/** - 引数VersionInfoがNULL_PTRである場合，DETにWDGM_E_INV_POINTERをレポートする【IWDGM050】〔WDGM256〕 */
	WDGM_CHECK_PARAM(VersionInfo != NULL_PTR, WdgMServiceId_GetVersionInfo, WDGM_E_INV_POINTER);

	/** - バージョン情報を取得する */
	VersionInfo->vendorID = WDGM_VENDOR_ID;
	VersionInfo->moduleID = WDGM_MODULE_ID;
	VersionInfo->sw_major_version = WDGM_SW_MAJOR_VERSION;
	VersionInfo->sw_minor_version = WDGM_SW_MINOR_VERSION;
	VersionInfo->sw_patch_version = WDGM_SW_PATCH_VERSION;

error_exit:
	return;
}
#endif /* WDGM_VERSION_INFO_API */

/**
* @brief システムサービス：WdgM_SetMode【IWDGM051】〔WDGM154〕
* @param[in] Mode モード切替後のモードID
* @param[in] CallerID 呼び出し元のID
* @return E_OK：モード切替成功
* @return E_NOT_OK：モード切替失敗
*/
Std_ReturnType
WdgM_SetMode(WdgM_ModeType Mode, uint16 CallerID)
{
	Std_ReturnType					ercd = E_NOT_OK;
	WdgM_SupervisedEntityIdType		se_index;
	const WdgM_ModeInitBlock		*p_next_mode_inib;
	const WdgM_SEInitBlock			*p_se_inib;
	WdgM_LocalStatusControlBlock	*p_local_status_cb;

	/**
	* - 同時動作不可能なシステムサービスが動作中でないかチェックし，
	*   動作中でなければ自分自身が動作するためにロックする【IWDGM052】〔NWDGM103〕
	*/
	WDGM_CHECK_AND_LOCK_CONCURRENTLY_CALLED(WdgMServiceId_SetMode);

	/* API同時コール検出試験用 */
#ifdef TWDGM_TEST_MODE
	concurrently_call();
#endif

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートする【IWDGM053】〔WDGM021〕〔WDGM142_2〕〔DWDGM040〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_SetMode);

	/** - 引数Modeが異常な場合，DETにWDGM_E_PARAM_MODEをレポートする【IWDGM054】〔WDGM020〕〔WDGM142_2〕 */
	WDGM_CHECK_PARAM((Mode < p_cur_wdgm_config->tnum_mode), WdgMServiceId_SetMode, WDGM_E_PARAM_MODE);

	/** - WdgM_PerformResetが呼ばれている場合，なにもせず終了する【IWDGM055】〔NWDGM071〕 */
	WDGM_CHECK_UNPERFORMRESET();

	/** - WDGM_DEFENSIVE_BEHAVIORがSTD_ONである場合，CallerIDのチェックを行う【IWDGM056】〔DWDGM083〕〔DWDGM084〕*/
#if (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U)
	ercd = check_caller_id(CallerID);
	if (ercd == FALSE) {
		goto error_exit;
	}
#endif /* (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U) */


	/** モード切替処理を行う前から，モード切替処理の完了までをWdgM_CriticalSection_Supervisionで排他する【IWDGM057】〔DWDGM042〕 */
	SchM_Enter_WdgM_CriticalSection_Supervision();

	/** - グローバル監視状態がWDGM_GLOBAL_STATUS_OK or WDGM_GLOBAL_STATUS_FAILEDの場合，
	*     モード切替を実行する【IWDGM058】〔WDGM145〕〔WDGM316〕*/
	if ((WdgM_GlobalStatus.global_status == WDGM_GLOBAL_STATUS_OK) ||
		(WdgM_GlobalStatus.global_status == WDGM_GLOBAL_STATUS_FAILED)) {
		p_next_mode_inib = GET_MODE(Mode);
		ercd = set_mode_core(p_next_mode_inib);

		if (ercd == E_OK) {
			/* WdgM_CheckpointReachedと排他する */

			/** - 設定されたモードを開始する前に監視機能が管理する変数をクリア【IWDGM059】〔NWDGM086〕〔NWDGM085〕 */
			clear_mode_dependent_var(p_next_mode_inib);

			/** - モードが変化しない場合，監視エンティティのアクティブ/非アクティブは切り替わらないため処理を飛ばす【IWDGM060】〔WDGM182〕 */
			if (current_mode != Mode) {
				/**
				* - 非アクティブからアクティブになる監視エンティティのローカル監視状態をWDGM_LOCAL_STATUS_OKに遷移する【IWDGM061】〔WDGM209〕
				* - アクティブからアクティブになる監視エンティティにはなにもしない【IWDGM062】〔WDGM182〕
				*/
				for (se_index = 0U; se_index < p_next_mode_inib->tnum_active_se; se_index++) {
					p_se_inib = p_next_mode_inib->p_active_se_inib_list[se_index].p_se_inib;
					p_local_status_cb = p_se_inib->p_local_status_cb;

					if (p_local_status_cb->local_status == WDGM_LOCAL_STATUS_DEACTIVATED) {
						set_local_status(p_se_inib, WDGM_LOCAL_STATUS_OK);
					}
				}

				/**
				* - アクティブから非アクティブになる監視エンティティのローカル監視状態を
				*   WDGM_LOCAL_STATUS_DEACTIVATEDに遷移する【IWDGM063】〔WDGM207〕〔WDGM291〕
				* - アクティブからアクティブになる監視エンティティにはなにもしない【IWDGM064】〔WDGM182〕
				*/
				for (se_index = 0U; se_index < p_next_mode_inib->tnum_deactive_se; se_index++) {
					p_se_inib = p_next_mode_inib->pp_deactive_se_inib_list[se_index];
					p_local_status_cb = p_se_inib->p_local_status_cb;

					if ((p_local_status_cb->local_status == WDGM_LOCAL_STATUS_OK) ||
						(p_local_status_cb->local_status == WDGM_LOCAL_STATUS_FAILED)) {

						/** - アクティブから非アクティブになるになるとき，各監視結果やカウンタをクリアする【IWDGM065】〔WDGM315〕 */
						clear_local_status(p_local_status_cb);/* ローカル監視状態管理ブロックをクリア */
						clear_in_logical(p_se_inib->p_in_logical_cb);/* 内部ロジカル監視をクリア */

						set_local_status(p_se_inib, WDGM_LOCAL_STATUS_DEACTIVATED);
					}
				}
				/** - 新しいモードが完全にアクティブになってから，すなわち一番最後に現在のモードを更新する【IWDGM066】〔WDGM170〕 */
				current_mode = Mode;
			}
		}
	}
	else {
		ercd = E_NOT_OK;
	}

	SchM_Exit_WdgM_CriticalSection_Supervision();


error_exit:
	/** - ロック解除【IWDGM067】〔NWDGM103〕 */
	WDGM_UNLOCK_CONCURRENTLY_CALLED();

	return(ercd);
}

/**
* @brief システムサービス：WdgM_GetMode【IWDGM068】〔WDGM168〕
* @param[out] Mode 現在のモードIDの格納領域
* @return E_OK：モード取得の成功
* @return E_NOT_OK：モード取得の失敗
*/
Std_ReturnType
WdgM_GetMode(WdgM_ModeType *Mode)
{
	Std_ReturnType ercd = E_NOT_OK;

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートする【IWDGM069】〔WDGM253〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_GetMode);
	/** - 引数ModeがNULL_PTRである場合，DETにWDGM_E_PARAM_MODEをレポートする【IWDGM070】〔WDGM254〕 */
	WDGM_CHECK_PARAM(Mode != NULL_PTR, WdgMServiceId_GetMode, WDGM_E_INV_POINTER);

	/** - 現在のWdgMモードを取得する【IWDGM071】〔WDGM170〕 */
	*Mode = current_mode;
	ercd = E_OK;

error_exit:
	return(ercd);
}

/**
* @brief システムサービス：WdgM_CheckpointReached【IWDGM072】〔WDGM263〕
* @param[in] SEID チェックポイント到達を通知する監視エンティティのID
* @param[in] CheckpointID 到達したチェックポイントのID
* @return E_OK：チェックポイント到達時処理成功
* @return E_NOT_OK：チェックポイント到達時処理失敗
*/
Std_ReturnType
WdgM_CheckpointReached(WdgM_SupervisedEntityIdType SEID, WdgM_CheckpointIdType CheckpointID)
{
	Std_ReturnType				ercd = E_NOT_OK;
	uint16						i;
	TickType					timestamp = 0U;
	const WdgM_SEInitBlock		*p_se_inib;
	const WdgM_CheckpointInitBlock		*p_cp_inib;
	const WdgM_ModeInitBlock	*p_mode_inib;
	const WdgM_AliveInitBlock	*p_alive_inib;

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートする【IWDGM073】〔WDGM279〕〔DWDGM040〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_CheckpointReached);

	/** - 監視エンティティが存在しない場合，DETにWDGM_E_PARAM_SEIDをレポートする【IWDGM074】〔WDGM278〕 */
	WDGM_CHECK_PARAM((SEID <= WDGM_MAX_SEID), WdgMServiceId_CheckpointReached, WDGM_E_PARAM_SEID);
	p_se_inib = GET_SE(SEID);
	WDGM_CHECK_PARAM((p_se_inib != NULL_PTR), WdgMServiceId_CheckpointReached, WDGM_E_PARAM_SEID);

	/** - チェックポイントIDが不正な場合，DETにWDGM_E_PARAM_SEIDをレポートする【IWDGM075】〔WDGM284〕 */
	WDGM_CHECK_PARAM((CheckpointID < p_se_inib->tnum_cp), WdgMServiceId_CheckpointReached, WDGM_E_CPID);

	/** - 割込み禁止の場合OSカウンタの機能が利用できなくなるため，排他区間外でタイムスタンプを取得【IWDGM076】〔DWDGM044〕 */
	if (p_se_inib->os_counter != NULL_PTR) {
		GET_COUNTER_VALUE_MACRO(p_se_inib->os_counter->id, &timestamp, ercd);
	}

	/** 監視処理実施前から，監視処理完了までをWdgM_CriticalSection_Supervisionで排他する【IWDGM077】〔DWDGM043〕 */
	SchM_Enter_WdgM_CriticalSection_Supervision();

#if WDGM_DEV_ERROR_DETECT == STD_ON
	/** 
	* - 監視エンティティのローカル監視状態がWDGM_LOCAL_STATUS_DEACTIVATEDである場合，
	*   その監視エンティティは現在のモードでアクティブでないため，
	*   DETにWDGM_E_SEDEACTIVATEDをレポートする【IWDGM078】〔WDGM319〕
	* - ローカル監視状態のチェックは，WdgM_SetModeによる途中の切替を防ぐため，
	*   WdgM_CriticalSection_Supervisionによる排他区間内で行う【IWDGM079】〔DWDGM045〕
	*/
	if (p_se_inib->p_local_status_cb->local_status == WDGM_LOCAL_STATUS_DEACTIVATED) {
		SchM_Exit_WdgM_CriticalSection_Supervision();
		WDGM_CHECK_PARAM(FALSE, WdgMServiceId_CheckpointReached, WDGM_E_SEDEACTIVATED);
	}
#endif

	/** - 現在のモード取得 */
	p_mode_inib = GET_MODE(current_mode);

	/** - チェックポイント情報取得 */
	p_cp_inib = GET_CP(SEID, CheckpointID);

	/* アライブ監視 */
	/** - チェックポイントがアライブ監視を有している場合，ALIVEカウンタをインクリメントする【IWDGM080】〔WDGM321〕 */
	for (i = 0U; i < p_cp_inib->tnum_alive; i++) {
		p_alive_inib = p_cp_inib->pp_alive_inib_list[i];
		/** - アライブカウンタのオーバーフローチェック【IWDGM081】〔DWDGM051〕 */
		if (p_alive_inib->p_alive_cb->alive_cnt < 0xFFFFFFFF) {
			/** - アライブカウンタをインクリメントする【IWDGM082】〔PWDGM024〕 */
			(p_alive_inib->p_alive_cb->alive_cnt)++;
		}
	}

	/** - チェックポイントがいずれかのグラフに所属していた場合，ロジカル監視を実施する【IWDGM083】〔WDGM323〕 */
	if (p_cp_inib->p_ex_logical_inib != NULL_PTR) {
		/* 外部ロジカル監視 */
		ex_logical_supervision(SEID, CheckpointID);
	}
	else if ((p_se_inib->p_in_logical_inib_list != NULL_PTR) &&
		(p_se_inib->p_in_logical_inib_list[CheckpointID].in_logical_type != WDGM_LOGICAL_EXCLUDED)) {
		/* 内部ロジカル監視 */
		in_logical_supervision(SEID, CheckpointID);
	}
	else {
		/** - チェックポイントがいずれのグラフに属していない場合，何もしない【IWDGM084】〔WDGM297〕 */
	}

	/** - チェックポイントがデッドライン監視を有している場合，デッドライン監視を実施する【IWDGM085】〔WDGM322〕 */
	deadline_supervision(p_se_inib, p_cp_inib, timestamp);

	SchM_Exit_WdgM_CriticalSection_Supervision();

	ercd = E_OK;
error_exit:

	return(ercd);
}

/**
* @brief システムサービス：WdgM_GetLocalStatus【IWDGM086】〔WDGM169〕
* @param[in] SEID ローカル監視状態を取得したい監視エンティティのID
* @param[out] Status ローカル監視状態を格納する領域
* @return E_OK：ローカル監視状態の取得成功
* @return E_NOT_OK：ローカル監視状態の取得失敗
*/
Std_ReturnType
WdgM_GetLocalStatus(WdgM_SupervisedEntityIdType SEID, WdgM_LocalStatusType *Status)
{
	Std_ReturnType	ercd = E_NOT_OK;
	const WdgM_SEInitBlock	*p_se_inib;

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートする【IWDGM087】〔WDGM173〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_GetLocalStatus);

	/** - 引数StatusがNULL_PTRである場合，DETにWDGM_E_INV_POINTERをレポートする【IWDGM088】〔WDGM257〕 */
	WDGM_CHECK_PARAM(Status != NULL_PTR, WdgMServiceId_GetLocalStatus, WDGM_E_INV_POINTER);

	/** - 監視エンティティが不正な場合，DETにWDGM_E_PARAM_SEIDをレポートする【IWDGM089】〔WDGM172〕 */
	WDGM_CHECK_PARAM((SEID <= WDGM_MAX_SEID), WdgMServiceId_GetLocalStatus, WDGM_E_PARAM_SEID);
	p_se_inib = GET_SE(SEID);
	WDGM_CHECK_PARAM((p_se_inib != NULL_PTR), WdgMServiceId_GetLocalStatus, WDGM_E_PARAM_SEID);

	/** - 引数SEIDのローカル監視状態を取得する【IWDGM090】〔WDGM171〕 */
	*Status = p_se_inib->p_local_status_cb->local_status;
	ercd = E_OK;

error_exit:
	return(ercd);
}

/**
* @brief システムサービス：WdgM_GetGlobalStatus【IWDGM091】〔WDGM175〕
* @param[out] Status グローバル監視状態を格納する領域
* @return E_OK：グローバル監視状態の取得成功
* @return E_NOT_OK：グローバル監視状態の取得失敗
*/
Std_ReturnType
WdgM_GetGlobalStatus(WdgM_GlobalStatusType *Status)
{
	Std_ReturnType ercd = E_NOT_OK;

	/** - 本関数は初期化より前に実行可能であるため，初期化済み判定を実施しない【IWDGM092】〔NWDGM059〕 */

	/** - 引数StatusがNULL_PTRの場合，DETにWDGM_E_INV_POINTERをレポートする【IWDGM093】〔WDGM258〕〔WDGM344〕*/
	WDGM_CHECK_PARAM(Status != NULL_PTR, WdgMServiceId_GetGlobalStatus, WDGM_E_INV_POINTER);

	/** - 現在のグローバル監視状態を取得する【IWDGM094】〔WDGM175〕 */
	*Status = WdgM_GlobalStatus.global_status;
	ercd = E_OK;

error_exit:
	return(ercd);
}

/**
* @brief システムサービス：WdgM_PerformReset【IWDGM095】〔WDGM264〕
* @return なし
*/
void
WdgM_PerformReset(void)
{
	const WdgM_ModeInitBlock		*p_mode_inib;

	/** - 初期化されていない場合，DETにWDGM_E_INV_POINTERをレポートする【IWDGM096】〔WDGM270〕〔DWDGM040〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_PerformReset);

	/** - WdgM_PerformResetが呼ばれている場合，なにもせず終了する【IWDGM097】〔NWDGM073〕 */
	WDGM_CHECK_UNPERFORMRESET();

	/** 
	* - 現在のモードで有効なWatchog Driverのトリガコンディション設定を0にする【IWDGM098】〔WDGM232〕
	* - set_trigger_condition_incorrectをコールすると以降のトリガコンディション設定が禁止される【IWDGM099】〔WDGM233〕
	*/
	p_mode_inib = GET_MODE(current_mode);
	set_trigger_condition_incorrect(p_mode_inib);

error_exit:
	return;
}

/**
* @brief システムサービス：WdgM_GetFirstExpiredSEID【IWDGM100】〔WDGM346〕
* @param[out] SEID 監視エンティティのIDを格納する領域
* @return E_OK：監視エンティティのIDの取得成功
* @return E_NOT_OK：監視エンティティのIDの取得失敗
*/
Std_ReturnType
WdgM_GetFirstExpiredSEID(WdgM_SupervisedEntityIdType *SEID)
{
	Std_ReturnType ercd = E_NOT_OK;

	/** - 本関数は初期化より前に実行可能であるため，初期化済み判定を実施しない【IWDGM101】〔WDGM348〕 */

	/** - 引数StatusがNULL_PTRの場合，DETにWDGM_E_INV_POINTERをレポートする【IWDGM102】〔WDGM347〕*/
	WDGM_CHECK_PARAM(SEID != NULL_PTR, WdgMServiceId_GetFirstExpiredSEID, WDGM_E_INV_POINTER);

	/** - FirstExpiredSEIDを操作する間，CriticalSection_FirstExpiredSEIDで排他する【IWDGM103】〔DWDGM037〕 */
	SchM_Enter_WdgM_CriticalSection_FirstExpiredSEID();
	/** 別ファイルに定義されたメモリからFirstExpiredSEIDを読み出す【IWDGM104】〔WDGM349〕〔DWDGM009〕 */
	ercd = WdgM_GetFirstExpiredSEIDCore(SEID);
	SchM_Exit_WdgM_CriticalSection_FirstExpiredSEID();

error_exit:
	return(ercd);
}

/**
* @brief 周期関数：WdgM_MainFunction【IWDGM105】〔WDGM159〕
* - 本関数はグローバル監視状態がWDGM_GLOBAL_STATUS_DEACTIVATEDでない場合，WdgMSupervisionCycleの間隔でコールされることを想定して実装する
* - 周期的に実施される処理を本関数内に実装する【IWDGM106】〔WDGM060〕
*
* @return なし
*/
void
WdgM_MainFunction(void)
{
	Std_ReturnType					ercd = E_OK;
	const WdgM_ModeInitBlock		*p_mode_inib;
	/** 最悪ローカル監視状態を定義【IWDGM107】〔DWDGM056〕 */
	WdgM_LocalStatusType			worst_local_status;

	/** 
	* - 同時動作不可能なシステムサービスが動作中でないかチェックし，
	*   動作中でなければ自分自身が動作するためにロックする【IWDGM108】〔NWDGM104〕
	*/
	WDGM_CHECK_AND_LOCK_CONCURRENTLY_CALLED(WdgMServiceId_MainFunction);

	/* API同時コール検出試験用 */
#ifdef TWDGM_TEST_MODE
	concurrently_call();
#endif

	/** - 初期化されていない場合，DETにWDGM_E_NO_INITをレポートする【IWDGM109】〔NWDGM047〕〔DWDGM040〕 */
	WDGM_CHECK_INITIALIZED(WdgMServiceId_MainFunction);

	/** - WdgM_PerformResetが呼ばれている場合，なにもせず終了する【IWDGM110】〔NWDGM074〕 */
	WDGM_CHECK_UNPERFORMRESET();

	/** - WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLEDがSTD_ONのときのみ実施する【IWDGM111】〔DWDGM088〕〔DWDGM089〕〔PWDGM202〕〔PWDGM203〕 */
#if WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON
	/** - WdgM_MainFunctionコール周期/OSカウンタ監視は周期的に実施する【IWDGM112】〔NWDGM094〕 */
	ercd = mf_cycle_oscounter_supervision();
	if (ercd != E_OK) {
		goto error_exit;
	}
#endif /* WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED */

	/** - WDGM_DEADLINE_TIMEOUT_ENABLEDがSTD_ONのときのみ実施する【IWDGM113】〔DWDGM086〕〔DWDGM087〕〔PWDGM204〕〔PWDGM205〕 */
#if WDGM_DEADLINE_TIMEOUT_ENABLED == STD_ON
	/** - デッドライン監視タイムアウト検出は周期的に実施する【IWDGM114】〔NWDGM076〕 */
	ercd = deadline_timeout();
	if (ercd != E_OK) {
		goto error_exit;
	}
#endif /* WDGM_DEADLINE_TIMEOUT_ENABLED */

	/**
	* - ローカル監視状態更新する【IWDGM115】〔WDGM200〕〔WDGM325〕 
	* - change_local_status内部でローカル監視状態の遷移に基づくエラーハンドリングを実施する【IWDGM116】〔WDGM327〕
	*/
	worst_local_status = change_local_status();

	/**
	* - ローカル監視状態に基づきグローバル監視状態更新する【IWDGM117】〔WDGM326〕〔WDGM214〕
	* - change_global_status内部でグローバル監視状態の遷移に基づくエラーハンドリングを実施する【IWDGM118】〔WDGM327〕
	*/
	change_global_status(worst_local_status);

	/** - グローバル監視状態がWDGM_GLOBAL_STATUS_STOPPEDでなければ，トリガコンディション設定を実施する【IWDGM119】〔WDGM328〕〔NWDGM001〕 */
	if (WdgM_GlobalStatus.global_status != WDGM_GLOBAL_STATUS_STOPPED) {
		p_mode_inib = GET_MODE(current_mode);
		set_trigger_condition_correct(p_mode_inib);
	}

error_exit:
	/** - ロック解除【IWDGM120】〔NWDGM104〕*/
	WDGM_UNLOCK_CONCURRENTLY_CALLED();

	return;
}

/**
* @brief 内部関数：WdgIfへのモード切替処理の実施
* @param[in] p_mode_inib 切替後のモード初期化ブロック
* @return E_OK：モード切替成功
* @return E_NOT_OK：モード切替失敗
*/
LOCAL_INLINE Std_ReturnType
set_mode_core(const WdgM_ModeInitBlock *p_mode_inib)
{
	Std_ReturnType					ercd = E_OK;
	const WdgM_TriggerInitBlock		*p_trigger_inib_list;
	uint8							i;

	p_trigger_inib_list = p_mode_inib->p_trigger_inib_list;

	/** - モードで有効なウォッチドッグハードウェアにモードを設定する【IWDGM121】〔WDGM186〕 */
	for (i = 0U; i < p_mode_inib->tnum_trigger; i++) {
		ercd = WdgIf_SetMode(p_trigger_inib_list[i].wdg_id, p_trigger_inib_list[i].WdgMWatchdogMode);
		if (ercd == E_NOT_OK) {
			break;
		}
	}

	if (ercd == E_OK) {
		/** - WdgIf_SetModeが正常に完了したら，トリガコンディション設定を実施する【IWDGM122】〔NWDGM089〕 */
		set_trigger_condition_correct(p_mode_inib);
	}
	else {
		/** - WdgIf_SetModeが失敗したら，DEMにWDGM_E_SET_MODEをレポートする【IWDGM123】〔NWDGM042〕 */
		DEM_REPORT_SET_MODE();
		/** - WdgIf_SetModeが失敗したら，グローバル監視状態をWDGM_GLOBAL_STATUS_STOPPEDに遷移する【IWDGM124】〔WDGM139〕 */
		set_global_status(WDGM_GLOBAL_STATUS_STOPPED);
	}

	return ercd;
}

/**
* @brief 内部関数：コール元のIDチェック
* - 本関数はWdgM_SetModeからのみコールされることを想定している
* - 本関数はWDGM_DEFENSIVE_BEHAVIORがSTD_ONのときのみ有効である
*
* @return E_OK：許可されたコール元である
* @return E_NOT_OK：許可されていないコール元である
*/
#if (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U)
LOCAL_INLINE Std_ReturnType
check_caller_id(uint16 CallerID)
{
	Std_ReturnType					ercd = E_OK;
	boolean							ret_val = TRUE;
	uint8							mid, left, right;

	/**
	* - CallerIDがWdgMCallerIdに含まれているかをチェックを行い，
	*   もし許可されていないCallerIDであればDEMにWDGM_E_IMPROPER_CALLERをレポートし，
	*   返り値をE_NOT_OKとして終了する【IWDGM125】〔NWDGM046〕
	*/
	BINARY_SEARCH(WdgM_WdgMGeneral_WdgMCallerIds_WdgMCallerId, WDGM_CALLER_ID_NUM, CallerID, ret_val, mid, left, right);
	if (ret_val == FALSE) {
		DEM_REPORT_IMPROPER_CALLER();
		ercd = E_NOT_OK;
	}

	return ercd;
}
#endif /* (WDGM_DEFENSIVE_BEHAVIOR == STD_ON) && (WDGM_CALLER_ID_NUM != 0U) */

/**
* @brief 内部関数：WdgM_MainFunctionコール周期/OSカウンタ監視の実施
* - 本関数はWdgM_MainFunctionからのみコールされることを想定している
* - 本関数はWDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLEDがSTD_ONのときのみ有効である
* - 本関数は割込み禁止区間内で使用してはならない
*
* @return E_OK：監視処理成功
* @return E_NOT_OK：監視処理失敗
*/
#if WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON
LOCAL_INLINE Std_ReturnType
mf_cycle_oscounter_supervision(void)
{
	TickType			eval = 0U;
	Std_ReturnType		ercd = E_OK;
	uint16				i;
	const WdgM_ModeInitBlock			*p_mode_inib;

	p_mode_inib = GET_MODE(current_mode);

	/** - p_mf_osounter_inibがNULL_PTRであった場合，なにもせず終了する【IWDGM126】〔NWDGM096〕 */
	if (p_mode_inib->p_mf_osounter_inib != NULL_PTR) {
		/** - mf_oscounter_active_flgがFALSEであった場合，異常判定は実施せずタイムスタンプの更新のみ実施する【IWDGM127】〔NWDGM101〕 */
		if (mf_oscounter_active_flg != FALSE) {
			/** - リストされている全てのOSカウンタからタイムスタンプを取得し，経過時間を計測する【IWDGM128】〔NWDGM097〕 */
			for (i = 0U; i < p_mode_inib->tnum_mf_os_conter; i++) {
				GET_ELAPSED_VALUE_MACRO(p_mode_inib->p_mf_osounter_inib[i].p_os_counter->id, p_mode_inib->p_mf_osounter_inib[i].p_timestamp, &eval, ercd);
				/** 
				* - 経過時間がmax_tickより大きい，またはmin_tickより小さい場合を異常と判定する【IWDGM129】〔NWDGM098〕
				*  - 異常となった原因がWdgM_Mainfunctionの周期，OSカウンタのどちらにあるかは区別しない【IWDGM130】〔NWDGM100〕
				*/
				if ((p_mode_inib->p_mf_osounter_inib->max_tick < eval) || (p_mode_inib->p_mf_osounter_inib->min_tick > eval)) {
					/**
					* - 異常であればDEMにWDGM_E_MFCYCLE_OR_OSCOUNTERをレポートし，
					*   グローバル監視状態をWDGM_GLOBAL_STATUS_STOPPEDに遷移する【IWDGM131】〔NWDGM084〕〔NWDGM099〕
					*/
					DEM_REPORT_MFCYCLE_OR_OSCOUNTER();
					set_global_status(WDGM_GLOBAL_STATUS_STOPPED);
					goto error_exit;
				}
			}
		}
		else {
			/** - mf_oscounter_active_flgをTRUEにする【IWDGM132】〔NWDGM101〕 */
			mf_oscounter_active_flg = TRUE;
			for (i = 0U; i < p_mode_inib->tnum_mf_os_conter; i++) {
				GET_COUNTER_VALUE_MACRO(p_mode_inib->p_mf_osounter_inib[i].p_os_counter->id, p_mode_inib->p_mf_osounter_inib[i].p_timestamp, ercd);
			}
		}
	}
error_exit:
	return ercd;
}
#endif /* WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED */

/**
* @brief 内部関数：デッドライン監視タイムアウト検出の実施
* - 本関数はWdgM_MainFunctionからのみコールされることを想定している
* - 本関数はWDGM_DEADLINE_TIMEOUT_ENABLEDがSTD_ONのときのみ有効である
* - 本関数は割込み禁止区間内で使用してはならない
*
* @return E_OK：監視処理成功
* @return E_NOT_OK：監視処理失敗
*/
#if WDGM_DEADLINE_TIMEOUT_ENABLED == STD_ON
LOCAL_INLINE Std_ReturnType
deadline_timeout(void) {
	Std_ReturnType						ercd = E_OK;
	uint16								i, j;
	TickType							eval = 0U;
	TickType							timestamp = 0U;
	const WdgM_SEInitBlock				*p_se_inib;
	const WdgM_DeadlineInitBlock		*p_deadline_inib;
	const WdgM_ModeInitBlock			*p_mode_inib;

	p_mode_inib = GET_MODE(current_mode);

	/** - デッドライン監視タイムアウト検出機能を利用する監視エンティティでループ【IWDGM133】〔DWDGM052〕 */
	for (i = 0U; i < p_mode_inib->tnum_deadline_timeout_se; i++) {
		p_se_inib = p_mode_inib->p_deadline_timeout_se_inib_list[i].p_se_inib;
		/** - タイムスタンプの取得は排他区間外で実施する【IWDGM134】〔DWDGM048〕 */
		GET_COUNTER_VALUE_MACRO(p_se_inib->os_counter->id, &timestamp, ercd);

		/** - タイムアウト検出機能が有効なデッドライン監視でループ【IWDGM135】〔DWDGM053〕〔DWDGM054〕 */
		for (j = 0U; j < p_mode_inib->p_deadline_timeout_se_inib_list[i].tnum_deadline_timeout; j++) {
			/**
			* - 1つのデッドライン監視の処理はWdgM_CheckpoitReachedによる途中の更新を防ぐため， 
			*   排他区間WdgM_CriticalSection_Supervision内で実施する【IWDGM136】〔DWDGM047〕
			*/
			SchM_Enter_WdgM_CriticalSection_Supervision();
			p_deadline_inib = p_mode_inib->p_deadline_timeout_se_inib_list[i].pp_deadline_inib_list[j];

			/**
			* - デッドライン監視の開始チェックポイントが到達していれば経過時間を判定し，
			*   タイムアウトしていればデッドライン監視の監視結果を異常とする【IWDGM137】〔NWDGM076〕
			*/
			if (p_deadline_inib->p_deadline_cb->cp_reached != FALSE) {
				eval = elapsed_timestamp(p_se_inib->os_counter->OsCounterMaxAllowedValue, p_deadline_inib->p_deadline_cb->timestamp, timestamp);
				/** - タイムアウト判定【IWDGM138】〔DWDGM055〕〔PWDGM041〕 */
				if ((eval > p_deadline_inib->max_tick) && (eval < p_deadline_inib->timeout_limit_tick)) {
					p_deadline_inib->p_deadline_cb->deadline_incorrect = TRUE;
					/** デッドライン監視タイムアウト検出で異常と判定したとき，expired_flgも同時にTRUEにする【IWDGM139】〔PWDGM017〕 */
					p_se_inib->p_local_status_cb->expired_flg = TRUE;
				}
			}
			SchM_Exit_WdgM_CriticalSection_Supervision();
		}
	}

error_exit:
	return ercd;
}
#endif /* WDGM_DEADLINE_TIMEOUT_ENABLED */

/**
* @brief 内部関数：ローカル監視状態の遷移判定実施
* - 現在のモードでアクティブな監視エンティティに対して，ローカル監視状態の遷移判定を実施する
* - 本関数はWdgM_MainFunctionからのみコールされることを想定している
* - 本関数は現在のモードでアクティブな監視エンティティに対してローカル監視状態を判定，遷移し，
*   その中から最も悪いローカル監視状態を返す
*
* @return WDGM_LOCAL_STATUS_OK：全ての監視エンティティのローカル監視状態はWDGM_LOCAL_STATUS_OKである
* @return WDGM_LOCAL_STATUS_FAILED：ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDの監視エンティティはなく，
*                                   WDGM_LOCAL_STATUS_FAILEDの監視エンティティが1つ以上ある
* @return WDGM_LOCAL_STATUS_EXPIRED：WDGM_LOCAL_STATUS_EXPIREDの監視エンティティが1つ以上ある
*/
LOCAL_INLINE WdgM_LocalStatusType
change_local_status(void)
{
	boolean								alive_incorrect;
	uint16								i, j;
	WdgM_LocalStatusControlBlock		*p_local_status_cb;
	const WdgM_AliveInitBlock			*p_alive_inib;
	WdgM_AliveControlBlock				*p_alive_cb;
	WdgM_InternalLogicalControlBlock	*p_in_logical_cb;
	const WdgM_SEInitBlock				*p_se_inib;
	const WdgM_ModeInitBlock			*p_mode_inib;
	WdgM_LocalStatusType				worst_local_status;


	p_mode_inib = GET_MODE(current_mode);

	/** - 最悪ローカル監視状態をWDGM_LOCAL_STATUS_OKに初期化する【IWDGM140】〔DWDGM057〕*/
	worst_local_status = WDGM_LOCAL_STATUS_OK;

	/** - 現在有効な監視エンティティ数でループ【IWDGM141】〔WDGM325〕 */
	for (j = 0U; j < p_mode_inib->tnum_active_se; j++) {
		/**
		* - 同じ監視エンティティに所属するアライブ監視の処理は，
		*   WdgM_CheckpointReachedによる途中の更新を防ぐため，
		*   排他区間WdgM_CriticalSection_Supervision内で実施する【IWDGM142】〔DWDGM046〕
		*/
		SchM_Enter_WdgM_CriticalSection_Supervision();

		/** <br>@b アライブ監視の実施 */
		/** - 監視エンティティに構成されているアライブ監視を実施する【IWDGM143】〔WDGM098〕〔WDGM083〕 */
		alive_incorrect = FALSE;
		for (i = 0U; i < p_mode_inib->p_active_se_inib_list[j].tnum_alive; i++) {
			p_alive_inib = p_mode_inib->p_active_se_inib_list[j].pp_alive_inib_list[i];
			p_alive_cb = p_alive_inib->p_alive_cb;

			/**
			* - アライブ監視はWdgMSupervisionReferenceCycleの周期で実施するため，
			*   supervision_cntをWdgMSupervisionReferenceCycleからデクリメントし，
			*   supervision_cntが0になったときに監視結果の判定を実施する【IWDGM144】〔WDGM098〕〔PWDGM027〕
			*/
			(p_alive_cb->supervision_cnt)--;
			if (p_alive_cb->supervision_cnt == 0U) {
				/** - アライブ通知が許容範囲外である場合，異常と判定する【IWDGM145】〔WDGM115〕〔WDGM324〕〔PWDGM212〕 */
				if ((p_alive_cb->alive_cnt < p_alive_inib->min_eai) ||
					(p_alive_cb->alive_cnt > p_alive_inib->max_eai)) {
					alive_incorrect = TRUE;
					p_alive_cb->alive_incorrect = TRUE;
				}
				/** - アライブ通知が許容範囲内である場合，正常と判定する【IWDGM229】〔PWDGM213〕 */
				else{
					p_alive_cb->alive_incorrect = FALSE;
				}
				/** - アライブ監視の処理が完了したため，alive_cntを0にクリアする【IWDGM146】〔PWDGM025〕 */
				p_alive_cb->alive_cnt = 0U;
				/** - アライブ監視の処理が完了したため，supervision_cntを再設定する【IWDGM147】〔PWDGM028〕 */
				p_alive_cb->supervision_cnt = p_alive_inib->WdgMSupervisionReferenceCycle;
			}
			/** - アライブ監視の監視周期の間はアライブカウンタの検証を行わないため，前回検証結果が異常の場合は異常と判定する【IWDGM230】〔WDGM098〕 */
			else if(p_alive_cb->alive_incorrect == TRUE){
				alive_incorrect = TRUE;
			}
			else {
				;	/** - アライブ監視の監視周期の間はアライブカウンタの検証を行わないため，前回検証結果が正常の場合は処理無し */
			}
		}
		SchM_Exit_WdgM_CriticalSection_Supervision();

		p_se_inib = p_mode_inib->p_active_se_inib_list[j].p_se_inib;
		p_local_status_cb = p_se_inib->p_local_status_cb;
		p_in_logical_cb = p_se_inib->p_in_logical_cb;

		/** <br>@b ローカル監視状態判定＆遷移時アクション実行 */
		switch (p_local_status_cb->local_status) {

		/** <br>@b 遷移元のローカル監視状態がWDGM_LOCAL_STATUS_OKの場合 */
		case WDGM_LOCAL_STATUS_OK:
			/** -  デッドライン監視，またはロジカル監視が失敗していた場合，ローカル監視状態をWDGM_LOCAL_STATUS_EXPIREDに遷移する【IWDGM148】〔WDGM201〕*/
			if ((p_local_status_cb->expired_flg != FALSE) || ((p_in_logical_cb != NULL_PTR) && (p_in_logical_cb->in_logical_incorrect != FALSE))) {
				set_local_status(p_se_inib, WDGM_LOCAL_STATUS_EXPIRED);
			}
			else if (alive_incorrect == TRUE) {
				/** 
				* - アライブ監視に異常があり，WdgMFailedAliveSupervisionRefCycleTolが0であった場合， 
				*   ローカル監視状態をWDGM_LOCAL_STATUS_EXPIREDに遷移する【IWDGM149】〔WDGM201〕
				*/
				if (p_mode_inib->p_active_se_inib_list[j].WdgMFailedAliveSupervisionRefCycleTol == 0U) {
					set_local_status(p_se_inib, WDGM_LOCAL_STATUS_EXPIRED);
				}
				/**
				* - デッドライン監視，ロジカル監視に異常はなく，
				*   アライブ監視に異常があり，WdgMFailedAliveSupervisionRefCycleTolが0でなかった場合，
				*   ローカル監視状態をWDGM_LOCAL_STATUS_FAILEDに遷移し，FAILEDカウンタをインクリメントする【IWDGM150】〔WDGM203〕〔PWDGM019〕
				*/
				else {
					set_local_status(p_se_inib, WDGM_LOCAL_STATUS_FAILED);
					(p_local_status_cb->failed_cnt)++;
				}				
			}
			/* 異常がなかった場合 */
			else {
				/** - 異常が無かった場合，ローカル監視はWDGM_LOCAL_STATUS_OKのままである【IWDGM151】〔WDGM201〕*/
			}
			break;

		/** <br>@b 遷移元のローカル監視状態がWDGM_LOCAL_STATUS_FAILEDの場合 */
		case WDGM_LOCAL_STATUS_FAILED:
			/** -  デッドライン監視，またはロジカル監視が失敗していた場合，ローカル監視状態をWDGM_LOCAL_STATUS_EXPIREDに遷移する【IWDGM152】〔WDGM20４〕*/
			if ((p_local_status_cb->expired_flg != FALSE) || ((p_in_logical_cb != NULL_PTR) && (p_in_logical_cb->in_logical_incorrect != FALSE))) {
				set_local_status(p_se_inib, WDGM_LOCAL_STATUS_EXPIRED);
			}
			else if (alive_incorrect == TRUE) {
				/**
				* - アライブ監視に異常があり，FAILEDカウンタの値がWdgMFailedAliveSupervisionRefCycleTolに達していた場合，
				*   ローカル監視状態をWDGM_LOCAL_STATUS_EXPIREDに遷移する【IWDGM153】〔WDGM206〕
				*/
				if ((p_local_status_cb->failed_cnt) > p_mode_inib->p_active_se_inib_list[j].WdgMFailedAliveSupervisionRefCycleTol) {
					set_local_status(p_se_inib, WDGM_LOCAL_STATUS_EXPIRED);
				}
				/**
				* - デッドライン監視，ロジカル監視に異常はなく，
				*   アライブ監視に異常があり，FAILEDカウンタの値がWdgMFailedAliveSupervisionRefCycleTolに達していなかった場合，
				*   ローカル監視状態をWDGM_LOCAL_STATUS_FAILEDのまま維持し， FAILEDカウンタをインクリメントする【IWDGM154】〔WDGM204〕〔PWDGM019〕
				*/
				else {
					(p_local_status_cb->failed_cnt)++;
				}
			}
			else {
				/** 
				* - 全ての監視に異常は無く，FAILEDカウンタが1である場合，
				*   ローカル監視状態をWDGM_LOCAL_STATUS_OKに遷移し，
				*   FAILEDカウンタをデクリメントする【IWDGM155】〔WDGM205〕〔PWDGM020〕
				*/
				if (p_local_status_cb->failed_cnt == 1U) {					
					set_local_status(p_se_inib, WDGM_LOCAL_STATUS_OK);
				}
				/**
				* - 全ての監視に異常は無く，FAILEDカウンタが1でない場合，
				*   ローカル監視状態をWDGM_LOCAL_STATUS_FAILEDのまま維持し，
				*   FAILEDカウンタをデクリメントする【IWDGM156】〔WDGM300〕〔PWDGM020〕
				*/
				(p_local_status_cb->failed_cnt)--;
			}
			break;

		/** <br>@b 遷移元のローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDの場合 */
		case WDGM_LOCAL_STATUS_EXPIRED:
			/** - ローカル監視状態WDGM_LOCAL_STATUS_EXPIREDからの遷移は存在しない */
			break;

		/** <br>@b 遷移元のローカル監視状態がWDGM_LOCAL_STATUS_DEACTIVATEDの場合 */
		case WDGM_LOCAL_STATUS_DEACTIVATED:
		default:
			/** - ローカル監視状態がWDGM_LOCAL_STATUS_DEACTIVATEDである場合は，WdgM_MainFunctionでは何もしない【IWDGM157】〔WDGM208〕 */
			break;
		}

		/** <br>@b 最悪ローカル監視状態更新 */
		switch (p_local_status_cb->local_status) {
		/** 
		* - 判定したローカル監視状態がWDGM_LOCAL_STATUS_OKの場合，
		*   最悪ローカル監視状態を更新しない【IWDGM158】〔DWDGM058〕
		*/
		case WDGM_LOCAL_STATUS_OK:
			break;

		/** 
		* - 判定したローカル監視状態がWDGM_LOCAL_STATUS_FAILEDの場合，
		*   現在の最悪ローカル監視状態がWDGM_LOCAL_STATUS_OKであれば
		*   WDGM_LOCAL_STATUS_FAILEDに更新する【IWDGM159】〔DWDGM059〕
		*/
		case WDGM_LOCAL_STATUS_FAILED:
			if (worst_local_status == WDGM_LOCAL_STATUS_OK) {
				worst_local_status = WDGM_LOCAL_STATUS_FAILED;
			}
			break;

		/** 
		* - 判定したローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDの場合，
		*   最悪ローカル監視状態をWDGM_LOCAL_STATUS_EXPIREDに更新する【IWDGM160】〔DWDGM060〕
		*/
		case WDGM_LOCAL_STATUS_EXPIRED:
			worst_local_status = WDGM_LOCAL_STATUS_EXPIRED;
			break;

		default:
			break;
		}
	}

	return(worst_local_status);
}

/**
* @brief 内部関数：グローバル監視状態の遷移判定実施
* - 最悪ローカル監視状態に基づいてグローバル監視状態の遷移判定を実施する
* - 本関数はWdgM_MainFunctionからのみコールされることを想定している
*
* @param[in] worst_local_status 最悪ローカル監視状態
* @sa change_local_status
* @return なし
*/
LOCAL_INLINE void
change_global_status(const WdgM_LocalStatusType worst_local_status)
{
	const WdgM_ModeInitBlock			*p_mode_inib;

	p_mode_inib = GET_MODE(current_mode);

	/** <br>@b グローバル監視状態判定＆遷移時アクション実行 */
	switch (WdgM_GlobalStatus.global_status) {

	/** <br>@b 遷移元のグローバル監視状態がWDGM_GLOBAL_STATUS_OK，またはWDGM_GLOBAL_STATUS_FAILEDの場合 */
	case WDGM_GLOBAL_STATUS_OK:
	case WDGM_GLOBAL_STATUS_FAILED:

		if (worst_local_status == WDGM_LOCAL_STATUS_EXPIRED) {
			/**
			* - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであり，
			*   WdgMExpiredSupervisionCycleTolが0であった場合，
			*   グローバル監視状態をWDGM_GLOBAL_STATUS_STOPPEDに遷移する【IWDGM161】〔WDGM117〕〔WDGM216〕
			*  - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであることは，
			*    1つ以上の監視エンティティのローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであることを意味する
			*/
			if (p_mode_inib->WdgMExpiredSupervisionCycleTol == 0U) {
				set_global_status(WDGM_GLOBAL_STATUS_STOPPED);
			}
			/**
			* - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであり，
			*   WdgMExpiredSupervisionCycleTolが0でなかった場合，
			*   グローバル監視状態をWDGM_GLOBAL_STATUS_EXPIREDに遷移し，
			*   EXPIREDカウンタをインクリメントする【IWDGM162】〔NWDGM048〕〔NWDGM105〕〔PWDGM010〕
			*  - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであることは，
			*    1つ以上の監視エンティティのローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDであることを意味する
			*/
			else {
				set_global_status(WDGM_GLOBAL_STATUS_EXPIRED);
				(WdgM_GlobalStatus.expired_cnt)++;
			}
		}
		/**
		* - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_FAILEDであった場合，
		*   グローバル監視状態をWDGM_GLOBAL_STATUS_FAILEDに遷移する【IWDGM163】〔WDGM076〕〔WDGM217〕
		*  - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_FAILEDであることは，
		*    ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDの監視エンティティは1つもなく，
		*    1つ以上の監視エンティティのローカル監視状態がWDGM_LOCAL_STATUS_FAILEDであることを意味する
		*/
		else if (worst_local_status == WDGM_LOCAL_STATUS_FAILED) {
			set_global_status(WDGM_GLOBAL_STATUS_FAILED);
		}
		/**
		* - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_OKであった場合，
		*   グローバル監視状態をWDGM_GLOBAL_STATUS_OKに遷移する【IWDGM164】〔WDGM078〕〔WDGM218〕
		*  - 最悪ローカル監視状態がWDGM_LOCAL_STATUS_OKであることは，
		*    全ての監視エンティティのローカル監視状態がWDGM_LOCAL_STATUS_OKであることを意味する
		*/
		else {
			set_global_status(WDGM_GLOBAL_STATUS_OK);
		}
		break;

	/** <br>@b 遷移元のグローバル監視状態がWDGM_GLOBAL_STATUS_EXPIREDの場合 */
	case WDGM_GLOBAL_STATUS_EXPIRED:

		/** 
		* - EXPIREDカウンタがWdgMExpiredSupervisionCycleTolに達した場合，
		*   グローバル監視状態をWDGM_GLOBAL_STATUS_STOPPEDに遷移する【IWDGM165】〔NWDGM050〕
		*/
		if (p_mode_inib->WdgMExpiredSupervisionCycleTol == WdgM_GlobalStatus.expired_cnt) {
			set_global_status(WDGM_GLOBAL_STATUS_STOPPED);
		}
		/**
		* - EXPIREDカウンタがWdgMExpiredSupervisionCycleTolに達していない場合，
		*   グローバル監視状態をWDGM_GLOBAL_STATUS_EXPIREDのまま維持し，
		*   EXPIREDカウンタをインクリメントする【IWDGM166】〔WDGM239_2〕〔NWDGM049〕
		*/
		else {
			(WdgM_GlobalStatus.expired_cnt)++;
		}
		break;

	case WDGM_GLOBAL_STATUS_STOPPED:
	default:
		/** 
		* - グローバル監視状態がWDGM_GLOBAL_STATUS_STOPPEDの場合，
		*   WDGM_GLOBAL_STATUS_STOPPEDのまま維持し続ける【IWDGM167】〔WDGM221〕
		*/
		break;
	}

	return;
}

/**
* @brief 内部関数：デッドライン監視実施
* @param[in,out] p_se_inib 到達したチェックポイントが所属する監視エンティティ初期化ブロック
* @param[in,out] p_cp_inib 到達したチェックポイント初期化ブロック
* @param[in] timestamp チェックポイントが到達したときのタイムスタンプ
* @return なし
*/
LOCAL_INLINE void
deadline_supervision(const WdgM_SEInitBlock *p_se_inib, const WdgM_CheckpointInitBlock *p_cp_inib, const TickType timestamp)
{
	uint16							i;
	TickType						eval = 0U;
	const WdgM_DeadlineInitBlock	*p_deadline_inib;
	WdgM_DeadlineControlBlock		*p_deadline_cb;

	/** <br>@b 開始チェックポイント到達時処理 */
	/** - チェックポイントにデッドライン監視が設定されていない場合， tnum_deadline_startは0であるため無視する【IWDGM168】〔WDGM299〕 */
	for (i = 0U; i < p_cp_inib->tnum_deadline_start; i++) {
		p_deadline_inib = p_cp_inib->pp_deadline_inib_start_list[i];
		p_deadline_cb = p_deadline_inib->p_deadline_cb;

		/** - デッドライン開始チェックポイントに到達した場合，タイムスタンプを更新する【IWDGM169】〔WDGM228〕〔PWDGM035〕 */
		p_deadline_cb->timestamp = timestamp;

		/** 開始チェックポイント到達済みに設定する【IWDGM170】〔NWDGM054〕〔PWDGM037〕 */
		p_deadline_cb->cp_reached = TRUE;
	}

	/** <br>@b 終了チェックポイント到達時処理 */
	/** - チェックポイントにデッドライン監視が設定されていない場合， tnum_deadline_endは0であるため無視する【IWDGM171】〔WDGM299〕 */
	for (i = 0U; i < p_cp_inib->tnum_deadline_end; i++) {
		p_deadline_inib = p_cp_inib->pp_deadline_inib_end_list[i];
		p_deadline_cb = p_deadline_inib->p_deadline_cb;

		/** - 開始チェックポイント到達フラグがTRUEのとき，タイムスタンプの差を計算し，開始チェックポイント到達フラグをFALSEにする【IWDGM172】〔NWDGM055〕〔PWDGM038〕 */
		if (p_deadline_cb->cp_reached != FALSE) {
			eval = elapsed_timestamp(p_se_inib->os_counter->OsCounterMaxAllowedValue, p_deadline_cb->timestamp, timestamp);

			/** - WdgMDeadlineMin ≦ 経過時間(eval) ≦ WdgMDeadlineMax の範囲内であれば正常，そうでなければ異常とする【IWDGM173】〔WDGM294〕〔PWDGM040〕 */
			if ((eval > p_deadline_inib->max_tick) || (eval < p_deadline_inib->min_tick)) {
				p_deadline_cb->deadline_incorrect = TRUE;
				/** デッドライン監視で異常と判定したとき，expired_flgも同時にTRUEにする【IWDGM174】〔PWDGM017〕 */
				p_se_inib->p_local_status_cb->expired_flg = TRUE;
			}

			p_deadline_cb->cp_reached = FALSE;
		}
	}

	return;
}

/**
* @brief 内部関数：タイムスタンプ差計算
* @param[in] max タイムスタンプを取得したOSカウンタのOsCounterMaxAllowedValue
* @param[in] start タイムスタンプの始まり
* @param[in] end タイムスタンプの終わり
* @return タイムスタンプの差
*/
LOCAL_INLINE TickType
elapsed_timestamp(const TickType max, const TickType start, TickType end)
{
	TickType ret;

	/** - end ≧ startであれば，通常の引き算【IWDGM175】〔DWDGM061〕 */
	if (end >= start) {
		ret = end - start;
	}
	/** - end < startであれば，OSカウンタが一周する境なので，差にmax + 1を足す【IWDGM176】〔DWDGM062〕 */
	else {
		ret = (end - start) + (max + 1U);
	}

	return ret;
}


/**
* @brief 内部関数：外部ロジカル監視実施
* @param[in] SEID 到達したチェックポイントの所属する監視エンティティのID
* @param[in] CheckpointID 到達したチェックポイントのID
* @return なし
*/
LOCAL_INLINE void
ex_logical_supervision(const WdgM_SupervisedEntityIdType SEID, const WdgM_CheckpointIdType CheckpointID)
{
	boolean								ret_val;
	const WdgM_SEInitBlock				*p_se_inib;
	const WdgM_CheckpointInitBlock		*p_cp_inib;
	const WdgM_ExternalLogicalInitBlock	*p_ex_logical_dest_inib;
	const WdgM_ExternalLogicalInitBlock	*p_ex_logical_inib;
	const WdgM_CheckpointIdType			*p_ex_trans_dest_cp_id_list;
	WdgM_ExternalLogicalControlBlock	*p_ex_logical_cb;
	uint16								mid, left, right;

	p_se_inib = GET_SE(SEID);
	p_cp_inib = GET_CP(SEID, CheckpointID);

	/** 
	* - チェックポイント初期化ブロックには1つのグラフが設定されているので，
	*   その初期化ブロックを所属しているグラフと認識して処理する【IWDGM177】〔WDGM295〕
	*  - チェックポイントがどのグラフにも所属していない場合，p_ex_logical_inibはNULL_PTRであるが，
	*    本関数をコールする時点でp_ex_logical_inibがNULL_PTRでないことを確認している
	*/
	p_ex_logical_inib = p_cp_inib->p_ex_logical_inib;
	p_ex_logical_cb = p_ex_logical_inib->p_ex_logical_cb;

	/** <br>@b 始端チェックポイント未到達時処理 */
	if (p_ex_logical_cb->active_flg == FALSE) {
		/**
		* - アクティブフラグがFALSEのとき，到達したチェックポイントがWdgMExternalCheckpointInitialRefから参照されるものであれば正常
		*   そうでなければ異常とする【IWDGM178】〔WDGM252〕
		*  - WDGM_LOGICAL_INITであれば，チェックポイントがWdgMExternalCheckpointInitialRefから参照されることを意味する
		*/
		if (p_ex_logical_inib->ex_logical_type == WDGM_LOGICAL_INIT) {
			/** - アクティブフラグをTRUEにする【IWDGM179】〔WDGM332〕〔PWDGM050〕 */
			p_ex_logical_cb->active_flg = TRUE;
			ret_val = TRUE;
		}
		else {
			ret_val = FALSE;
		}
	}
	/** <br>@b 始端チェックポイント到達時処理 */
	else {
		/**
		* - 記憶しておいた前回到達チェックポイントから見て，
		*   今回到達したチェックポイントが遷移先として妥当かどうか判定する【IWDGM180】〔WDGM246〕
		*/
		p_cp_inib = GET_CP(p_ex_logical_cb->ex_trans_source_se_id, p_ex_logical_cb->ex_trans_source_cp_id);
		
		/**
		* - 外部ロジカル監視は監視エンティティをまたいだ監視であるため，
		*   まず監視エンティティが遷移先として妥当か判定する【IWDGM181】〔DWDGM071〕
		*/
		p_ex_logical_dest_inib = p_cp_inib->p_ex_logical_inib;
		BINARY_SEARCH(p_ex_logical_dest_inib->p_ex_trans_dest_se_id_list, p_ex_logical_dest_inib->tnum_ex_trans_dest_se, SEID, ret_val, mid, left, right);
		
		/** - 遷移先の監視エンティティが妥当ならチェックポイントでも検索する【IWDGM182】〔DWDGM071〕 */
		if (ret_val != FALSE) {
			/** - SEIDを発見した箇所のインデックス(mid)でさらにチェックポイントを検索する【IWDGM183】〔DWDGM072〕 */
			p_ex_trans_dest_cp_id_list = p_ex_logical_dest_inib->p_ex_trans_dest_cp_id_list[mid].p_ex_trans_dest_cp_id_list;
			BINARY_SEARCH(p_ex_trans_dest_cp_id_list, p_ex_logical_dest_inib->p_ex_trans_dest_cp_id_list[mid].tnum_ex_trans_dest_cp, CheckpointID, ret_val, mid, left, right);
		}

		/**
		* - 最終チェックポイントの場合，アクティブフラグをFALSEにする【IWDGM184】〔WDGM331〕〔PWDGM051〕
		*  - WDGM_LOGICAL_FINALであれば，チェックポイントがWdgMInternalCheckpointFinalRefから参照されることを意味する
		*/
		if (p_ex_logical_inib->ex_logical_type == WDGM_LOGICAL_FINAL) {
			p_ex_logical_cb->active_flg = FALSE;
		}
	}

	if (ret_val != FALSE) {
		/** - 外部遷移が正常な場合，到達したチェックポイントIDと所属する監視エンティティIDを記憶する【IWDGM185】〔WDGM246〕〔DWDGM066〕〔PWDGM053〕〔PWDGM055〕 */
		p_ex_logical_cb->ex_trans_source_se_id = SEID;
		p_ex_logical_cb->ex_trans_source_cp_id = CheckpointID;
	}
	else {
		/** 外部遷移が異常な場合，ex_logical_incorrectをTRUEにする【IWDGM186】〔PWDGM048〕 */
		p_ex_logical_cb->ex_logical_incorrect = TRUE;
		/** 外部ロジカル監視で異常と判定したとき，expired_flgも同時にTRUEにする【IWDGM187】〔PWDGM017〕 */
		p_se_inib->p_local_status_cb->expired_flg = TRUE;
	}

}

/**
* @brief 内部関数：内部ロジカル監視実施
* @param[in] SEID 到達したチェックポイントの所属する監視エンティティのID
* @param[in] CheckpointID 到達したチェックポイントのID
* @return なし
*/
LOCAL_INLINE void
in_logical_supervision(const WdgM_SupervisedEntityIdType SEID, const WdgM_CheckpointIdType CheckpointID)
{
	boolean								ret_val;
	const WdgM_SEInitBlock				*p_se_inib;
	const WdgM_InternalLogicalInitBlock	*p_in_logical_dest_inib;
	const WdgM_InternalLogicalInitBlock	*p_in_logical_inib_list;
	WdgM_InternalLogicalControlBlock	*p_in_logical_cb;
	WdgM_LocalStatusControlBlock		*p_local_status_cb;
	uint16								mid, left, right;

	p_se_inib = GET_SE(SEID);
	p_in_logical_cb = p_se_inib->p_in_logical_cb;
	p_local_status_cb = p_se_inib->p_local_status_cb;

	/**
	* - 監視エンティティ初期化ブロックには1つのグラフが設定されているので，
	*   その初期化ブロックを所属しているグラフと認識して処理する【IWDGM188】〔WDGM295〕
	*  - 監視エンティティがグラフを持っていない場合，p_in_logical_inib_listはNULL_PTRであるが，
	*    本関数をコールする時点でp_in_logical_inib_listがNULL_PTRでないことを確認している
	*/
	p_in_logical_inib_list = p_se_inib->p_in_logical_inib_list;


	/** <br>@b 始端チェックポイント未到達時処理 */
	if (p_in_logical_cb->active_flg == FALSE) {
		/**
		* - アクティブフラグがFALSEのとき，到達したチェックポイントがWdgMInternalCheckpointInitialRefから参照されるものであれば正常
		*   そうでなければ異常とする【IWDGM189】〔WDGM274〕
		*  - WDGM_LOGICAL_INITであれば，チェックポイントがWdgMInternalCheckpointInitialRefから参照されることを意味する
		*/
		if (p_in_logical_inib_list[CheckpointID].in_logical_type == WDGM_LOGICAL_INIT) {
			/** - 到達したチェックポイントが始端チェックポイントであれば，active_flgをTRUEにする【IWDGM190】〔PWDGM092〕 */
			p_in_logical_cb->active_flg = TRUE;
			/** - アクティブフラグをTRUEにする【IWDGM191】〔WDGM332〕 */
			p_in_logical_cb->in_trans_source_cp_id = CheckpointID;
			ret_val = TRUE;
		}
		else {
			ret_val = FALSE;
		}
	}
	/** <br>@b 始端チェックポイント到達時処理 */
	else {
		/**
		* - 記憶しておいた前回到達チェックポイントから見て，
		*   今回到達したチェックポイントが遷移先として妥当かどうか判定する【IWDGM192】〔WDGM246〕〔DWDGM065〕
		*/
		p_in_logical_dest_inib = &p_in_logical_inib_list[p_in_logical_cb->in_trans_source_cp_id];
		BINARY_SEARCH(p_in_logical_dest_inib->p_in_trans_dest_cp_list, p_in_logical_dest_inib->tnum_in_trans_dest_cp, CheckpointID, ret_val, mid, left, right);

		/** 
		* - 最終チェックポイントの場合，アクティブフラグをFALSEにする【IWDGM193】〔WDGM331〕〔PWDGM093〕
		*  - WDGM_LOGICAL_FINALであれば，チェックポイントがWdgMInternalCheckpointFinalRefから参照されることを意味する
		*/
		if (p_in_logical_inib_list[CheckpointID].in_logical_type == WDGM_LOGICAL_FINAL) {
			p_in_logical_cb->active_flg = FALSE;
		}
	}

	if (ret_val != FALSE) {
		/** - 到達したチェックポイントIDを記憶する【IWDGM194】〔WDGM246〕〔PWDGM095〕 */
		p_in_logical_cb->in_trans_source_cp_id = CheckpointID;
	}
	else {
		/** 内部遷移が異常であれば，in_logical_incorrectをTRUEにする【IWDGM195】〔PWDGM090〕 */
		p_in_logical_cb->in_logical_incorrect = TRUE;
	}

}

/**
* @brief 内部関数：ローカル監視状態管理ブロック初期化実施
* - 本関数の引数にNULL_PTRが指定されないことを呼び出し元で保障すること
* @param[out] p_local_status_cb クリアするローカル監視状態管理ブロック
* @return なし
*/
LOCAL_INLINE void
clear_local_status(WdgM_LocalStatusControlBlock *p_local_status_cb)
{
	/** - EXPIREDフラグをFALSEに初期化【IWDGM196】〔PWDGM016〕 */
	p_local_status_cb->expired_flg = FALSE;
	/** - FAILEDカウンタを0に初期化【IWDGM197】〔PWDGM018〕 */
	p_local_status_cb->failed_cnt = 0U;

	/** 
	* - ローカル監視状態(local_status)の初期値は設定依存なため，
	*   set_local_statusをコールして別途設定することとし，
	*   ここでの初期化は実施しない【IWDGM198】〔PWDGM198〕
	*/

	return;
}

/**
* @brief 内部関数：内部ロジカル監視管理ブロック初期化実施
* @param[out] p_in_logical_cb クリアする内部ロジカル監視管理ブロック
* @return なし
*/
LOCAL_INLINE void
clear_in_logical(WdgM_InternalLogicalControlBlock *p_in_logical_cb)
{
	if (p_in_logical_cb != NULL_PTR) {
		/** - 内部ロジカル監視の監視結果をFALSEに初期化【IWDGM199】〔PWDGM089〕 */
		p_in_logical_cb->in_logical_incorrect = FALSE;
		/** - 内部ロジカル監視のアクティブフラグをFALSEに初期化【IWDGM200】〔PWDGM091〕 */
		p_in_logical_cb->active_flg = FALSE;
		/** - 内部ロジカル監視の前回到達チェックポイントIDを全Bit 1に初期化【IWDGM201】〔PWDGM094〕 */
		p_in_logical_cb->in_trans_source_cp_id = (WdgM_CheckpointIdType)~(WdgM_CheckpointIdType)0U;
	}

	return;
}

/**
* @brief 内部関数：モード依存変数初期化実施
* @param[in,out] p_mode_inib クリアする変数が依存するモードの初期化ブロック
* @return なし
*/
LOCAL_INLINE void
clear_mode_dependent_var(const WdgM_ModeInitBlock *p_mode_inib)
{
	uint16 i;

	/** 
	* - モード依存変数をクリアするタイミングでMF周期/OSカウンタ監視の
	*   アクティブフラグをFALSEに初期化する【IWDGM202】〔NWDGM101〕
	*/
#if WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON
	mf_oscounter_active_flg = FALSE;
#endif

	/** <br>@b アライブ監視機能が利用する変数を初期化 */
	for (i = 0U; i < p_mode_inib->tnum_alive; i++) {
		/** - アライブカウンタを0に初期化【IWDGM203】〔PWDGM023〕 */
		p_mode_inib->pp_alive_inib_list[i]->p_alive_cb->alive_cnt = 0U;
		/** - アライブ監視周期カウンタをWdgMSupervisionReferenceCycleに初期化【IWDGM204】〔PWDGM026〕 */
		p_mode_inib->pp_alive_inib_list[i]->p_alive_cb->supervision_cnt = p_mode_inib->pp_alive_inib_list[i]->WdgMSupervisionReferenceCycle;
		/** - アライブ監視の監視結果をFALSEに初期化【IWDGM231】〔PWDGM211〕 */
		p_mode_inib->pp_alive_inib_list[i]->p_alive_cb->alive_incorrect = FALSE;
	}

	/** <br>@b デッドライン監視機能が利用する変数を初期化 */
	for (i = 0U; i < p_mode_inib->tnum_deadline; i++) {
		/** - timestampを0に初期化【IWDGM205】〔PWDGM206〕 */
		p_mode_inib->pp_deadline_inib_list[i]->p_deadline_cb->timestamp = 0U;
		/** - 開始チェックポイント到達フラグをFALSEに初期化【IWDGM206】〔PWDGM036〕 */
		p_mode_inib->pp_deadline_inib_list[i]->p_deadline_cb->cp_reached = FALSE;
		/** - デッドライン監視の監視結果をFALSEに初期化【IWDGM207】〔PWDGM039〕 */
		p_mode_inib->pp_deadline_inib_list[i]->p_deadline_cb->deadline_incorrect = FALSE;
	}

	/** <br>@b 外部ロジカル監視機能が利用する変数を初期化 */
	for (i = 0U; i < p_mode_inib->tnum_ex_logical; i++) {
		/** - 外部ロジカル監視の監視結果をFALSEに初期化【IWDGM208】〔PWDGM199〕 */
		p_mode_inib->p_ex_logical_cb_list[i].ex_logical_incorrect = FALSE;
		/** - 外部ロジカル監視のアクティブフラグをFALSEに初期化【IWDGM209】〔PWDGM049〕 */
		p_mode_inib->p_ex_logical_cb_list[i].active_flg = FALSE;
		/** - 外部ロジカル監視の前回到達監視エンティティIDを全bit 1に初期化【IWDGM210】〔PWDGM052〕 */
		p_mode_inib->p_ex_logical_cb_list[i].ex_trans_source_se_id = (WdgM_SupervisedEntityIdType)~(WdgM_SupervisedEntityIdType)0U;
		/** - 外部ロジカル監視の前回到達チェックポイントIDを全bit 1に初期化【IWDGM211】〔PWDGM054〕 */
		p_mode_inib->p_ex_logical_cb_list[i].ex_trans_source_cp_id = (WdgM_CheckpointIdType)~(WdgM_CheckpointIdType)0U;
	}

	return;
}

/**
* @brief 内部関数：ローカル監視状態遷移＆遷移時アクション実施
* @param[in,out] p_se_inib ローカル監視所状態を遷移する監視エンティティの初期化ブロック
* @param[in] local_status 遷移先のローカル監視状態
* @return なし
*/
LOCAL_INLINE void
set_local_status(const WdgM_SEInitBlock *p_se_inib, const WdgM_LocalStatusType local_status)
{
	if (p_se_inib->p_local_status_cb->local_status != local_status) {
		/** ローカル監視状態を更新する【IWDGM212】〔PWDGM015〕 */
		p_se_inib->p_local_status_cb->local_status = local_status;
		/** - ローカル監視状態の変化をモードポートを介して通知する【IWDGM213】〔WDGM197〕 */
		(void)(p_se_inib->switch_local_status)(local_status);

		switch (local_status)
		{
		case WDGM_LOCAL_STATUS_EXPIRED:
			/** - FirstExpiredSEIDを操作する間，WdgM_CriticalSection_FirstExpiredSEIDで排他する【IWDGM214】〔DWDGM037〕 */
			SchM_Enter_WdgM_CriticalSection_FirstExpiredSEID();
			/**
			* - ローカル監視状態がWDGM_LOCAL_STATUS_EXPIREDに遷移した場合，
			*   FirstExpiredSEIDにSEIDを格納する【IWDGM215】〔WDGM351〕 
			*/
			WdgM_SetFirstExpiredSEID(p_se_inib->WdgMSupervisedEntityId);
			SchM_Exit_WdgM_CriticalSection_FirstExpiredSEID();
			break;
/** 
* <br>@b OsScalabilityClassがOSAPに対応したものであった場合，
*     @b ローカル監視状態がWDGM_LOCAL_STATUS_FAILEDに遷移した場合の
*     @b エラーハンドリング処理を実施する【IWDGM216】〔DWDGM082〕〔PWDGM200〕〔PWDGM201〕
*/
#if WDGM_OSAP_SUPPORT == STD_ON
		case WDGM_LOCAL_STATUS_FAILED:
			/**
			* - ローカル監視状態がWDGM_LOCAL_STATUS_FAILEDに遷移し，partition_reset_enableがTRUEであれば，
			*   BswM_WdgM_RequestPartitionResetをコールする【IWDGM217】〔WDGM225〕
			*  - partition_reset_enableがTRUEのとき，監視エンティティがWdgMOsApplicationRefを持っていることを意味する
			*/
			if (p_se_inib->partition_reset_enable != FALSE) {
				BswM_WdgM_RequestPartitionReset(p_se_inib->osap_id);
			}
			break;
#endif
		default:
			break;
		}
	}

	return;
}

/**
* @brief 内部関数：グローバル監視状態遷移＆遷移時アクション実施
* @param[in] global_status 遷移先のグローバル監視状態
* @return なし
*/
LOCAL_INLINE void
set_global_status(const WdgM_GlobalStatusType global_status) {
	const WdgM_ModeInitBlock	*p_mode_inib;

	if (WdgM_GlobalStatus.global_status != global_status) {
		/** グローバル監視状態の更新【IWDGM218】〔PWDGM008〕 */
		WdgM_GlobalStatus.global_status = global_status;
		switch (global_status)
		{
		case WDGM_GLOBAL_STATUS_STOPPED:
			/** - グローバル監視状態がWDGM_GLOBAL_STATUS_STOPPEDに遷移したときはトリガコンディションを0に設定する【IWDGM219】〔NWDGM001〕 */
			p_mode_inib = GET_MODE(current_mode);
			set_trigger_condition_incorrect(p_mode_inib);

			/** WDGM_DEM_STOPPED_SUPERVISION_REPORTがSTD_ONの場合のみ，DEMを発行する【IWDGM220】〔DWDGM085〕 */
#if WDGM_DEM_STOPPED_SUPERVISION_REPORT == STD_ON
			DEM_REPORT_STOPPED_SUPERVISION();
#endif /* WDGM_DEM_STOPPED_SUPERVISION_REPORT == STD_ON */

			/** 
			* - WDGM_IMMEDIATE_RESETがSTD_ONであり，グローバル監視状態がWDGM_GLOBAL_STATUS_STOPPEDに遷移した場合，
			*   Mcu_PerformResetをコールする【IWDGM221】〔WDGM133〕〔DWDGM077〕
			* - Mcu_PerformResetをコールする場合はRTEモードポートを介した通知を実施しない【IWDGM222】〔WDGM134〕
			*/
#if WDGM_IMMEDIATE_RESET == STD_ON
			Mcu_PerformReset();
#else /* WDGM_IMMEDIATE_RESET != STD_OFF */
			/** - WDGM_IMMEDIATE_RESETがSTD_ONでないとき，WDGM_GLOBAL_STATUS_STOPPEDに遷移する場合もRTEモードポートを介して通知する【IWDGM223】〔DWDGM078〕 */
			(void)Rte_Switch_globalMode_currentMode(global_status);
#endif /* WDGM_IMMEDIATE_RESET == STD_ON */
			break;

		default:
			/** - グローバル監視状態の変化を，変化に対して１回だけRTEモードポートを介して通知する【IWDGM224】〔WDGM198〕〔WDGM199〕 */
			(void)Rte_Switch_globalMode_currentMode(global_status);
			break;
		}
	}

	return;
}

/**
* @brief 内部関数：トリガコンディション正常設定
* @param[in] p_mode_inib トリガコンディションの情報を持つモード初期化ブロック
* @return なし
*/
LOCAL_INLINE void
set_trigger_condition_correct(const WdgM_ModeInitBlock *p_mode_inib)
{
	uint8				i;
	const WdgM_TriggerInitBlock	* const *pp_trigger_inib_list;

	/** トリガコンディション設定は排他区間WdgM_CriticalSection_SetTriggerCondition内で実施する【IWDGM225】〔DWDGM049〕 */
	SchM_Enter_WdgM_CriticalSection_SetTriggerCondition();

	/** - トリガコンディション設定が禁止されていない場合，トリガコンディションを正常設定する【IWDGM226】〔DWDGM014〕 */
	if (prohibited_set_trigger_condition == FALSE) {
		pp_trigger_inib_list = p_mode_inib->pp_trigger_inib_condition_list;

		for (i = 0U; i < p_mode_inib->tnum_trigger_condition; i++) {
			WdgIf_SetTriggerCondition((*(pp_trigger_inib_list[i])).wdg_id, (*(pp_trigger_inib_list[i])).WdgMTriggerConditionValue);
		}
	}
	SchM_Exit_WdgM_CriticalSection_SetTriggerCondition();
	return;

}

/**
* @brief 内部関数：トリガコンディション異常設定
* @param[in] p_mode_inib トリガコンディションの情報を持つモード初期化ブロック
* @return なし
*/
LOCAL_INLINE void
set_trigger_condition_incorrect(const WdgM_ModeInitBlock *p_mode_inib)
{
	uint8				i;
	const WdgM_TriggerInitBlock	* const *pp_trigger_inib_list;

	/** トリガコンディション設定は排他区間WdgM_CriticalSection_SetTriggerCondition内で実施する【IWDGM227】〔DWDGM049〕 */
	SchM_Enter_WdgM_CriticalSection_SetTriggerCondition();

	/** 
	* - トリガコンディション設定がまだ禁止されていなければ，
	*   トリガコンディションを0に設定して，以降のトリガコンディションを禁止する【IWDGM228】〔DWDGM014〕〔DWDGM015〕
	*/
	if (prohibited_set_trigger_condition == FALSE) {
		prohibited_set_trigger_condition = TRUE;

		pp_trigger_inib_list = p_mode_inib->pp_trigger_inib_condition_list;
		for (i = 0U; i < p_mode_inib->tnum_trigger_condition; i++) {
			WdgIf_SetTriggerCondition((*(pp_trigger_inib_list[i])).wdg_id, 0U);
		}
	}
	SchM_Exit_WdgM_CriticalSection_SetTriggerCondition();
	return;
}
