/*
 *  TOPPERS/A-WDGM
 *      Automotive Watchdog Manager
 *
 *  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2015-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2017 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgM.h 3486 2017-03-09 07:17:36Z suzuki-kawaguchi $
 */

/**
* @file WdgM.h
* @brief WdgMのヘッダファイル
* - 外部インタフェースをインクルード【PWDGM001】〔WDGM011〕〔WDGM014〕
* - プレコンパイルのパラメータはWdgM_Cfg.h，WdgM_Cfg.cにジェネレートされるので，
*   WdgM.hでWdgM_Cfg.hをインクルードし，コンフィギュレーションされた値を使用する【PWDGM002】〔WDGM025〕〔WDGM104〕
* - RTEの型定義ファイルをインクルードする【PWDGM003】〔WDGMa005〕
* - 型，および定数値を定義する【PWDGM210】〔DWDGM004〕〔NWDGM110〕〔WDGM038〕〔WDGM235〕
*/

#ifndef WDGM_H
#define WDGM_H

#include "Os.h"
#include "Std_Types.h"
#include "WdgIf.h"
#include "Rte_WdgM_Type.h"
#include "WdgM_Cfg.h"

/**
* @defgroup define_public_info BSW標準公開情報
* @brief BSW標準公開情報を記載する
* - BSW標準公開情報を記載する【PWDGM004】〔WDGM267〕
*/
/*@{*/
/* バージョン情報 */
#define WDGM_AR_RELEASE_MAJOR_VERSION		4U
#define WDGM_AR_RELEASE_MINOR_VERSION		0U
#define WDGM_AR_RELEASE_REVISION_VERSION	3U
/* サプライヤバージョン情報 */
#define WDGM_SW_MAJOR_VERSION				1U
#define WDGM_SW_MINOR_VERSION				2U
#define WDGM_SW_PATCH_VERSION				0U
#define WDGM_VENDOR_ID						65U	/* NCESベンダID */
/* WDGMモジュールID */
#define WDGM_MODULE_ID						0x0dU
/*@}*/

/**
* @defgroup define_dev_erorr 拡張エラーコード
* @brief 拡張エラーコードを1Byteのunsigned型で定義
* - 拡張エラーコードを1Byteのunsigned型で定義【PWDGM005】〔WDGM259〕
*/
/*@{*/
#define WDGM_E_NO_INIT						0x10U
#define WDGM_E_PARAM_CONFIG					0x11U
#define WDGM_E_PARAM_MODE					0x12U
#define WDGM_E_PARAM_SEID					0x13U
#define WDGM_E_INV_POINTER					0x14U
#define WDGM_E_DISABLE_NOT_ALLOWED			0x15U
#define WDGM_E_CPID							0x16U
#define WDGM_E_DEPRECATED					0x17U
#define WDGM_E_AMBIGIOUS					0x18U
#define WDGM_E_SEDEACTIVATED				0x19U
#define WDGM_E_CONCURRENTLY_CALLED			0xa0U
/*@}*/

/**
* @defgroup define_local_status ローカル監視状態
* @brief WdgM_LocalStatusType型の変数の値を1Byteのunsigned型で定義
*/
/*@{*/
#define WDGM_LOCAL_STATUS_OK				0x00U
#define WDGM_LOCAL_STATUS_FAILED			0x01U
#define WDGM_LOCAL_STATUS_EXPIRED			0x02U
#define WDGM_LOCAL_STATUS_DEACTIVATED		0x04U
/*@}*/

/**
* @defgroup define_global_status グローバル監視状態
* @brief WdgM_GlobalStatusType型の変数の値を1Byteのunsigned型で定義
*/
/*@{*/
#define WDGM_GLOBAL_STATUS_OK				0x00U
#define WDGM_GLOBAL_STATUS_FAILED			0x01U
#define WDGM_GLOBAL_STATUS_EXPIRED			0x02U
#define WDGM_GLOBAL_STATUS_STOPPED			0x03U
#define WDGM_GLOBAL_STATUS_DEACTIVATED		0x04U
/*@}*/

/**
* @defgroup define_api_id システムサービスID
* @brief 各システムサービスのIDを1Byteのunsigned型で定義
*/
/*@{*/
#define WdgMServiceId_Init							0x00U
#define WdgMServiceId_DeInit						0x01U
#define WdgMServiceId_GetVersionInfo				0x02U
#define WdgMServiceId_SetMode						0x03U
#define WdgMServiceId_GetMode						0x0bU
#define WdgMServiceId_CheckpointReached				0x0eU
#define WdgMServiceId_GetLocalStatus				0x0cU
#define WdgMServiceId_GetGlobalStatus				0x0dU
#define WdgMServiceId_PerformReset					0x0fU
#define WdgMServiceId_GetFirstExpiredSEID			0x10U
#define WdgMServiceId_MainFunction					0x08U
/*@}*/

/**
* @defgroup define_logical_cp_type ロジカル監視状態のチェックポイントタイプ
* @brief チェックポイントのロジカル監視における役割を定義
*/
/*@{*/
/** @brief ロジカル監視とは無関係(内部ロジカル監視でのみ使用) */
#define WDGM_LOGICAL_EXCLUDED					0x00U
/** @brief ロジカル監視の始端チェックポイント */
#define WDGM_LOGICAL_INIT						0x01U
/** @brief ロジカル監視の中間チェックポイント */
#define WDGM_LOGICAL_MID						0x02U
/** @brief ロジカル監視の終端チェックポイント */
#define WDGM_LOGICAL_FINAL						0x03U
/*@}*/

/**
* @brief Watchdog Managerモード型
*/
typedef uint8	WdgM_ModeType;
/**
* @brief 監視エンティティ個別のローカル監視状態型
*/
typedef uint8	WdgM_LocalStatusType;
/**
* @brief Watchdog Managerのグローバル監視状態型
*/
typedef uint8	WdgM_GlobalStatusType;
/**
* @brief ローカル監視状態を通知するモードポート
* - 引数Rte_ModeType_WdgM_Mode型１つ，返り値Std_ReturnType型である関数型を
*   ローカル監視状態変化時の通知用関数と定義する【PWDGM006】〔WDGM196〕
*/
typedef Std_ReturnType(*WdgM_SwitchLocalStatus)(Rte_ModeType_WdgM_Mode mode);

/**
* @brief グローバル監視状態管理ブロック
* - ジェネレート対象外
*/
typedef struct global_status_control_block {
	/**
	* @brief グローバル監視状態
	* - 初期値：WDGM_GLOBAL_STATUS_DEACTIVATED【PWDGM007】〔WDGM285〕
	* - 更新条件：グローバル監視状態が変化する条件を満たしたとき，そのグローバル監視状態に遷移する【PWDGM008】〔WDGM214〕
	*/
	WdgM_GlobalStatusType		global_status;
	/**
	* @brief Expiredカウンタ
	* - 初期値：0【PWDGM009】〔NWDGM105〕〔NWDGM048〕〔NWDGM049〕
	* - 更新条件：global_statusがWDGM_GLOBAL_STATUS_EXPIREDとなるとき，
	*   およびWDGM_GLOBAL_STATUS_EXPIREDであるときインクリメントする【PWDGM010】〔NWDGM105〕〔NWDGM048〕〔NWDGM049〕
	*/
	uint16						expired_cnt;
} WdgM_GlobalStatusControlBlock;

/**
* @brief ローカル監視状態管理ブロック
* - 生成数：WdgMSupervisedEntityの数【PWDGM011】〔WDGM200〕
* - 備考：デバッグにローカル監視状態を利用する場合，この構造体のメンバを参照することで可能【PWDGM012】〔WDGM238〕
*/
typedef struct local_status_control_block {
	/** 
	* @brief ローカル監視状態
	* - 初期値：モードに依存【PWDGM198】〔WDGM238〕
	*  - 現在のモードで対応する監視エンティティがアクティブであった場合，WDGM_LOCALSTATUS_OK【PWDGM013】〔WDGM268〕
	*  - 現在のモードで対応する監視エンティティが非アクティブであった場合，WDGM_LOCAL_STATUS_DEACTIVATED【PWDGM014】〔WDGM269〕
	* - 更新条件：ローカル監視状態が変化する条件を満たしたとき，そのローカル監視状態に遷移する【PWDGM015】〔WDGM200〕
	*/
	WdgM_LocalStatusType	local_status;
	/**
	* @brief EXPIRED状態になった監視エンティティを示すフラグ
	* - 初期値：FALSE【PWDGM016】〔WDGM201〕〔WDGM202〕〔WDGM203〕〔WDGM204〕〔WDGM300〕〔WDGM205〕〔WDGM206〕〔WDGM207〕
	* - 更新条件：このパラメータは現在のモードのデッドライン監視，
	*   外部ロジカル監視のいずれかの監視結果が異常であるときTRUEとなる【PWDGM017】〔WDGM201〕〔WDGM202〕〔WDGM203〕〔WDGM204〕〔WDGM300〕〔WDGM205〕〔WDGM206〕〔WDGM207〕
	* - 備考：ローカル監視状態を判定する際，デッドライン監視，外部ロジカル監視のいずれかに異常があったかの判定にはこのフラグを使用する
	*/
	boolean					expired_flg;
	/**
	* @brief FAILEDカウンタ
	* - 初期値：0【PWDGM018】〔WDGM203〕〔WDGM204〕〔WDGM300〕〔WDGM205〕
	* - 更新条件：local_statusがWDGM_LOCAL_STATUS_FAILEDとなるとき，あるいはWDGM_LOCAL_STATUS_FAILEDであるとき，インクリメントする【PWDGM019】〔WDGM203〕〔WDGM204〕
	* - 更新条件：監視結果に異常がないとき，デクリメントする【PWDGM020】〔WDGM300〕〔WDGM205〕
	* - 備考：0xFFFFのときインクリメントしない
	*/
	uint16					failed_cnt;
} WdgM_LocalStatusControlBlock;

/**
* @brief アライブ監視管理ブロック
* - 生成数：全てのWdgMConfigSetの全てのWdgMModeの中でWdgMAliveSupervisionの最大数【PWDGM021】〔WDGM085〕〔DWDGM021〕
* - 備考：デバッグにアライブ監視の管理する情報を利用する場合，この構造体のメンバを参照することで可能【PWDGM022】〔WDGM240〕
*/
typedef struct alive_control_block {
	/**
	* @brief アライブカウンタ
	* - 初期値：0【PWDGM023】〔WDGM321〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントにアライブ監視が設定されていればインクリメントする【PWDGM024】〔WDGM321〕
	* - 更新条件：supervision_cntが0となったとき，0にクリアする【PWDGM025】〔WDGM074〕〔WDGM098〕
	* - 備考：uint32で定義する【PWDGM209】〔DWDGM050〕
	*/
	uint32			alive_cnt;
	/**
	* @brief アライブ監視周期カウンタ
	* - 初期値：WdgMSupervisionReferenceCycle【PWDGM026】〔WDGM098〕
	* - 更新条件：WdgM_MainFuinctionでデクリメントする【PWDGM027】〔WDGM098〕
	* - 更新条件：supervision_cntが0となったとき，WdgM_AliveInitBlockのWdgMSupervisionReferenceCycleに再設定する【PWDGM028】〔WDGM098〕
	*/
	uint16			supervision_cnt;
	/**
	* @brief アライブ監視 監視結果
	* - 初期値：FALSE【PWDGM211】〔WDGM115〕
	* - 更新条件：アライブ監視の監視結果が異常であった場合，TRUEにする【PWDGM212】〔WDGM115〕
	* - 更新条件：アライブ監視の監視結果が正常であった場合，FALSEにする【PWDGM213】〔WDGM115〕
	*/
	boolean			alive_incorrect;
} WdgM_AliveControlBlock;

/**
* @brief アライブ監視初期化ブロック
* - 対応コンテナ：WdgMAliveSupervision
*/
typedef struct alive_initialization_block {
	/**
	* @brief 期待アライブ通知回数(最小値)
	* - 生成規則：WdgMExpectedAliveIndications - WdgMMinMargin【PWDGM029】〔WDGM074〕
	*/
	uint16						min_eai;
	/**
	* @brief 期待アライブ通知回数(最大値)
	* - 生成規則：WdgMExpectedAliveIndications + WdgMMaxMargin【PWDGM030】〔WDGM074〕
	*/
	uint16						max_eai;
	/**
	* @brief アライブ監視を実施するWdgM_MainFunction数
	* - 生成規則：WdgMSupervisionReferenceCycle【PWDGM031】〔WDGM098〕
	*/
	uint16						WdgMSupervisionReferenceCycle;	
	/**
	* @brief WdgM_AliveControlBlockへの参照
	* - 生成規則：同一のWdgMModeに所属しているWdgMAliveSupervision内で同じ領域を参照しないこと【PWDGM032】〔DWDGM024〕〔DWDGM030〕〔PWDGM021〕
	*/
	WdgM_AliveControlBlock		*p_alive_cb;
} WdgM_AliveInitBlock;

/**
* @brief デッドライン監視管理ブロック
* - 生成数：全てのWdgMConfigSetの全てのWdgMModeの中でWdgMDeadlineSupervisionの最大数【PWDGM033】〔WDGM293_1〕〔DWDGM022〕
* - 備考：デバッグにデッドライン監視の管理する情報を利用する場合，この構造体のメンバを参照することで可能【PWDGM034】〔WDGM241〕
*/
typedef struct deadline_control_block {
	/**
	* @brief タイムスタンプ
	* - 初期値：0【PWDGM206】〔WDGM298〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントが
	*             デッドライン監視の開始チェックポイントに設定されていた場合，現在のタイムスタンプに更新する【PWDGM035】〔WDGM228〕
	*/
	TickType	timestamp;
	/**
	* @brief 開始チェックポイント到達フラグ
	* - 初期値：FALSE【PWDGM036】〔NWDGM053〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントがデッドライン監視の開始チェックポイントに設定されていた場合，TRUEにする【PWDGM037】〔NWDGM054〕
	* - 更新条件：cp_reachedがTRUEであり，WdgM_CheckpointReachedで通知されたチェックポイントが
	*             デッドライン監視の終了チェックポイントに設定されていた場合，FALSEにする【PWDGM038】〔NWDGM055〕
	*/
	boolean		cp_reached;
	/**
	* @brief デッドライン監視 監視結果
	* - 初期値：FALSE【PWDGM039】〔WDGM294〕〔NWDGM076〕
	* - 更新条件：デッドライン監視の監視結果が異常であった場合，TRUEにする【PWDGM040】〔WDGM294〕
	* - 更新条件：デッドライン監視がタイムアウトした場合，TRUEにする【PWDGM041】〔NWDGM076〕
	*/
	boolean		deadline_incorrect;
} WdgM_DeadlineControlBlock;

/**
* @brief デッドライン監視初期化ブロック
* - 対応コンテナ：WdgMDeadlineSupervision
*/
typedef struct deadline_initialization_block {
	/**
	* @brief 期待経過時間(最小値)
	* - 生成規則：WdgMDeadlineMin / OsSecondsPerTick(デッドライン監視が参照しているチェックポイントが所属している監視エンティティのOSカウンタ)【PWDGM042】〔WDGM294〕
	*/
	TickType						min_tick;
	/**
	* @brief 期待経過時間(最大値)
	* - 生成規則：WdgMDeadlineMax / OsSecondsPerTick(デッドライン監視が参照しているチェックポイントが所属している監視エンティティのOSカウンタ)【PWDGM043】〔WDGM294〕
	*/
	TickType						max_tick;
	/**
	* @brief タイムアウト上限値
	* - 生成規則：(WdgMDeadlineMax + (WdgMSupervisionCycle * 2)) / OsSecondsPerTick【PWDGM044】〔DWDGM055〕
	*/
	TickType						timeout_limit_tick;
	/**
	* @brief WdgM_DeadlineControlBlockへの参照
	* - 生成規則：同一のWdgMModeに所属しているWdgMDeadlineSupervision内で同じ領域を参照しないこと【PWDGM045】〔DWDGM024〕〔DWDGM032〕〔PWDGM033〕
	*/
	WdgM_DeadlineControlBlock		*p_deadline_cb;
} WdgM_DeadlineInitBlock;

/**
* @brief 外部ロジカル監視管理ブロック
* - 生成数：全てのWdgMConfigSetの全てのWdgMModeの中でWdgMExternalLogicalSupervisionの最大数【PWDGM046】〔WDGM293_2〕〔DWDGM023〕
* - 備考：デバッグに外部ロジカル監視の管理する情報を利用する際は，この構造体のメンバを参照することで可能【PWDGM047】〔WDGM242〕〔DWDGM036〕
*/
typedef struct external_logical_control_block {
	/**
	* @brief 外部ロジカル監視 監視結果
	* - 初期値：FALSE【PWDGM199】〔WDGM252〕
	* - 更新条件：外部ロジカル監視の監視結果が異常であった場合，TRUEにする【PWDGM048】〔WDGM252〕
	*/
	boolean							ex_logical_incorrect;
	/**
	* @brief 外部ロジカル監視 アクティブフラグ
	* - 初期値：FALSE【PWDGM049】〔WDGM331〕〔WDGM271〕〔WDGM296〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントがWDGM_LOGICAL_INITであった場合，TRUEにする【PWDGM050】〔WDGM332〕〔WDGM271〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントがWDGM_LOGICAL_FINALであった場合，FALSEにする【PWDGM051】〔WDGM331〕〔WDGM271〕
	*/
	boolean							active_flg;
	/**
	* @brief 外部ロジカル監視 遷移先チェックポイントが所属する監視エンティティID
	* - 初期値：全bit 1【PWDGM052】〔PWDGM049〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントが外部ロジカル監視の遷移先であった場合，引数のSEIDに更新する【PWDGM053】〔DWDGM066〕
	* - 備考：本変数はactive_flgがTRUEのとき以外参照しない
	*/
	WdgM_SupervisedEntityIdType		ex_trans_source_se_id;
	/**
	* @brief 外部ロジカル監視 監視結果
	* - 初期値：全bit 1【PWDGM054】〔PWDGM049〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントが外部ロジカル監視の遷移先であった場合，引数のCheckpointIDに更新する【PWDGM055】〔WDGM246〕
	* - 備考：本変数はactive_flgがTRUEのとき以外参照しない
	*/
	WdgM_CheckpointIdType			ex_trans_source_cp_id;
} WdgM_ExternalLogicalControlBlock;

/**
* @brief 外部ロジカル監視 遷移先チェックポイント初期化ブロック
* - 対応コンテナ：WdgMExternalTransition
*/
typedef struct external_logical_dest_cp_initialization_block {
	/**
	* @brief 外部遷移先チェックポイント数
	* - 生成規則：p_ex_trans_dest_cp_id_listの要素数【PWDGM056】〔PWDGM057〕
	*/
	uint16								tnum_ex_trans_dest_cp;
	/**
	* @brief 外部遷移先チェックポイントリスト
	* - 生成規則：以下の条件を全て満たすWdgMCheckpointIdの配列を作成し，そのポインタを配置する【PWDGM057】〔DWDGM069〕
	*  - 条件1：WdgMExternalTransition内でWdgMExternalTransitionSourceRefが同一の参照先を持つ【PWDGM058】〔DWDGM069〕
	*  - 条件2：WdgMExternalTransitionDestRefの参照先が同じ監視エンティティに所属している【PWDGM059】〔DWDGM069〕
	*  - 条件3：条件1,2を満たす，WdgMExternalTransitionDestRefの参照先のWdgMCheckpointId【PWDGM060】〔DWDGM069〕
	* - 生成規則：このリスト内は昇順に並べる【PWDGM061】〔DWDGM070〕
	*/
	const WdgM_CheckpointIdType			*p_ex_trans_dest_cp_id_list;
} WdgM_ExternalLogicalDestCPInitBlock;

/**
* @brief 外部ロジカル監視初期化ブロック
* - 対応コンテナ：WdgMExternalLogicalSupervision
*/
typedef struct external_logical_initialization_block {
	/**
	* @brief グラフ内におけるチェックポイントの属性
	* - 生成規則：本構造体を配置するチェックポイントがWdgMExternalCheckpointInitialRefに設定されている場合，WDGM_LOGICAL_INITとする【PWDGM062】〔WDGM332〕
	* - 生成規則：本構造体を配置するチェックポイントがWdgMExternalCheckpointFinalRefに設定されている場合，WDGM_LOGICAL_FINALとする【PWDGM063】〔WDGM331〕
	* - 生成規則：本構造体を配置するチェックポイントがWdgMExternalCheckpointInitialRefに設定されておらず，
	*             WdgMExternalTransitionのWdgMExternalTransitionSourceRefに設定されている場合，WDGM_LOGICAL_MIDとする【PWDGM064】〔WDGM332〕〔WDGM331〕
	*/
	uint8										ex_logical_type;
	/**
	* @brief 遷移先チェックポイントが所属する監視エンティティ数
	* - 生成規則：p_ex_trans_dest_se_id_listとp_ex_trans_dest_cp_id_listの要素数【PWDGM065】〔PWDGM066〕
	*/
	uint16										tnum_ex_trans_dest_se;
	/**
	* @brief 遷移先チェックポイントが所属する監視エンティティリスト
	* - 生成規則：以下の条件を全て満たすWdgMSupervisedEntityIdの配列を作成し，そのポインタを配置する【PWDGM066】〔DWDGM067〕
	*  - 条件1：WdgMExternalTransition内でWdgMExternalTransitionSourceRefが同一の参照先を持つ【PWDGM067】〔DWDGM067〕
	*  - 条件2：条件1を満たすWdgMExternalTransitionDestRefの参照先のチェックポイントが所属している監視エンティティのWdgMSupervisedEntityId【PWDGM068】〔DWDGM067〕
	* - 生成規則：このリスト内は昇順に並べる【PWDGM069】〔DWDGM068〕
	* - 生成規則：ex_logical_typeがWDGM_LOGICAL_FINALである場合，NULL_PTR【PWDGM070】〔WDGM252〕
	*/
	const WdgM_SupervisedEntityIdType			*p_ex_trans_dest_se_id_list;
	/**
	* @brief 遷移先チェックポイントリストへの参照リスト
	* - 生成規則：WdgM_ExternalLogicalDestCPInitBlock型の配列を作成し，そのポインタを配置する【PWDGM071】〔DWDGM069〕
	* - 生成規則：p_ex_trans_dest_se_id_listとp_ex_trans_dest_cp_id_listは同じ要素番号に同じ監視エンティティの情報を配置する【PWDGM072】〔DWDGM069〕
	* - 生成規則：ex_logical_typeがWDGM_LOGICAL_FINALである場合，NULL_PTR【PWDGM073】〔WDGM252〕
	*/
	const WdgM_ExternalLogicalDestCPInitBlock	*p_ex_trans_dest_cp_id_list;
	/**
	* @brief WdgM_ExternalLogicalControlBlockへの参照
	* - 生成規則：同一のWdgMModeに所属しているWdgMExternalLogicalSupervision内で同じ領域を参照しない【PWDGM074】〔DWDGM024〕〔PWDGM046〕
	* - 生成規則：同一のWdgMExternalLogicalSupervisionから生成された初期化ブロックには同じ領域を配置する【PWDGM075】〔PWDGM046〕
	*/
	WdgM_ExternalLogicalControlBlock			*p_ex_logical_cb;
} WdgM_ExternalLogicalInitBlock;

/**
* @brief チェックポイント モード依存情報 初期化ブロック
* - 対応コンテナ：WdgMMode
* - 備考：通常チェックポイントはWdgMSupervisedEntityのサブコンテナとして設定されるものであるが，
*         モード毎にどのチェックポイントにどの監視機能が紐づいているかを保持することで，
*         WdgM_CheckpointReachedの引数であるSEIDとCheckpointIDから実行すべき監視を高速で検索できるようにする
*/
typedef struct checkpoint_initialization_block {
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているアライブ監視の数
	* - 生成規則：pp_alive_inib_listの要素数【PWDGM076】〔PWDGM079〕
	*/
	uint16							tnum_alive;
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているデッドライン監視開始の数
	* - 生成規則：pp_deadline_inib_start_listの要素数【PWDGM077】〔PWDGM081〕
	*/
	uint16							tnum_deadline_start;
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているデッドライン監視終了の数
	* - 生成規則：pp_deadline_inib_end_listの要素数【PWDGM078】〔PWDGM083〕
	*/
	uint16							tnum_deadline_end;
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているアライブ監視のポインタリスト
	* - 生成規則：同じWdgMModeに所属しているWdgMAliveSupervisionコンテナの中で
	*             同じWdgMAliveSupervisionCheckpointRefを持つアライブ監視の配列を作成し，そのポインタを配置する【PWDGM079】〔WDGM115〕
	* - 生成規則：どのWdgMAliveSupervisionCheckpointRefからも参照されていない場合，NULL_PTR【PWDGM080】〔WDGM083〕
	*/
	const WdgM_AliveInitBlock		* const *pp_alive_inib_list;
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているデッドライン監視のポインタリスト
	* - 生成規則：同じWdgMModeに所属しているWdgMDeadlineSupervisionコンテナの中で
	*             同じWdgMDeadlineStartRefを持つデッドライン監視の配列を作成し，そのポインタを配置する【PWDGM081】〔WDGM228〕〔NWDGM054〕
	* - 生成規則：どのWdgMDeadlineStartRefからも参照されていない場合，NULL_PTR【PWDGM082】〔WDGM228〕〔NWDGM054〕
	*/
	const WdgM_DeadlineInitBlock	* const *pp_deadline_inib_start_list;
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいているデッドライン監視のポインタリスト
	* - 生成規則：同じWdgMModeに所属しているWdgMDeadlineSupervisionコンテナの中で
	*             同じWdgMDeadlineStopRefを持つデッドライン監視の配列を作成し，そのポインタを配置する【PWDGM083】〔NWDGM055〕
	* - 生成規則：どのWdgMDeadlineStopRefからも参照されていない場合，NULL_PTR【PWDGM084】〔NWDGM055〕
	*/
	const WdgM_DeadlineInitBlock	* const *pp_deadline_inib_end_list;	
	/**
	* @brief 現在のモードでこのチェックポイントに紐づいている外部ロジカル監視のポインタリスト
	* - 生成規則：同じWdgMModeに所属しているWdgMExternalLogicalSupervisionコンテナの中で
	*             同じWdgMExternalCheckpointFinalRefを持つ，または，
	*             同じWdgMExternalTransition/WdgMExternalTransitionSourceRefを持つ外部ロジカル監視の配列を作成し，そのポインタを配置する【PWDGM085】〔WDGM295〕
	* - 生成規則：どのWdgMExternalCheckpointFinalRef，WdgMExternalTransitionSourceRefからも参照されていない場合，NULL_PTR【PWDGM086】〔WDGM297〕
	*/
	const WdgM_ExternalLogicalInitBlock		*p_ex_logical_inib;
} WdgM_CheckpointInitBlock;

/**
* @brief 内部ロジカル監視管理ブロック
* - 生成数：内部ロジカル監視を持つ監視エンティティ数【PWDGM087】〔WDGM293_2〕
* - 備考：デバッグに内部ロジカル監視の管理する情報を利用する際は，この構造体のメンバを参照することで可能【PWDGM088】〔WDGM242〕
*/
typedef struct internal_logical_control_block {
	/**
	* @brief 内部ロジカル監視の監視結果
	* - 初期値：FALSE【PWDGM089】〔WDGM274〕
	* - 更新条件：内部ロジカル監視の監視結果が異常であった場合，TRUEにする【PWDGM090】〔WDGM274〕
	*/
	boolean						in_logical_incorrect;
	/**
	* @brief 内部ロジカル監視のアクティブフラグ
	* - 初期値：FALSE【PWDGM091】〔WDGM271〕〔WDGM332〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントがWDGM_LOGICAL_INITであった場合，TRUEにする【PWDGM092】〔WDGM271〕〔WDGM332〕
	* - 更新条件：WdgM_CheckpointReachedで通知されたチェックポイントがWDGM_LOGICAL_FINALであった場合，FALSEにする【PWDGM093】〔WDGM271〕〔WDGM331〕
	*/
	boolean						active_flg;
	/**
	* @brief 内部ロジカル監視の前回到達チェックポイントID
	* - 初期値：全bit 1【PWDGM094】〔PWDGM091〕
	* - 更新条件：WdgM_CheckpointReachedによって通知されたチェックポイントが内部ロジカル監視で正常なチェックポイントであった場合，
	*             引数のCheckpointIDで更新する【PWDGM095】〔WDGM246〕
	* - 備考：本変数はactive_flgがTRUEのとき以外参照しない
	*/
	WdgM_CheckpointIdType		in_trans_source_cp_id;
} WdgM_InternalLogicalControlBlock;

/**
* @brief 内部ロジカル監視初期化ブロック
* - 対応コンテナ：WdgMSupervisedEntity
*/
typedef struct internal_logical_initialization_block {
	/**
	* @brief グラフ内におけるチェックポイントの属性
	* - 生成規則：本構造体を配置するチェックポイントがWdgMInternalCheckpointInitialRefに設定されている場合，WDGM_LOGICAL_INITとする【PWDGM096】〔WDGM332〕
	* - 生成規則：本構造体を配置するチェックポイントがWdgMInternalCheckpointFinalRefに設定されている場合，WDGM_LOGICAL_FINALとする【PWDGM097】〔WDGM331〕
	* - 生成規則：本構造体を配置するチェックポイントがWdgMInternalCheckpointInitialRefに設定されておらず，
	*             WdgMInternalTransitionのWdgMInternalTransitionSourceRefに設定されている場合，WDGM_LOGICAL_MIDとする【PWDGM098】〔WDGM331〕〔WDGM332〕
	* - 生成規則：本構造体を配置するチェックポイントが内部ロジカル監視と関係がない場合，WDGM_LOGICAL_EXCLUDEDとする【PWDGM099】〔WDGM297〕
	*/
	uint8							in_logical_type;
	/**
	* @brief 内部ロジカル監視の遷移先チェックポイント数
	* - 生成規則：p_in_trans_dest_cp_listの要素数【PWDGM100】〔PWDGM101〕
	*/
	uint16							tnum_in_trans_dest_cp;
	/**
	* @brief 内部ロジカル監視の遷移先チェックポイントリスト
	* - 生成規則：同じWdgMInternalTransitionSourceRefと対になっているWdgMInternalTransitionDestRefの配列を作成し，そのポインタを配置する【PWDGM101】〔DWDGM063〕
	* - 生成規則：このリスト内は昇順に並べる【PWDGM207】〔DWDGM064〕
	* - 生成規則：in_logical_typeがWDGM_LOGICAL_FINAL，WDGM_LOGICAL_EXCLUDEDの場合，NULL_PTR【PWDGM102】〔WDGM297〕〔WDGM274〕
	*/
	const WdgM_CheckpointIdType		*p_in_trans_dest_cp_list;
} WdgM_InternalLogicalInitBlock;

/**
* @brief OSカウンタの情報
* - 対応コンテナ：OsCounter(外部モジュール情報)
*/
typedef struct os_counter_info {
	/**
	* @brief OSカウンタID
	* - 生成規則：OsCounterのショートネーム【PWDGM103】〔NWDGM088〕
	*/
	CounterType					id;
	/**
	* @brief OSカウンタの最大ティック値
	* - 生成規則：OsCounterMaxAllowedValue【PWDGM104】〔DWDGM061〕〔DWDGM062〕
	*/
	TickType					OsCounterMaxAllowedValue;
} WdgM_OsCounterInitBlock;

/**
* @brief 監視エンティティ初期化ブロック
* - 対応コンテナ：WdgMSupervisedEntity
*/
typedef struct supervised_entity_initialization_block {
	/**
	* @brief 監視エンティティID
	* - 生成規則：WdgMSupervisedEntityId【PWDGM105】〔WDGM351〕
	*/
	WdgM_SupervisedEntityIdType			WdgMSupervisedEntityId;
	/**
	* @brief 所属するチェックポイント数
	* - 生成規則：WdgMSupervisedEntityコンテナに所属するWdgMCheckpointコンテナの数【PWDGM106】〔WDGM284〕〔NWDGM061〕
	*/
	uint16								tnum_cp;
#if WDGM_OSAP_SUPPORT == STD_ON
	/**
	* @brief パーティションリセット有効/無効
	* - 生成規則：WdgMOsApplicationRefが設定されていればTRUE，そうでなければFALSE【PWDGM107】〔WDGM225〕
	* - 生成規則：WDGM_OSAP_SUPPORTがSTD_ONのときのみのときのみ有効【PWDGM200】〔DWDGM081〕
	*/
	boolean								partition_reset_enable;
	/**
	* @brief OSAP ID
	* - 生成規則：WdgMOsApplicationRefが設定されていればその参照先のショートネーム，そうでなければ0【PWDGM108】〔WDGM225〕
	* - 生成規則：WDGM_OSAP_SUPPORTがSTD_ONのときのみのときのみ有効【PWDGM201】〔DWDGM081〕
	*/
	ApplicationType						osap_id;
#endif
	/**
	* @brief ローカル監視状態を通知するモードポート
	* - 生成規則：Rte_Switch_mode<WdgMSupervisedEntityId>_currentMode【PWDGM109】〔WDGM196〕
	*/
	WdgM_SwitchLocalStatus				switch_local_status;
	/**
	* @brief 内部ロジカル初期化ブロックリスト
	* - 生成規則：WdgM_InternalLogicalInitBlock型の配列をチェックポイントの数だけ作成し，そのポインタを配置する【PWDGM110】〔WDGM295〕
	* - 生成規則：配列の要素番号とWdgMCheckpointIdは対応させること【PWDGM111】〔WDGM295〕
	* - 生成規則：WdgMInternalCheckpointInitialRefが設定されていない場合，NULL_PTR【PWDGM112】〔WDGM295〕
	*/
	const WdgM_InternalLogicalInitBlock	*p_in_logical_inib_list;
	/**
	* @brief OSカウンタへの参照
	* - 生成規則：WdgMOSCounterから参照されるOsCounterの情報が格納されたWdgM_OsCounterInitBlockのポインタ【PWDGM113】〔NWDGM088〕
	* - 生成規則：WdgMOSCounterが設定されていない場合，NULL_PTR【PWDGM114】〔NWDGM088〕
	*/
	const WdgM_OsCounterInitBlock		*os_counter;
	/**
	* @brief WdgM_InternalLogicalControlBlockへの参照
	* - 生成規則：p_in_logical_inib_listがNULL_PTRでない場合，WdgM_InternalLogicalControlBlockを１つ作成し，そのポインタを配置する【PWDGM115】〔WDGM212_3〕〔DWDGM034〕〔PWDGM087〕
	* - 生成規則：p_in_logical_inib_listがNULL_PTRである場合，NULL_PTR【PWDGM116】〔WDGM212_3〕〔PWDGM112〕
	*/
	WdgM_InternalLogicalControlBlock	*p_in_logical_cb;
	/**
	* @brief WdgM_LocalStatusControlBlockへの参照
	* - 生成規則：監視エンティティ１つにつきWdgM_LocalStatusControlBlockを１つ作成し，そのポインタを配置する【PWDGM117】〔DWDGM026〕〔PWDGM011〕
	*/
	WdgM_LocalStatusControlBlock		*p_local_status_cb;
} WdgM_SEInitBlock;

/**
* @brief トリガ初期化ブロック
* - 生成規則：WdgMTrigger
*/
typedef struct trigger_initialization_block {
	/**
	* @brief 現在のモードで対象のWatchdog Driver に与えるトリガコンディション量
	* - 生成規則：WdgMTriggerConditionValue【PWDGM118】〔NWDGM001〕
	*/
	uint16					WdgMTriggerConditionValue;
	/**
	* @brief 現在のモードで対象のWatchdog Driver に与えるモード
	* - 生成規則：WdgMWatchdogMode【PWDGM119】〔WDGM181〕
	*/
	WdgIf_ModeType			WdgMWatchdogMode;
	/**
	* @brief 操作するWatchdog Driver のID
	* - 生成規則：WdgMTriggerWatchdogRefが参照するWdgMWatchdogコンテナのWdgMWatchdogName【PWDGM120】〔WDGM119〕〔WDGM120〕〔WDGM121〕〔WDGM122〕
	*/
	uint8					wdg_id;
} WdgM_TriggerInitBlock;

/**
* @brief MF周期/OSカウンタ監視の初期化ブロック
* - 対応コンテナ：WdgMMFCycleAndOsCounterSupervision
*/
typedef struct mainfunction_oscounter_supervision_initialization_block {
	/**
	* @brief 許容される経過時間最大値(Tick値)
	* - 生成規則：WdgMMFCycleMaxTol / OsSecondsPerTick(本監視が参照しているOSカウンタ)【PWDGM121】〔NWDGM098〕
	*/
	TickType						max_tick;
	/**
	* @brief 許容される経過時間最小値(Tick値)
	* - 生成規則：WdgMMFCycleMinTol / OsSecondsPerTick(本監視が参照しているOSカウンタ)【PWDGM122】〔NWDGM098〕
	*/
	TickType						min_tick;
	/**
	* @brief 監視するOSカウンタリスト
	* - 生成規則：WdgMOsCounterRefが参照するOSカウンタの情報をWdgM_OsCounterInitBlock型として作成し，そのポインタを配置する【PWDGM123】〔NWDGM097〕
	*/
	const WdgM_OsCounterInitBlock	*p_os_counter;
	/**
	* @brief タイムスタンプ格納領域への参照
	* - 生成規則：同一のWdgMModeに所属するWdgMMFCycleAndOsCounterSupervision内では同一の領域を参照しないこと【PWDGM124】〔PWDGM125〕
	* - 生成規則：全てのWdgMModeの中で最も多くのOSカウンタへの参照を持っているWdgMOsCounterRefの数だけTickType型の記憶領域を作成する【PWDGM125】〔NWDGM097〕〔DWDGM020〕
	*/
	TickType						*p_timestamp;
} WdgM_MFOsCounterInitBlock;

/**
* @brief アクティブな監視エンティティだけを集めた監視エンティティリスト
* - 対応コンテナ：WdgMMode，WdgMLocalStatusParams
*/
typedef struct active_se_initialization_block {
	/**
	* @brief 監視エンティティへの参照
	* - 生成規則：WdgMLocalStatusSupervisedEntityRefから参照される監視エンティティのポインタを配置する【PWDGM126】〔WDGM325〕
	*/
	const WdgM_SEInitBlock			*p_se_inib;
	/**
	* @brief 失敗したアライブ監視の許容量
	* - 生成規則：WdgMFailedAliveSupervisionRefCycleTol【PWDGM127】〔WDGM202〕〔WDGM203〕〔WDGM204〕〔WDGM206〕
	*/
	uint8							WdgMFailedAliveSupervisionRefCycleTol;
	/**
	* @brief 現在のモードで監視エンティティに関連しているアライブ監視の数
	* - 生成規則：pp_alive_inib_listの要素数【PWDGM128】〔PWDGM129〕
	*/
	uint16							tnum_alive;
	/**
	* @brief 現在のモードで監視エンティティに関連しているアライブ監視のリスト
	* - 生成規則：WdgMModeに所属しているWdgMAliveSupervisionの中でWdgMAliveSupervisionCheckpointRefが
	*             参照するチェックポイントが所属する監視エンティティが同一のものの配列を作成し，そのポインタを配置する【PWDGM129】〔WDGM324〕
	* - 生成規則：その監視エンティティがp_se_inibと等しいものを配置する【PWDGM130】〔WDGM324〕
	* - 生成規則：p_se_inibから参照する監視エンティティの全てのWdgMCheckpointが，
	*             どのWdgMAliveSupervisionCheckpointRefからも参照されない場合，NULL_PTR【PWDGM131】〔WDGM324〕
	*/
	const WdgM_AliveInitBlock		* const *pp_alive_inib_list;
} WdgM_ActiveSEInitBlock;

/**
* @brief デッドライン監視のタイムアウト機能を利用する監視エンティティ初期化ブロック
* - 生成規則：WdgMDeadlineSupervision
*/
typedef struct deadline_timeout_se_initialization_block {
	/**
	* @brief デッドライン監視から参照する監視エンティティへの参照
	* - 生成規則：WdgMDeadlineSupervisionのWdgMDeadlineStartRefおよびWdgMDeadlineStopRefが
	*             参照するチェックポイントが所属する監視エンティティのポインタを配置する【PWDGM132】〔DWDGM052〕
	*/
	const WdgM_SEInitBlock			*p_se_inib;	
	/**
	* @brief 監視エンティティに設定されているタイムアウト機能を使用するデッドライン監視数
	* - 生成規則：pp_deadline_inib_listの要素数【PWDGM133】〔PWDGM134〕
	*/
	uint16							tnum_deadline_timeout;
	/**
	* @brief 監視エンティティに設定されているタイムアウト機能を使用するデッドライン監視のポインタリスト
	* - 生成規則：以下の条件を全て満たすWdgMDeadlineSupervisionのポインタの配列を作成し，ポインタを配置する【PWDGM134】〔DWDGM053〕
	*  - 条件1：同一のWdgMModeに所属する【PWDGM135】〔DWDGM053〕
	*  - 条件2：p_se_inibを参照する【PWDGM136】〔DWDGM053〕
	*  - 条件3：WdgMDeadlineTimeoutDetectがTRUEに設定されている【PWDGM137】〔DWDGM053〕
	*/
	const WdgM_DeadlineInitBlock	* const *pp_deadline_inib_list;
} WdgM_SEWithDeadlineTimeoutInitBlock;

/**
* @brief モード初期化ブロック
* - 対応コンテナ：WdgMMode
*/
typedef struct mode_initialization_block {
	/**
	* @brief 失効した監視サイクルの許容量
	* - 生成規則：WdgMExpiredSupervisionCycleTol【PWDGM138】〔WDGM239_2〕
	*/
	uint16								WdgMExpiredSupervisionCycleTol;
	/**
	* @brief アライブ監視数
	* - 生成規則：p_alive_inib_listの要素数【PWDGM139】〔PWDGM146〕
	*/
	uint16								tnum_alive;
	/**
	* @brief デッドライン監視数
	* - 生成規則：p_deadline_inib_listの要素数【PWDGM140】〔PWDGM148〕
	*/
	uint16								tnum_deadline;
	/**
	* @brief 外部ロジカル監視数
	* - 生成規則：p_ex_logical_cb_listの要素数【PWDGM141】〔PWDGM150〕
	*/
	uint16								tnum_ex_logical;
	/**
	* @brief 使用するWatchdog数
	* - 生成規則：p_trigger_inib_listの要素数【PWDGM142】〔PWDGM152〕
	*/
	uint8								tnum_trigger;
	/**
	* @brief トリガコンディション設定を実施するWatchdog数
	* - 生成規則：pp_trigger_inib_condition_listの要素数【PWDGM143】〔PWDGM153〕
	*/
	uint8								tnum_trigger_condition;
	/**
	* @brief アクティブな監視エンティティ数
	* - 生成規則：p_active_se_inib_listの要素数【PWDGM144】〔PWDGM154〕
	*/
	uint16								tnum_active_se;	
	/**
	* @brief 非アクティブな監視エンティティ数
	* - 生成規則：pp_deactive_se_inib_listの要素数【PWDGM145】〔PWDGM156〕
	*/
	uint16								tnum_deactive_se;
	/**
	* @brief アライブ監視リスト
	* - 生成規則：同一のWdgMModeに所属するWdgMAliveSupervisionの配列を作成し，そのポインタを配置する【PWDGM146】〔NWDGM085〕
	* - 生成規則：WdgMAliveSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM147】〔NWDGM085〕
	*/
	const WdgM_AliveInitBlock			* const *pp_alive_inib_list;
	/**
	* @brief デッドライン監視リスト
	* - 生成規則：同一のWdgMModeに所属するWdgMDeadlineSupervisionの配列を作成し，そのポインタを配置する【PWDGM148】〔NWDGM085〕
	* - 生成規則：WdgMDeadlineSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM149】〔NWDGM085〕
	*/
	const WdgM_DeadlineInitBlock		* const *pp_deadline_inib_list;
	/**
	* @brief 外部ロジカル監視管理ブロックリスト
	* - 生成規則：同一のWdgMModeに所属するWdgMExternalLogicalSupervisionが参照する管理ブロックの配列を作成し，そのポインタを配置する【PWDGM150】〔NWDGM085〕
	* - 生成規則：WdgMExternalLogicalSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM151】〔NWDGM085〕
	*/
	WdgM_ExternalLogicalControlBlock	*p_ex_logical_cb_list;
	/**
	* @brief 使用するWatchdogリスト
	* - 生成規則：同一のWdgMModeに所属するWdgMTriggerの配列を作成し，そのポインタを配置する【PWDGM152】〔WDGM002〕〔WDGM186〕〔NWDGM089〕
	*/
	const WdgM_TriggerInitBlock			*p_trigger_inib_list;
	/**
	* @brief トリガコンディション設定を実施するWatchdogリスト
	* - 生成規則：p_trigger_inib_listのうち，WdgMWatchdogModeがWDGIF_OFF_MODEではないもののポインタの配列を作成し，
	*   そのポインタを配置する【PWDGM153】〔WDGM119〕〔WDGM120〕〔WDGM121〕〔WDGM122〕
	* - 生成規則：WdgMWatchdogModeがWDGIF_OFF_MODEではないWdgMWatchdogが1つも設定されていない場合，
	*   NULL_PTR【PWDGM197】〔WDGM119〕〔WDGM120〕〔WDGM121〕〔WDGM122〕
	*/
	const WdgM_TriggerInitBlock			* const *pp_trigger_inib_condition_list;
	/**
	* @brief アクティブな監視エンティティリスト
	* - 生成規則：WdgMLocalStatusParamsのWdgMLocalStatusSupervisedEntityRefから参照される監視エンティティに関する
	*             WdgM_ActiveSEInitBlock型の配列を作成し，そのポインタを配置する【PWDGM154】〔WDGM209〕〔WDGM325〕〔WDGM212_2〕
	* - 生成規則：WdgMLocalStatusParamsが1つも設定されていない場合，NULL_PTR【PWDGM155】〔WDGM209〕〔WDGM325〕
	*/
	const WdgM_ActiveSEInitBlock		* p_active_se_inib_list;
	/**
	* @brief 非アクティブな監視エンティティリスト
	* - 生成規則：全ての監視エンティティのうち，WdgMLocalStatusParamsのWdgMLocalStatusSupervisedEntityRefから参照されない
	*             監視エンティティのポインタのリストを作成し，そのポインタを配置する【PWDGM156】〔WDGM315〕
	* - 生成規則：WdgMLocalStatusParamsのWdgMLocalStatusSupervisedEntityRefから参照されない監視エンティティ
	*             が1つも設定されていない場合，NULL_PTR【PWDGM157】〔WDGM315〕
	*/
	const WdgM_SEInitBlock				* const *pp_deactive_se_inib_list;
	/**
	* @brief チェックポイントテーブル[SEID][CheckpointID]
	* - 生成規則：以下の条件を全て満たすWdgM_CheckpointInitBlock型の配列を作成するPWDGM158】〔WDGM263〕
	*  - 条件1：同一のWdgMModeに所属する監視機能を情報持っている【PWDGM158】〔WDGM263〕
	*  - 条件2：同一の監視エンティティに所属している【PWDGM159】〔WDGM263〕
	* - 生成規則：WdgMSupervisedEntityIdを要素番号に対応させ，上記のリストのポインタを配置する【PWDGM160】〔WDGM263〕
	* - 生成規則：存在しない，または非アクティブな監視エンティティの要素番号はNULL_PTRで埋める【PWDGM161】〔WDGM263〕
	* - 生成規則：p_active_se_inib_listがNULL_PTRの場合，NULL_PTR【PWDGM162】〔PWDGM155〕
	*/
	const WdgM_CheckpointInitBlock		* const *pp_cp_inib_table;
#if WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON
	/**
	* @brief MF周期/OSカウンタ監視数
	* - 生成規則：p_mf_osounter_inibの要素数【PWDGM163】〔PWDGM164〕
	* - 生成規則：WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLEDがSTD_ONのときのみのときのみ有効【PWDGM202】〔DWDGM088〕
	*/
	uint16								tnum_mf_os_conter;
	/**
	* @brief MF周期/OSカウンタ監視リスト
	* - 生成規則：WdgM_MFOsCounterInitBlock型の配列を作成し，そのポインタを配置する【PWDGM164】〔NWDGM097〕
	* - 生成規則：WdgMMFCycleAndOsCounterSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM165】〔NWDGM097〕
	* - 生成規則：WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLEDがSTD_ONのときのみのときのみ有効【PWDGM203】〔DWDGM088〕
	* - 備考：WdgMMFCycleAndOsCounterSupervisionは最大１つであるが，参照するOSカウンタによってOsSecondsPerTickが異なるため，
	*         WdgMOsCounterRefで参照するOSカウンタの数だけ配列を作成する
	*/
	const WdgM_MFOsCounterInitBlock		*p_mf_osounter_inib;
#endif /* WDGM_MF_CYCLE_AND_OSCOUNTER_SURPERVISION_ENABLED == STD_ON */
#if WDGM_DEADLINE_TIMEOUT_ENABLED == STD_ON
	/**
	* @brief タイムアウト機能を使用するデッドライン監視数
	* - 生成規則：p_deadline_timeout_se_inib_listの要素数【PWDGM166】〔PWDGM167〕
	* - 生成規則：WDGM_DEADLINE_TIMEOUT_ENABLEDがSTD_ONのときのみのときのみ有効【PWDGM204】〔DWDGM086〕
	*/
	uint16								tnum_deadline_timeout_se;
	/**
	* @brief タイムアウト機能を使用するデッドライン監視リスト
	* - 生成規則：WdgM_SEWithDeadlineTimeoutInitBlock型の配列を作成し，そのポインタを配置する【PWDGM167】〔DWDGM052〕
	* - 生成規則：WdgM_SEWithDeadlineTimeoutInitBlockを生成する条件を1つも満たさない場合，NULL_PTR【PWDGM168】〔DWDGM052〕
	* - 生成規則：WDGM_DEADLINE_TIMEOUT_ENABLEDがSTD_ONのときのみのときのみ有効【PWDGM205】〔DWDGM086〕
	*/
	const WdgM_SEWithDeadlineTimeoutInitBlock	*p_deadline_timeout_se_inib_list;
#endif /* WDGM_DEADLINE_TIMEOUT_ENABLED == STD_ON */
} WdgM_ModeInitBlock;

/**
* @brief DEM初期化ブロック
* - 対応コンテナ：WdgMDemEventParameterRefs
*/
typedef struct dem_initialization_block {
	/**
	* @brief 不適切な発信者を検出した場合のエラーID
	* - 生成規則：WDGM_E_IMPROPER_CALLER【PWDGM169】〔NWDGM046〕
	* - 生成規則：WDGM_E_IMPROPER_CALLERが設定されていない場合，NULL_PTR【PWDGM170】〔NWDGM004〕
	*/
	const uint16	*WDGM_E_IMPROPER_CALLER;
	/**
	* @brief グローバル監視状態がWDGM_GLOBAL_STATUS_STOPPEDに遷移した場合のエラーID
	* - 生成規則：WDGM_E_SUPERVISION【PWDGM171】〔DWDGM085〕
	* - 生成規則：WDGM_E_SUPERVISIONが設定されていない場合，NULL_PTR【PWDGM172】〔NWDGM004〕
	*/
	const uint16	*WDGM_E_SUPERVISION;
	/**
	* @brief Watchdog Driver のモード切替えが失敗した場合のエラーID
	* - 生成規則：WDGM_E_SET_MODE【PWDGM173】〔NWDGM042〕
	* - 生成規則：WDGM_E_SET_MODEが設定されていない場合，NULL_PTR【PWDGM174】〔NWDGM004〕
	*/
	const uint16	*WDGM_E_SET_MODE;
	/**
	* @brief MF周期/OSカウンタ監視が異常の場合のエラーID
	* - 生成規則：WDGM_E_MFCYCLE_OR_OSCOUNTER【PWDGM175】〔NWDGM099〕
	* - 生成規則：WDGM_E_MFCYCLE_OR_OSCOUNTERが設定されていない場合，NULL_PTR【PWDGM176】〔NWDGM004〕
	*/
	const uint16	*WDGM_E_MFCYCLE_OR_OSCOUNTER;
	/**
	* @brief OSカウンタの返り値が異常の場合のエラーID
	* - 生成規則：WDGM_E_OSCOUNTER【PWDGM177】〔NWDGM093〕
	* - 生成規則：WDGM_E_OSCOUNTERが設定されていない場合，NULL_PTR【PWDGM178】〔NWDGM004〕
	*/
	const uint16	*WDGM_E_OSCOUNTER;
} WdgM_DemInitBlock;

/**
* @brief コンフィギュレーション初期化ブロック
* - 対応コンテナ：WdgMConfigSet
* - 備考：ポストビルドタイムで構成されるパラメータは本構造体から参照される位置に配置する【PWDGM179】〔WDGM029〕 〔WDGM042〕
*  - WdgMInitialMode：initial_mode_id
*  - WdgMDemEventParameterRefs：p_dem_inib
*  - WdgMMode：p_mode_inib_list
*/
typedef struct wdgm_config_type {
	/**
	* @brief 初期モードID
	* - 生成規則：WdgMInitialModeが参照するWdgMModeのWdgMModeId【PWDGM180】〔WDGM179〕
	*/
	WdgM_ModeType				initial_mode_id;
	/**
	* @brief モード数
	* - 生成規則：p_mode_inib_listの要素数【PWDGM181】〔PWDGM182〕
	*/
	uint8						tnum_mode;
	/**
	* @brief モードリスト
	* - 生成規則：同一のWdgMConfigSetに所属するWdgMModeの情報を持つWdgM_ModeInitBlockの配列を作成し，そのポインタを配置する【PWDGM182】〔WDGM154〕
	*/
	const WdgM_ModeInitBlock	*p_mode_inib_list;
	/**
	* @brief DEM初期化ブロックへの参照
	* - 生成規則：WdgM_DemInitBlock型の初期化ブロックを作成し，そのポインタを配置する【PWDGM183】〔NWDGM004〕
	* - 生成規則：WdgMDemEventParameterRefsが設定されていない場合，NULL_PTR【PWDGM184】〔NWDGM004〕
	*/
	const WdgM_DemInitBlock		*p_dem_inib;
	/**
	* @brief アライブ監視の共有バッファのサイズ
	* - 生成規則：p_buffer_WdgM_AliveControlBlockの要素数【PWDGM185】〔PWDGM189〕
	*/
	uint16						max_size_WdgM_AliveControlBlock;
	/**
	* @brief デッドライン監視の共有バッファのサイズ
	* - 生成規則：p_buffer_WdgM_DeadlineControlBlockの要素数【PWDGM186】〔PWDGM191〕
	*/
	uint16						max_size_WdgM_DeadlineControlBlock;
	/**
	* @brief 外部ロジカル監視の共有バッファのサイズ
	* - 生成規則：p_buffer_WdgM_ExternalLogicalControlBlockの要素数【PWDGM187】〔PWDGM193〕
	*/
	uint16						max_size_WdgM_ExternalLogicalControlBlock;
	/**
	* @brief MF周期/OSカウンタ監視の共有バッファのサイズ
	* - 生成規則：p_buffer_TickTypeの要素数【PWDGM188】〔PWDGM195〕
	*/
	uint16						max_size_TickType;
	/**
	* @brief アライブ監視の共有バッファへのポインタ
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeのうち，WdgMAliveSupervisionの最大数分WdgM_AliveControlBlock型の配列を作成し，そのポインタを配置する【PWDGM189】〔PWDGM021〕
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeでWdgMAliveSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM190】〔PWDGM021〕
	*/
	WdgM_AliveControlBlock		*p_buffer_WdgM_AliveControlBlock;
	/**
	* @brief デッドライン監視の共有バッファへのポインタ
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeのうち，WdgMDeadlineSupervisionの最大数分WdgM_DeadlineControlBlock型の配列を作成し，そのポインタを配置する【PWDGM191】〔PWDGM033〕
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeでWdgMDeadlineSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM192】〔PWDGM033〕
	*/
	WdgM_DeadlineControlBlock	*p_buffer_WdgM_DeadlineControlBlock;
	/**
	* @brief 外部ロジカル監視の共有バッファへのポインタ
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeのうち，WdgMExternalLogicalSupervisionの最大数分WdgM_ExternalLogicalControlBlock型の配列を作成し，そのポインタを配置する【PWDGM193】〔PWDGM046〕
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeでWdgMExternalLogicalSupervisionが1つも設定されていない場合，NULL_PTR【PWDGM194】〔PWDGM046〕
	*/
	WdgM_ExternalLogicalControlBlock *p_buffer_WdgM_ExternalLogicalControlBlock;
	/**
	* @brief  MF周期/OSカウンタ監視の共有バッファへのポインタ
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeのWdgMMFCycleAndOsCounterSupervisionのうち，WdgMOsCounterRefの最大数分TickType型の配列を作成し，そのポインタを配置する【PWDGM195】〔PWDGM125〕
	* - 生成規則：全てのWdgMConfigSetの全てのWdgMModeでWdgMMFCycleAndOsCounterSupervisionが設定されていない場合，NULL_PTR【PWDGM196】〔PWDGM125〕
	*/
	TickType					*p_buffer_TickType;
} WdgM_ConfigType;

/**
* @defgroup extern_api WdgMシステムサービスプロトタイプ宣言
* @brief WdgMシステムサービスプロトタイプ宣言
* - WdgMシステムサービスプロトタイプ宣言【PWDGM208】〔DWDGM003〕
*/
/*@{*/
extern void WdgM_Init(const WdgM_ConfigType *ConfigPtr);
extern void WdgM_DeInit(void);
extern void WdgM_GetVersionInfo(Std_VersionInfoType *VersionInfo);
extern Std_ReturnType WdgM_SetMode(WdgM_ModeType Mode, uint16 CallerID);
extern Std_ReturnType WdgM_GetMode(WdgM_ModeType *Mode);
extern Std_ReturnType WdgM_GetLocalStatus(WdgM_SupervisedEntityIdType SEID, WdgM_LocalStatusType *Status);
extern Std_ReturnType WdgM_GetGlobalStatus(WdgM_GlobalStatusType *Status);
extern void WdgM_PerformReset(void);
extern Std_ReturnType WdgM_GetFirstExpiredSEID(WdgM_SupervisedEntityIdType *SEID);
extern void WdgM_MainFunction(void);
/*@}*/

/*
 *  以下，未サポート機能
 */

/* <WDGM155> */
extern Std_ReturnType WdgM_UpdateAliveCounter(WdgM_SupervisedEntityIdType SEID);

#endif /* WDGM_H */
