/*
 *  TOPPERS/A-WDGM
 *      Automotive Watchdog Manager
 *
 *  Copyright (C) 2014-2017 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2015-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *  Copyright (C) 2014-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2017 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgM_NonInitialized_RAM.c 3466 2017-03-07 06:25:11Z suzuki-kawaguchi $
 */

/**
* @file WdgM_NonInitialized_RAM.c
* @brief NonInitialized-RAMの定義＆操作用 実装ファイル
* @attention 本ファイルで定義されるstatic領域のRAMはシステムの初期化ルーチンで初期化されない領域に配置すること
*/
#include "WdgM_NonInitialized_RAM.h"
#include "MemMap.h"

/** @brief SEIDの真値と反転値の格納領域【IWDGM229】〔DWDGM010〕〔DWDGM011〕 */
static uint32 double_first_expired_seid;

#define SHIFT (sizeof(WdgM_SupervisedEntityIdType) * 8U)
#define MASK (((uint32)1 << SHIFT) - 1)

/**
* @brief グローバルな内部関数：SEIDの初期化【IWDGM230】〔DWDGM007〕
* @return なし
*/
void
WdgM_ClearFirstExpiredSEID(void)
{
	/**
	* - double_first_expired_seidを0にクリアする【IWDGM231】〔WDGM350〕
	*  - double_first_expired_seidを0にすると真値と反転値が一致しなくなるので，
	*    WdgM_ClearFirstExpiredSEID後，一回目のWdgM_SetFirstExpiredSEIDで
	*    double_first_expired_seidが更新されるようになる
	*/
	double_first_expired_seid = 0U;
}

/**
* @brief グローバルな内部関数：SEIDの書き込み【IWDGM232】〔DWDGM008〕
* @param SEID 書き込むSEID
* @return なし
*/
void
WdgM_SetFirstExpiredSEID(WdgM_SupervisedEntityIdType SEID)
{
	uint32 tmp;
	WdgM_SupervisedEntityIdType first_expired_seid;
	WdgM_SupervisedEntityIdType inv_first_expired_seid;

	/** - グローバル変数をローカル変数に退避してからSEIDを真値と反転値に分離する【IWDGM233】〔DWDGM012〕 */
	tmp = double_first_expired_seid;
	first_expired_seid = tmp & MASK;
	inv_first_expired_seid = tmp >> SHIFT;

	/**
	* - first_expired_seidとinv_first_expired_seidの反転値が一致していない場合，引数SEIDの真値と反転値を保持する【IWDGM234】〔WDGM351〕 
	*  - 本条件が，最初の書き込み時であること意味する
	*/
	if (first_expired_seid != (WdgM_SupervisedEntityIdType)~inv_first_expired_seid) {
		/* - ローカル変数で全ての処理を終えてからグローバル変数を更新する【IWDGM235】〔DWDGM012〕 */
		tmp = (uint32)SEID | (uint32)((~SEID) << SHIFT);
		double_first_expired_seid = tmp;
	}
}

/**
* @brief グローバルな内部関数：SEIDの読み込み【IWDGM236】〔DWDGM009〕
* @param SEID 読み込んだSEIDを格納する領域
* @return 読み込み結果
*/
Std_ReturnType
WdgM_GetFirstExpiredSEIDCore(WdgM_SupervisedEntityIdType *SEID)
{
	uint32 tmp;
	WdgM_SupervisedEntityIdType first_expired_seid;
	WdgM_SupervisedEntityIdType inv_first_expired_seid;
	Std_ReturnType ercd;

	/** - グローバル変数をローカル変数に退避してからSEIDを真値と反転値に分離する【IWDGM237】〔DWDGM012〕 */
	tmp = double_first_expired_seid;
	first_expired_seid = tmp & MASK;
	inv_first_expired_seid = tmp >> SHIFT;

	/**
	* - first_expired_seidとinv_first_expired_seidの反転値が一致していれば，first_expired_seidを引数SEIDに格納し，
	*   一致していなければSEIDに0を格納して返り値をE_NOT_OKとする【IWDGM238】〔WDGM349〕
	*/
	if (first_expired_seid == (WdgM_SupervisedEntityIdType)~inv_first_expired_seid) {
		*SEID = first_expired_seid;
		ercd = E_OK;
	}
	else {
		*SEID = 0;
		ercd = E_NOT_OK;
	}

	return(ercd);
}

/* ExpiredSEID試験用 */
#ifdef TWDGM_TEST_MODE
void
WdgM_SetFirstExpiredSEID_Error(WdgM_SupervisedEntityIdType SEID)
{
	uint32 tmp;

	tmp = (uint32)SEID | (uint32)((SEID) << SHIFT);
	double_first_expired_seid = tmp;
}
#endif
